@@ -1,38 +1,42 @@
-/*****************************************************************************
- * 
- * Sample rate transposer. Changes sample rate by using linear interpolation 
- * together with anti-alias filtering (first order interpolation with anti-
- * alias filtering should be quite adequate for this application)
- *
- * Author        : Copyright (c) Olli Parviainen
- * Author e-mail : oparviai @ iki.fi
- * File created  : 13-Jan-2002
- * 
- * Last changed  : $Date: 2004/10/26 19:09:37 $
- * File revision : $Revision: 1.3 $
- *
- * $Id: RateTransposer.cpp,v 1.3 2004/10/26 19:09:37 vjohnson Exp $
- *
- * License :
- * 
- *  SoundTouch sound processing library
- *  Copyright (c) Olli Parviainen
- *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation; either
- *  version 2.1 of the License, or (at your option) any later version.
- *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
- *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- *****************************************************************************/
+////////////////////////////////////////////////////////////////////////////////
+///
+/// Sample rate transposer. Changes sample rate by using linear interpolation
+/// together with anti-alias filtering (first order interpolation with anti-
+/// alias filtering should be quite adequate for this application)
+///
+/// Author        : Copyright (c) Olli Parviainen
+/// Author e-mail : oparviai 'at' iki.fi
+/// SoundTouch WWW: http://www.surina.net/soundtouch
+///
+////////////////////////////////////////////////////////////////////////////////
+//
+// Last changed  : $Date: 2006/09/18 22:29:22 $
+// File revision : $Revision: 1.5 $
+//
+// $Id: RateTransposer.cpp,v 1.5 2006/09/18 22:29:22 martynshaw Exp $
+//
+////////////////////////////////////////////////////////////////////////////////
+//
+// License :
+//
+//  SoundTouch audio processing library
+//  Copyright (c) Olli Parviainen
+//
+//  This library is free software; you can redistribute it and/or
+//  modify it under the terms of the GNU Lesser General Public
+//  License as published by the Free Software Foundation; either
+//  version 2.1 of the License, or (at your option) any later version.
+//
+//  This library is distributed in the hope that it will be useful,
+//  but WITHOUT ANY WARRANTY; without even the implied warranty of
+//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+//  Lesser General Public License for more details.
+//
+//  You should have received a copy of the GNU Lesser General Public
+//  License along with this library; if not, write to the Free Software
+//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+//
+////////////////////////////////////////////////////////////////////////////////
 
 #include <memory.h>
 #include <assert.h>
@@ -45,7 +49,6 @@
 using namespace soundtouch;
 
 
-
 /// A linear samplerate transposer class that uses integer arithmetics.
 /// for the transposing.
 class RateTransposerInteger : public RateTransposer
@@ -53,22 +56,22 @@ class RateTransposerInteger : public RateTransposer
 protected:
     int iSlopeCount;
     uint uRate;
-    soundtouch::SAMPLETYPE sPrevSampleL, sPrevSampleR;
+    SAMPLETYPE sPrevSampleL, sPrevSampleR;
 
     virtual void resetRegisters();
 
-    virtual uint transposeStereo(soundtouch::SAMPLETYPE *dest, 
-                         const soundtouch::SAMPLETYPE *src, 
+    virtual uint transposeStereo(SAMPLETYPE *dest,
+                         const SAMPLETYPE *src,
                          uint numSamples);
-    virtual uint transposeMono(soundtouch::SAMPLETYPE *dest, 
-                       const soundtouch::SAMPLETYPE *src, 
+    virtual uint transposeMono(SAMPLETYPE *dest,
+                       const SAMPLETYPE *src,
                        uint numSamples);
 
 public:
     RateTransposerInteger();
     virtual ~RateTransposerInteger();
 
-    /// Sets new target rate. Normal rate = 1.0, smaller values represent slower 
+    /// Sets new target rate. Normal rate = 1.0, smaller values represent slower
     /// rate, larger faster rates.
     virtual void setRate(float newRate);
 
@@ -82,15 +85,15 @@ class RateTransposerFloat : public RateTransposer
 protected:
     float fSlopeCount;
     float fRateStep;
-    soundtouch::SAMPLETYPE sPrevSampleL, sPrevSampleR;
+    SAMPLETYPE sPrevSampleL, sPrevSampleR;
 
     virtual void resetRegisters();
 
-    virtual uint transposeStereo(soundtouch::SAMPLETYPE *dest, 
-                         const soundtouch::SAMPLETYPE *src, 
+    virtual uint transposeStereo(SAMPLETYPE *dest,
+                         const SAMPLETYPE *src,
                          uint numSamples);
-    virtual uint transposeMono(soundtouch::SAMPLETYPE *dest, 
-                       const soundtouch::SAMPLETYPE *src, 
+    virtual uint transposeMono(SAMPLETYPE *dest,
+                       const SAMPLETYPE *src,
                        uint numSamples);
 
 public:
@@ -106,12 +109,12 @@ public:
 #endif
 
 
-// Operator 'new' is overloaded so that it automatically creates a suitable instance 
+// Operator 'new' is overloaded so that it automatically creates a suitable instance
 // depending on if we've a MMX/SSE/etc-capable CPU available or not.
 void * RateTransposer::operator new(size_t s)
 {
     // Notice! don't use "new TDStretch" directly, use "newInstance" to create a new instance instead!
-    assert(FALSE);  
+    assert(FALSE);
     return NULL;
 }
 
@@ -167,7 +170,7 @@ AAFilter *RateTransposer::getAAFilter() const
 
 
 
-// Sets new target uRate. Normal uRate = 1.0, smaller values represent slower 
+// Sets new target uRate. Normal uRate = 1.0, smaller values represent slower
 // uRate, larger faster uRates.
 void RateTransposer::setRate(float newRate)
 {
@@ -176,11 +179,11 @@ void RateTransposer::setRate(float newRate)
     fRate = newRate;
 
     // design a new anti-alias filter
-    if (newRate > 1.0f) 
+    if (newRate > 1.0f)
     {
         fCutoff = 0.5f / newRate;
-    } 
-    else 
+    }
+    else
     {
         fCutoff = 0.5f * newRate;
     }
@@ -220,7 +223,7 @@ void RateTransposer::upsample(const SAMPLETYPE *src, uint numSamples)
     // If the parameter 'uRate' value is smaller than 'SCALE', first transpose
     // the samples and then apply the anti-alias filter to remove aliasing.
 
-    // First check that there's enough room in 'storeBuffer' 
+    // First check that there's enough room in 'storeBuffer'
     // (+16 is to reserve some slack in the destination buffer)
     sizeTemp = (int)((float)numSamples / fRate + 16.0f);
 
@@ -231,7 +234,7 @@ void RateTransposer::upsample(const SAMPLETYPE *src, uint numSamples)
     // Apply the anti-alias filter to samples in "store output", output the
     // result to "dest"
     num = storeBuffer.numSamples();
-    count = pAAFilter->evaluate(outputBuffer.ptrEnd(num), 
+    count = pAAFilter->evaluate(outputBuffer.ptrEnd(num),
         storeBuffer.ptrBegin(), num, uChannels);
     outputBuffer.putSamples(count);
 
@@ -253,13 +256,13 @@ void RateTransposer::downsample(const SAMPLETYPE *src, uint numSamples)
     // Add the new samples to the end of the storeBuffer */
     storeBuffer.putSamples(src, numSamples);
 
-    // Anti-alias filter the samples to prevent folding and output the filtered 
+    // Anti-alias filter the samples to prevent folding and output the filtered
     // data to tempBuffer. Note : because of the FIR filter length, the
     // filtering routine takes in 'filter_length' more samples than it outputs.
     assert(tempBuffer.isEmpty());
     sizeTemp = storeBuffer.numSamples();
 
-    count = pAAFilter->evaluate(tempBuffer.ptrEnd(sizeTemp), 
+    count = pAAFilter->evaluate(tempBuffer.ptrEnd(sizeTemp),
         storeBuffer.ptrBegin(), sizeTemp, uChannels);
 
     // Remove the filtered samples from 'storeBuffer'
@@ -272,7 +275,7 @@ void RateTransposer::downsample(const SAMPLETYPE *src, uint numSamples)
 }
 
 
-// Transposes sample rate by applying anti-alias filter to prevent folding. 
+// Transposes sample rate by applying anti-alias filter to prevent folding.
 // Returns amount of samples returned in the "dest" buffer.
 // The maximum amount of samples that can be returned at a time is set by
 // the 'set_returnBuffer_size' function.
@@ -286,7 +289,7 @@ void RateTransposer::processSamples(const SAMPLETYPE *src, uint numSamples)
 
     // If anti-alias filter is turned off, simply transpose without applying
     // the filter
-    if (bUseAAFilter == FALSE) 
+    if (bUseAAFilter == FALSE)
     {
         sizeReq = (int)((float)numSamples / fRate + 1.0f);
         count = transpose(outputBuffer.ptrEnd(sizeReq), src, numSamples);
@@ -295,26 +298,26 @@ void RateTransposer::processSamples(const SAMPLETYPE *src, uint numSamples)
     }
 
     // Transpose with anti-alias filter
-    if (fRate < 1.0f) 
+    if (fRate < 1.0f)
     {
         upsample(src, numSamples);
-    } 
-    else  
+    }
+    else
     {
         downsample(src, numSamples);
     }
 }
 
 
-// Transposes the sample rate of the given samples using linear interpolation. 
+// Transposes the sample rate of the given samples using linear interpolation.
 // Returns the number of samples returned in the "dest" buffer
 inline uint RateTransposer::transpose(SAMPLETYPE *dest, const SAMPLETYPE *src, uint numSamples)
 {
-    if (uChannels == 2) 
+    if (uChannels == 2)
     {
         return transposeStereo(dest, src, numSamples);
-    } 
-    else 
+    }
+    else
     {
         return transposeMono(dest, src, numSamples);
     }
@@ -360,7 +363,7 @@ uint RateTransposer::isEmpty()
 //////////////////////////////////////////////////////////////////////////////
 //
 // RateTransposerInteger - integer arithmetic implementation
-// 
+//
 
 /// fixed-point interpolation routine precision
 #define SCALE    65536
@@ -384,25 +387,25 @@ RateTransposerInteger::~RateTransposerInteger()
 void RateTransposerInteger::resetRegisters()
 {
     iSlopeCount = 0;
-    sPrevSampleL = 
+    sPrevSampleL =
     sPrevSampleR = 0;
 }
 
 
 
-// Transposes the sample rate of the given samples using linear interpolation. 
-// 'Mono' version of the routine. Returns the number of samples returned in 
+// Transposes the sample rate of the given samples using linear interpolation.
+// 'Mono' version of the routine. Returns the number of samples returned in
 // the "dest" buffer
 uint RateTransposerInteger::transposeMono(SAMPLETYPE *dest, const SAMPLETYPE *src, uint numSamples)
 {
     unsigned int i, used;
     LONG_SAMPLETYPE temp, vol1;
 
-    used = 0;    
+    used = 0;
     i = 0;
 
     // Process the last sample saved from the previous call first...
-    while (iSlopeCount <= SCALE) 
+    while (iSlopeCount <= SCALE)
     {
         vol1 = (LONG_SAMPLETYPE)(SCALE - iSlopeCount);
         temp = vol1 * sPrevSampleL + iSlopeCount * src[0];
@@ -415,7 +418,7 @@ uint RateTransposerInteger::transposeMono(SAMPLETYPE *dest, const SAMPLETYPE *sr
 
     while (1)
     {
-        while (iSlopeCount > SCALE) 
+        while (iSlopeCount > SCALE)
         {
             iSlopeCount -= SCALE;
             used ++;
@@ -436,8 +439,8 @@ end:
 }
 
 
-// Transposes the sample rate of the given samples using linear interpolation. 
-// 'Mono' version of the routine. Returns the number of samples returned in 
+// Transposes the sample rate of the given samples using linear interpolation.
+// 'Stereo' version of the routine. Returns the number of samples returned in
 // the "dest" buffer
 uint RateTransposerInteger::transposeStereo(SAMPLETYPE *dest, const SAMPLETYPE *src, uint numSamples)
 {
@@ -446,11 +449,11 @@ uint RateTransposerInteger::transposeStereo(SAMPLETYPE *dest, const SAMPLETYPE *
 
     if (numSamples == 0) return 0;  // no samples, no work
 
-    used = 0;    
+    used = 0;
     i = 0;
 
     // Process the last sample saved from the sPrevSampleLious call first...
-    while (iSlopeCount <= SCALE) 
+    while (iSlopeCount <= SCALE)
     {
         vol1 = (LONG_SAMPLETYPE)(SCALE - iSlopeCount);
         temp = vol1 * sPrevSampleL + iSlopeCount * src[0];
@@ -465,7 +468,7 @@ uint RateTransposerInteger::transposeStereo(SAMPLETYPE *dest, const SAMPLETYPE *
 
     while (1)
     {
-        while (iSlopeCount > SCALE) 
+        while (iSlopeCount > SCALE)
         {
             iSlopeCount -= SCALE;
             used ++;
@@ -490,7 +493,7 @@ end:
 }
 
 
-// Sets new target uRate. Normal uRate = 1.0, smaller values represent slower 
+// Sets new target uRate. Normal uRate = 1.0, smaller values represent slower
 // uRate, larger faster uRates.
 void RateTransposerInteger::setRate(float newRate)
 {
@@ -502,7 +505,7 @@ void RateTransposerInteger::setRate(float newRate)
 //////////////////////////////////////////////////////////////////////////////
 //
 // RateTransposerFloat - floating point arithmetic implementation
-// 
+//
 //////////////////////////////////////////////////////////////////////////////
 
 // Constructor
@@ -524,24 +527,24 @@ RateTransposerFloat::~RateTransposerFloat()
 void RateTransposerFloat::resetRegisters()
 {
     fSlopeCount = 0;
-    sPrevSampleL = 
+    sPrevSampleL =
     sPrevSampleR = 0;
 }
 
 
 
-// Transposes the sample rate of the given samples using linear interpolation. 
-// 'Mono' version of the routine. Returns the number of samples returned in 
+// Transposes the sample rate of the given samples using linear interpolation.
+// 'Mono' version of the routine. Returns the number of samples returned in
 // the "dest" buffer
 uint RateTransposerFloat::transposeMono(SAMPLETYPE *dest, const SAMPLETYPE *src, uint numSamples)
 {
     unsigned int i, used;
 
-    used = 0;    
+    used = 0;
     i = 0;
 
     // Process the last sample saved from the previous call first...
-    while (fSlopeCount <= 1.0f) 
+    while (fSlopeCount <= 1.0f)
     {
         dest[i] = (SAMPLETYPE)((1.0f - fSlopeCount) * sPrevSampleL + fSlopeCount * src[0]);
         i++;
@@ -549,9 +552,11 @@ uint RateTransposerFloat::transposeMono(SAMPLETYPE *dest, const SAMPLETYPE *src,
     }
     fSlopeCount -= 1.0f;
 
+    if (numSamples == 1) goto end;
+
     while (1)
     {
-        while (fSlopeCount > 1.0f) 
+        while (fSlopeCount > 1.0f)
         {
             fSlopeCount -= 1.0f;
             used ++;
@@ -569,8 +574,8 @@ end:
 }
 
 
-// Transposes the sample rate of the given samples using linear interpolation. 
-// 'Mono' version of the routine. Returns the number of samples returned in 
+// Transposes the sample rate of the given samples using linear interpolation.
+// 'Mono' version of the routine. Returns the number of samples returned in
 // the "dest" buffer
 uint RateTransposerFloat::transposeStereo(SAMPLETYPE *dest, const SAMPLETYPE *src, uint numSamples)
 {
@@ -578,11 +583,11 @@ uint RateTransposerFloat::transposeStereo(SAMPLETYPE *dest, const SAMPLETYPE *sr
 
     if (numSamples == 0) return 0;  // no samples, no work
 
-    used = 0;    
+    used = 0;
     i = 0;
 
     // Process the last sample saved from the sPrevSampleLious call first...
-    while (fSlopeCount <= 1.0f) 
+    while (fSlopeCount <= 1.0f)
     {
         dest[2 * i] = (SAMPLETYPE)((1.0f - fSlopeCount) * sPrevSampleL + fSlopeCount * src[0]);
         dest[2 * i + 1] = (SAMPLETYPE)((1.0f - fSlopeCount) * sPrevSampleR + fSlopeCount * src[1]);
@@ -592,9 +597,11 @@ uint RateTransposerFloat::transposeStereo(SAMPLETYPE *dest, const SAMPLETYPE *sr
     // now always (iSlopeCount > 1.0f)
     fSlopeCount -= 1.0f;
 
+    if (numSamples == 1) goto end;
+
     while (1)
     {
-        while (fSlopeCount > 1.0f) 
+        while (fSlopeCount > 1.0f)
         {
             fSlopeCount -= 1.0f;
             used ++;
@@ -602,9 +609,9 @@ uint RateTransposerFloat::transposeStereo(SAMPLETYPE *dest, const SAMPLETYPE *sr
         }
         srcPos = 2 * used;
 
-        dest[2 * i] = (SAMPLETYPE)((1.0f - fSlopeCount) * src[srcPos] 
+        dest[2 * i] = (SAMPLETYPE)((1.0f - fSlopeCount) * src[srcPos]
             + fSlopeCount * src[srcPos + 2]);
-        dest[2 * i + 1] = (SAMPLETYPE)((1.0f - fSlopeCount) * src[srcPos + 1] 
+        dest[2 * i + 1] = (SAMPLETYPE)((1.0f - fSlopeCount) * src[srcPos + 1]
             + fSlopeCount * src[srcPos + 3]);
 
         i++;
