@@ -1,42 +1,45 @@
-/*****************************************************************************
- * 
- * Sampled sound tempo changer/time stretch algorithm. Changes the sound tempo 
- * while maintaining the original pitch by using a time domain WSOLA-like 
- * method with several performance-increasing tweaks.
- *
- * Note : MMX optimized functions reside in a separate, platform-specific 
- * file, e.g. 'mmx_win.cpp' or 'mmx_gcc.cpp'
- *
- * Author        : Copyright (c) Olli Parviainen 
- * Author e-mail : oparviai @ iki.fi
- * File created  : 13-Jan-2002
- *
- * Last changed  : $Date: 2004/10/26 19:09:37 $
- * File revision : $Revision: 1.4 $
- *
- * $Id: TDStretch.cpp,v 1.4 2004/10/26 19:09:37 vjohnson Exp $
- *
- * License :
- * 
- *  SoundTouch sound processing library
- *  Copyright (c) Olli Parviainen
- *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation; either
- *  version 2.1 of the License, or (at your option) any later version.
- *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
- *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- *****************************************************************************/
-
+////////////////////////////////////////////////////////////////////////////////
+///
+/// Sampled sound tempo changer/time stretch algorithm. Changes the sound tempo
+/// while maintaining the original pitch by using a time domain WSOLA-like
+/// method with several performance-increasing tweaks.
+///
+/// Note : MMX optimized functions reside in a separate, platform-specific
+/// file, e.g. 'mmx_win.cpp' or 'mmx_gcc.cpp'
+///
+/// Author        : Copyright (c) Olli Parviainen
+/// Author e-mail : oparviai 'at' iki.fi
+/// SoundTouch WWW: http://www.surina.net/soundtouch
+///
+////////////////////////////////////////////////////////////////////////////////
+//
+// Last changed  : $Date: 2006/09/18 22:29:22 $
+// File revision : $Revision: 1.6 $
+//
+// $Id: TDStretch.cpp,v 1.6 2006/09/18 22:29:22 martynshaw Exp $
+//
+////////////////////////////////////////////////////////////////////////////////
+//
+// License :
+//
+//  SoundTouch audio processing library
+//  Copyright (c) Olli Parviainen
+//
+//  This library is free software; you can redistribute it and/or
+//  modify it under the terms of the GNU Lesser General Public
+//  License as published by the Free Software Foundation; either
+//  version 2.1 of the License, or (at your option) any later version.
+//
+//  This library is distributed in the hope that it will be useful,
+//  but WITHOUT ANY WARRANTY; without even the implied warranty of
+//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+//  Lesser General Public License for more details.
+//
+//  You should have received a copy of the GNU Lesser General Public
+//  License along with this library; if not, write to the Free Software
+//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+//
+////////////////////////////////////////////////////////////////////////////////
 
 #include <string.h>
 #include <stdlib.h>
@@ -65,12 +68,10 @@ using namespace soundtouch;
  *****************************************************************************/
 
 
-#define MAX_SCAN_DELTA      124
-
 // Table for the hierarchical mixing position seeking algorithm
 int scanOffsets[4][24]={
-    { 124,  186,  248,  310,  372,  434,  496,  558,  620,  682,  744, 806, 
-      868,  930,  992, 1054, 1116, 1178, 1240, 1302, 1364, 1426, 1488,   0}, 
+    { 124,  186,  248,  310,  372,  434,  496,  558,  620,  682,  744, 806,
+      868,  930,  992, 1054, 1116, 1178, 1240, 1302, 1364, 1426, 1488,   0},
     {-100,  -75,  -50,  -25,   25,   50,   75,  100,    0,    0,    0,   0,
         0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,   0},
     { -20,  -15,  -10,   -5,    5,   10,   15,   20,    0,    0,    0,   0,
@@ -110,7 +111,7 @@ TDStretch::~TDStretch()
 }
 
 
-    
+
 // Calculates the x having the closest 2^x value for the given value
 static int _getClosest2Power(double value)
 {
@@ -124,11 +125,11 @@ static int _getClosest2Power(double value)
 //
 // 'sampleRate' = sample rate of the sound
 // 'sequenceMS' = one processing sequence length in milliseconds (default = 82 ms)
-// 'seekwindowMS' = seeking window length for scanning the best overlapping 
+// 'seekwindowMS' = seeking window length for scanning the best overlapping
 //      position (default = 28 ms)
 // 'overlapMS' = overlapping length (default = 12 ms)
 
-void TDStretch::setParameters(uint aSampleRate, uint aSequenceMS, 
+void TDStretch::setParameters(uint aSampleRate, uint aSequenceMS,
                               uint aSeekWindowMS, uint aOverlapMS)
 {
     this->sampleRate = aSampleRate;
@@ -182,7 +183,7 @@ void TDStretch::overlapMono(SAMPLETYPE *output, const SAMPLETYPE *input) const
 {
     int i, itemp;
 
-    for (i = 0; i < (int)overlapLength ; i ++) 
+    for (i = 0; i < (int)overlapLength ; i ++)
     {
         itemp = overlapLength - i;
         output[i] = (input[i] * i + pMidBuffer[i] * itemp ) / overlapLength;    // >> overlapDividerBits;
@@ -193,7 +194,7 @@ void TDStretch::overlapMono(SAMPLETYPE *output, const SAMPLETYPE *input) const
 
 void TDStretch::clearMidBuffer()
 {
-    if (bMidBufferDirty) 
+    if (bMidBufferDirty)
     {
         memset(pMidBuffer, 0, 2 * sizeof(SAMPLETYPE) * overlapLength);
         bMidBufferDirty = FALSE;
@@ -236,26 +237,26 @@ BOOL TDStretch::isQuickSeekEnabled() const
 // Seeks for the optimal overlap-mixing position.
 uint TDStretch::seekBestOverlapPosition(const SAMPLETYPE *refPos)
 {
-    if (channels == 2) 
+    if (channels == 2)
     {
         // stereo sound
-        if (bQuickseek) 
+        if (bQuickseek)
         {
             return seekBestOverlapPositionStereoQuick(refPos);
-        } 
-        else 
+        }
+        else
         {
             return seekBestOverlapPositionStereo(refPos);
         }
-    } 
-    else 
+    }
+    else
     {
         // mono sound
-        if (bQuickseek) 
+        if (bQuickseek)
         {
             return seekBestOverlapPositionMonoQuick(refPos);
-        } 
-        else 
+        }
+        else
         {
             return seekBestOverlapPositionMono(refPos);
         }
@@ -269,7 +270,7 @@ uint TDStretch::seekBestOverlapPosition(const SAMPLETYPE *refPos)
 // of 'ovlPos'.
 inline void TDStretch::overlap(SAMPLETYPE *output, const SAMPLETYPE *input, uint ovlPos) const
 {
-    if (channels == 2) 
+    if (channels == 2)
     {
         // stereo sound
         overlapStereo(output, input + 2 * ovlPos);
@@ -288,7 +289,7 @@ inline void TDStretch::overlap(SAMPLETYPE *output, const SAMPLETYPE *input, uint
 // The best position is determined as the position where the two overlapped
 // sample sequences are 'most alike', in terms of the highest cross-correlation
 // value over the overlapping period
-uint TDStretch::seekBestOverlapPositionStereo(const SAMPLETYPE *refPos) 
+uint TDStretch::seekBestOverlapPositionStereo(const SAMPLETYPE *refPos)
 {
     uint bestOffs;
     LONG_SAMPLETYPE bestCorr, corr;
@@ -302,14 +303,14 @@ uint TDStretch::seekBestOverlapPositionStereo(const SAMPLETYPE *refPos)
 
     // Scans for the best correlation value by testing each possible position
     // over the permitted range.
-    for (i = 0; i < seekLength; i ++) 
+    for (i = 0; i < seekLength; i ++)
     {
         // Calculates correlation value for the mixing position corresponding
         // to 'i'
         corr = calcCrossCorrStereo(refPos + 2 * i, pRefMidBuffer);
 
         // Checks for the highest correlation value
-        if (corr > bestCorr) 
+        if (corr > bestCorr)
         {
             bestCorr = corr;
             bestOffs = i;
@@ -328,7 +329,7 @@ uint TDStretch::seekBestOverlapPositionStereo(const SAMPLETYPE *refPos)
 // The best position is determined as the position where the two overlapped
 // sample sequences are 'most alike', in terms of the highest cross-correlation
 // value over the overlapping period
-uint TDStretch::seekBestOverlapPositionStereoQuick(const SAMPLETYPE *refPos) 
+uint TDStretch::seekBestOverlapPositionStereoQuick(const SAMPLETYPE *refPos)
 {
     uint j;
     uint bestOffs;
@@ -346,13 +347,13 @@ uint TDStretch::seekBestOverlapPositionStereoQuick(const SAMPLETYPE *refPos)
     // Scans for the best correlation value using four-pass hierarchical search.
     //
     // The look-up table 'scans' has hierarchical position adjusting steps.
-    // In first pass the routine searhes for the highest correlation with 
+    // In first pass the routine searhes for the highest correlation with
     // relatively coarse steps, then rescans the neighbourhood of the highest
     // correlation with better resolution and so on.
-    for (scanCount = 0;scanCount < 4; scanCount ++) 
+    for (scanCount = 0;scanCount < 4; scanCount ++)
     {
         j = 0;
-        while (scanOffsets[scanCount][j]) 
+        while (scanOffsets[scanCount][j])
         {
             tempOffset = corrOffset + scanOffsets[scanCount][j];
             if (tempOffset >= seekLength) break;
@@ -362,7 +363,7 @@ uint TDStretch::seekBestOverlapPositionStereoQuick(const SAMPLETYPE *refPos)
             corr = calcCrossCorrStereo(refPos + 2 * tempOffset, pRefMidBuffer);
 
             // Checks for the highest correlation value
-            if (corr > bestCorr) 
+            if (corr > bestCorr)
             {
                 bestCorr = corr;
                 bestOffs = tempOffset;
@@ -385,7 +386,7 @@ uint TDStretch::seekBestOverlapPositionStereoQuick(const SAMPLETYPE *refPos)
 // The best position is determined as the position where the two overlapped
 // sample sequences are 'most alike', in terms of the highest cross-correlation
 // value over the overlapping period
-uint TDStretch::seekBestOverlapPositionMono(const SAMPLETYPE *refPos) 
+uint TDStretch::seekBestOverlapPositionMono(const SAMPLETYPE *refPos)
 {
     uint bestOffs;
     LONG_SAMPLETYPE bestCorr, corr;
@@ -400,7 +401,7 @@ uint TDStretch::seekBestOverlapPositionMono(const SAMPLETYPE *refPos)
 
     // Scans for the best correlation value by testing each possible position
     // over the permitted range.
-    for (tempOffset = 0; tempOffset < seekLength; tempOffset ++) 
+    for (tempOffset = 0; tempOffset < seekLength; tempOffset ++)
     {
         compare = refPos + tempOffset;
 
@@ -409,7 +410,7 @@ uint TDStretch::seekBestOverlapPositionMono(const SAMPLETYPE *refPos)
         corr = calcCrossCorrMono(pRefMidBuffer, compare);
 
         // Checks for the highest correlation value
-        if (corr > bestCorr) 
+        if (corr > bestCorr)
         {
             bestCorr = corr;
             bestOffs = tempOffset;
@@ -428,7 +429,7 @@ uint TDStretch::seekBestOverlapPositionMono(const SAMPLETYPE *refPos)
 // The best position is determined as the position where the two overlapped
 // sample sequences are 'most alike', in terms of the highest cross-correlation
 // value over the overlapping period
-uint TDStretch::seekBestOverlapPositionMonoQuick(const SAMPLETYPE *refPos) 
+uint TDStretch::seekBestOverlapPositionMonoQuick(const SAMPLETYPE *refPos)
 {
     uint j;
     uint bestOffs;
@@ -446,13 +447,13 @@ uint TDStretch::seekBestOverlapPositionMonoQuick(const SAMPLETYPE *refPos)
     // Scans for the best correlation value using four-pass hierarchical search.
     //
     // The look-up table 'scans' has hierarchical position adjusting steps.
-    // In first pass the routine searhes for the highest correlation with 
+    // In first pass the routine searhes for the highest correlation with
     // relatively coarse steps, then rescans the neighbourhood of the highest
     // correlation with better resolution and so on.
-    for (scanCount = 0;scanCount < 4; scanCount ++) 
+    for (scanCount = 0;scanCount < 4; scanCount ++)
     {
         j = 0;
-        while (scanOffsets[scanCount][j]) 
+        while (scanOffsets[scanCount][j])
         {
             tempOffset = corrOffset + scanOffsets[scanCount][j];
             if (tempOffset >= seekLength) break;
@@ -462,7 +463,7 @@ uint TDStretch::seekBestOverlapPositionMonoQuick(const SAMPLETYPE *refPos)
             corr = calcCrossCorrMono(refPos + tempOffset, pRefMidBuffer);
 
             // Checks for the highest correlation value
-            if (corr > bestCorr) 
+            if (corr > bestCorr)
             {
                 bestCorr = corr;
                 bestOffs = tempOffset;
@@ -478,14 +479,14 @@ uint TDStretch::seekBestOverlapPositionMonoQuick(const SAMPLETYPE *refPos)
 }
 
 
-/// clear cross correlation routine state if necessary 
+/// clear cross correlation routine state if necessary
 void TDStretch::clearCrossCorrState()
 {
     // default implementation is empty.
 }
 
 
-// Sets new target tempo. Normal tempo = 'SCALE', smaller values represent slower 
+// Sets new target tempo. Normal tempo = 'SCALE', smaller values represent slower
 // tempo, larger faster tempo.
 void TDStretch::setTempo(float newTempo)
 {
@@ -493,12 +494,12 @@ void TDStretch::setTempo(float newTempo)
 
     tempo = newTempo;
 
-    // Calculate ideal skip length (according to tempo value) 
+    // Calculate ideal skip length (according to tempo value)
     nominalSkip = tempo * (seekWindowLength - overlapLength);
     skipFract = 0;
     intskip = (int)(nominalSkip + 0.5f);
 
-    // Calculate how many samples are needed in the 'inputBuffer' to 
+    // Calculate how many samples are needed in the 'inputBuffer' to
     // process another batch of samples
     sampleReq = max(intskip + overlapLength, seekWindowLength) + maxOffset;
 }
@@ -523,18 +524,18 @@ void TDStretch::processNominalTempo()
 {
     assert(tempo == 1.0f);
 
-    if (bMidBufferDirty) 
+    if (bMidBufferDirty)
     {
         // If there are samples in pMidBuffer waiting for overlapping,
-        // do a single sliding overlapping with them in order to prevent a 
+        // do a single sliding overlapping with them in order to prevent a
         // clicking distortion in the output sound
-        if (inputBuffer.numSamples() < overlapLength) 
+        if (inputBuffer.numSamples() < overlapLength)
         {
             // wait until we've got overlapLength input samples
             return;
         }
-        // Mix the samples in the beginning of 'inputBuffer' with the 
-        // samples in 'midBuffer' using sliding overlapping 
+        // Mix the samples in the beginning of 'inputBuffer' with the
+        // samples in 'midBuffer' using sliding overlapping
         overlap(outputBuffer.ptrEnd(overlapLength), inputBuffer.ptrBegin(), 0);
         outputBuffer.putSamples(overlapLength);
         inputBuffer.receiveSamples(overlapLength);
@@ -555,18 +556,21 @@ void TDStretch::processSamples()
     uint ovlSkip, offset;
     int temp;
 
-    if (tempo == 1.0f) 
+    /* Removed this small optimization - can introduce a click to sound when tempo setting
+       crosses the nominal value
+    if (tempo == 1.0f)
     {
         // tempo not changed from the original, so bypass the processing
         processNominalTempo();
         return;
     }
+    */
 
-    if (bMidBufferDirty == FALSE) 
+    if (bMidBufferDirty == FALSE)
     {
-        // if midBuffer is empty, move the first samples of the input stream 
+        // if midBuffer is empty, move the first samples of the input stream
         // into it
-        if (inputBuffer.numSamples() < overlapLength) 
+        if (inputBuffer.numSamples() < overlapLength)
         {
             // wait until we've got overlapLength samples
             return;
@@ -578,13 +582,13 @@ void TDStretch::processSamples()
 
     // Process samples as long as there are enough samples in 'inputBuffer'
     // to form a processing frame.
-    while (inputBuffer.numSamples() >= sampleReq) 
+    while (inputBuffer.numSamples() >= sampleReq)
     {
         // If tempo differs from the normal ('SCALE'), scan for the best overlapping
         // position
         offset = seekBestOverlapPosition(inputBuffer.ptrBegin());
 
-        // Mix the samples in the 'inputBuffer' at position of 'offset' with the 
+        // Mix the samples in the 'inputBuffer' at position of 'offset' with the
         // samples in 'midBuffer' using sliding overlapping
         // ... first partially overlap with the end of the previous sequence
         // (that's in 'midBuffer')
@@ -598,11 +602,11 @@ void TDStretch::processSamples()
             outputBuffer.putSamples(inputBuffer.ptrBegin() + channels * (offset + overlapLength), temp);
         }
 
-        // Copies the end of the current sequence from 'inputBuffer' to 
-        // 'midBuffer' for being mixed with the beginning of the next 
+        // Copies the end of the current sequence from 'inputBuffer' to
+        // 'midBuffer' for being mixed with the beginning of the next
         // processing sequence and so on
         assert(offset + seekWindowLength <= inputBuffer.numSamples());
-        memcpy(pMidBuffer, inputBuffer.ptrBegin() + channels * (offset + seekWindowLength - overlapLength), 
+        memcpy(pMidBuffer, inputBuffer.ptrBegin() + channels * (offset + seekWindowLength - overlapLength),
             channels * sizeof(SAMPLETYPE) * overlapLength);
         bMidBufferDirty = TRUE;
 
@@ -653,12 +657,12 @@ void TDStretch::acceptNewOverlapLength(uint newOverlapLength)
 }
 
 
-// Operator 'new' is overloaded so that it automatically creates a suitable instance 
+// Operator 'new' is overloaded so that it automatically creates a suitable instance
 // depending on if we've a MMX/SSE/etc-capable CPU available or not.
 void * TDStretch::operator new(size_t s)
 {
     // Notice! don't use "new TDStretch" directly, use "newInstance" to create a new instance instead!
-    assert(FALSE);  
+    assert(FALSE);
     return NULL;
 }
 
@@ -722,7 +726,7 @@ void TDStretch::precalcCorrReferenceStereo()
     int i, cnt2;
     int temp, temp2;
 
-    for (i=0 ; i < (int)overlapLength ;i ++) 
+    for (i=0 ; i < (int)overlapLength ;i ++)
     {
         temp = i * (overlapLength - i);
         cnt2 = i * 2;
@@ -743,7 +747,7 @@ void TDStretch::precalcCorrReferenceMono()
     long temp;
     long temp2;
 
-    for (i=0 ; i < (int)overlapLength ;i ++) 
+    for (i=0 ; i < (int)overlapLength ;i ++)
     {
         temp = i * (overlapLength - i);
         temp2 = (pMidBuffer[i] * temp) / slopingDivider;
@@ -752,7 +756,7 @@ void TDStretch::precalcCorrReferenceMono()
 }
 
 
-// Overlaps samples in 'midBuffer' with the samples in 'input'. The 'Stereo' 
+// Overlaps samples in 'midBuffer' with the samples in 'input'. The 'Stereo'
 // version of the routine.
 void TDStretch::overlapStereo(short *output, const short *input) const
 {
@@ -760,7 +764,7 @@ void TDStretch::overlapStereo(short *output, const short *input) const
     short temp;
     uint cnt2;
 
-    for (i = 0; i < (int)overlapLength ; i ++) 
+    for (i = 0; i < (int)overlapLength ; i ++)
     {
         temp = (short)(overlapLength - i);
         cnt2 = 2 * i;
@@ -784,8 +788,8 @@ void TDStretch::calculateOverlapLength(uint overlapMs)
 
     acceptNewOverlapLength(newOvl);
 
-    // calculate sloping divider so that crosscorrelation operation won't 
-    // overflow 32-bit register. Max. sum of the crosscorrelation sum without 
+    // calculate sloping divider so that crosscorrelation operation won't
+    // overflow 32-bit register. Max. sum of the crosscorrelation sum without
     // divider would be 2^30*(N^3-N)/3, where N = overlap length
     slopingDivider = (newOvl * newOvl - 1) / 3;
 }
@@ -797,7 +801,7 @@ long TDStretch::calcCrossCorrMono(const short *mixingPos, const short *compare)
     uint i;
 
     corr = 0;
-    for (i = 1; i < overlapLength; i ++) 
+    for (i = 1; i < overlapLength; i ++)
     {
         corr += (mixingPos[i] * compare[i]) >> overlapDividerBits;
     }
@@ -812,7 +816,7 @@ long TDStretch::calcCrossCorrStereo(const short *mixingPos, const short *compare
     uint i;
 
     corr = 0;
-    for (i = 2; i < 2 * overlapLength; i += 2) 
+    for (i = 2; i < 2 * overlapLength; i += 2)
     {
         corr += (mixingPos[i] * compare[i] +
                  mixingPos[i + 1] * compare[i + 1]) >> overlapDividerBits;
@@ -838,7 +842,7 @@ void TDStretch::precalcCorrReferenceStereo()
     int i, cnt2;
     float temp;
 
-    for (i=0 ; i < (int)overlapLength ;i ++) 
+    for (i=0 ; i < (int)overlapLength ;i ++)
     {
         temp = (float)i * (float)(overlapLength - i);
         cnt2 = i * 2;
@@ -855,7 +859,7 @@ void TDStretch::precalcCorrReferenceMono()
     int i;
     float temp;
 
-    for (i=0 ; i < (int)overlapLength ;i ++) 
+    for (i=0 ; i < (int)overlapLength ;i ++)
     {
         temp = (float)i * (float)(overlapLength - i);
         pRefMidBuffer[i] = (float)(pMidBuffer[i] * temp);
@@ -874,7 +878,7 @@ void TDStretch::overlapStereo(float *output, const float *input) const
 
     fScale = 1.0f / (float)overlapLength;
 
-    for (i = 0; i < (int)overlapLength ; i ++) 
+    for (i = 0; i < (int)overlapLength ; i ++)
     {
         fTemp = (float)(overlapLength - i) * fScale;
         fi = (float)i * fScale;
@@ -893,6 +897,9 @@ void TDStretch::calculateOverlapLength(uint overlapMs)
     newOvl = (sampleRate * overlapMs) / 1000;
     if (newOvl < 16) newOvl = 16;
 
+    // must be divisible by 8
+    newOvl -= newOvl % 8;
+
     acceptNewOverlapLength(newOvl);
 }
 
@@ -904,7 +911,7 @@ double TDStretch::calcCrossCorrMono(const float *mixingPos, const float *compare
     uint i;
 
     corr = 0;
-    for (i = 1; i < overlapLength; i ++) 
+    for (i = 1; i < overlapLength; i ++)
     {
         corr += mixingPos[i] * compare[i];
     }
@@ -919,7 +926,7 @@ double TDStretch::calcCrossCorrStereo(const float *mixingPos, const float *compa
     uint i;
 
     corr = 0;
-    for (i = 2; i < 2 * overlapLength; i += 2) 
+    for (i = 2; i < 2 * overlapLength; i += 2)
     {
         corr += mixingPos[i] * compare[i] +
                 mixingPos[i + 1] * compare[i + 1];
@@ -928,4 +935,4 @@ double TDStretch::calcCrossCorrStereo(const float *mixingPos, const float *compa
     return corr;
 }
 
-#endif // FLOAT_SAMPLES
\ No newline at end of file
+#endif // FLOAT_SAMPLES
