@@ -27,7 +27,6 @@ hold information about one contributor to Audacity.
 #include "Audacity.h"
 
 #include <wx/dialog.h>
-#include <wx/html/htmlwin.h>
 #include <wx/button.h>
 #include <wx/dcclient.h>
 #include <wx/sizer.h>
@@ -36,6 +35,9 @@ hold information about one contributor to Audacity.
 
 #include "AboutDialog.h"
 #include "FileNames.h"
+#include "Internat.h"
+#include "widgets/LinkingHtmlWindow.h"
+
 #include "portaudio.h"
 
 #include <wx/listimpl.cpp>
@@ -48,28 +50,41 @@ WX_DEFINE_LIST(AboutDialogCreditItemsList);
 
 void AboutDialog::CreateCreditsList()
 {
-   AddCredit(wxT("Dominic Mazzoni"), roleProjectLeader);
-   AddCredit(wxT("Matt Brubeck"), roleLeadDeveloper);
-   AddCredit(wxT("James Crook"), roleLeadDeveloper);
-   AddCredit(wxT("Vaughan Johnson"), roleLeadDeveloper);
-   AddCredit(wxT("Leland Lucius"), roleLeadDeveloper);
-   AddCredit(wxT("Markus Meyer"), roleLeadDeveloper);
+   // The Audacity 1.3.x and 1.4.x team: developers and support
+   // These will be split into two separate categories
+
+   AddCredit(wxT("Gale Andrews"), roleTeamSupport);
+   AddCredit(wxT("Richard Ash"), roleTeamSupport);
+   AddCredit(wxT("Christian Brochec"), roleTeamSupport);
+   AddCredit(wxT("James Crook"), roleTeamDeveloper);
+   AddCredit(wxT("Vaughan Johnson"), roleTeamDeveloper);
+   AddCredit(wxT("Leland Lucius"), roleTeamDeveloper);
+   AddCredit(wxT("Dominic Mazzoni"), roleTeamDeveloper);
+   AddCredit(wxT("Markus Meyer"), roleTeamDeveloper);
+   AddCredit(wxT("Alexandre Prokoudine"), roleTeamSupport);
+   AddCredit(wxT("Martyn Shaw"), roleTeamDeveloper);
+
+   // Emeritus: people who were "lead developers" or made an
+   // otherwise distinguished contribution, but who are no
+   // longer active.
+ 
+   AddCredit(wxT("Matt Brubeck"), roleEmeritusDeveloper);
+   AddCredit(wxT("Roger Dannenberg"), roleEmeritusDeveloper);
+   AddCredit(wxT("Joshua Haberman"), roleEmeritusDeveloper);
+   AddCredit(wxT("Monty Montgomery"), roleEmeritusDeveloper);
+   AddCredit(wxT("Shane Mueller"), roleEmeritusDeveloper);
+   AddCredit(wxT("Tony Oetzmann"), roleEmeritusSupport);
+
+   // All other contributors
    
-   AddCredit(wxT("Joshua Haberman"), roleDeveloper);
-   AddCredit(wxT("Monty Montgomery"), roleDeveloper);
-   AddCredit(wxT("Shane Mueller"), roleDeveloper);
-   AddCredit(wxT("Martyn Shaw"), roleDeveloper);
-   
-   AddCredit(wxT("Lynn Allan (CleanSpeech)"), roleContributor);
-   AddCredit(wxT("William Bland (Time Tracks)"), roleContributor);
-   AddCredit(wxT("Roger Dannenberg"), roleContributor);
+   AddCredit(wxT("Lynn Allan"), roleContributor);
+   AddCredit(wxT("William Bland"), roleContributor);
    AddCredit(wxT("Brian Gunlogson"), roleContributor);
    AddCredit(wxT("Arun Kishore"), roleContributor);
-   AddCredit(wxT("Harvey Lubin (Audacity logo)"), roleContributor);
+   AddCredit(wxT("Harvey Lubin"), roleContributor);
    AddCredit(wxT("Grek Mekkes"), roleContributor);
    AddCredit(wxT("Abe Milde"), roleContributor);
    AddCredit(wxT("Paul Nasca"), roleContributor);
-   AddCredit(wxT("Tony Oetzmann"), roleContributor);
    AddCredit(wxT("Augustus Saunders"), roleContributor);
    AddCredit(wxT("Mike Underwood"), roleContributor);
    AddCredit(wxT("Jun Wan"), roleContributor);
@@ -88,7 +103,6 @@ void AboutDialog::CreateCreditsList()
    AddCredit(wxT("SoundTouch"), roleLibrary);
    AddCredit(wxT("wxWidgets"), roleLibrary);
 
-   AddCredit(wxT("Richard Ash"), roleThanks);
    AddCredit(wxT("Dave Beydler"), roleThanks);
    AddCredit(wxT("Jason Cohen"), roleThanks);
    AddCredit(wxT("Dave Fancella"), roleThanks);
@@ -108,6 +122,7 @@ void AboutDialog::CreateCreditsList()
    AddCredit(wxT("David Topper"), roleThanks);
    AddCredit(wxT("Rudy Trubitt"), roleThanks);
    AddCredit(wxT("StreetIQ.com"), roleThanks);
+   AddCredit(wxT("UmixIt Technologies, LLC"), roleThanks);
    AddCredit(wxT("Verilogix, Inc."), roleThanks);
 }
 
@@ -121,63 +136,64 @@ END_EVENT_TABLE()
 IMPLEMENT_CLASS(AboutDialog, wxDialog)
 
 AboutDialog::AboutDialog(wxWindow * parent)
-   :  wxDialog(parent, -1, _("About Audacity..."),
+   :  wxDialog(parent, -1, _NoAcc("&About Audacity..."),
                wxDefaultPosition, wxDefaultSize)
 {
-   wxString versionStr = wxT(AUDACITY_VERSION_STRING);
+   wxString versionStr = AUDACITY_VERSION_STRING;
    wxString informationStr;
 
    creditItems.DeleteContents(true); // switchon automatic deletion of list items
    CreateCreditsList();
    
+   wxString enabled = wxString(wxT(": ")) + _("Enabled");
+   wxString disabled = wxString(wxT(": ")) + _("Disabled");
+
    #ifdef USE_LIBMAD
-   informationStr += _("MP3 importing enabled");
+   informationStr += wxT("Libmad") + enabled;
    #else
-   informationStr += _("MP3 importing disabled");
+   informationStr += wxT("Libmad") + disabled;
    #endif
    informationStr += wxT("<br>\n");
 
    #ifdef USE_LIBVORBIS
-   informationStr += _("Ogg Vorbis importing enabled");
+   informationStr += wxT("Ogg Vorbis") + enabled;
    #else
-   informationStr += _("Ogg Vorbis importing disabled");
+   informationStr += wxT("Ogg Vorbis") + disabled;
    #endif
    informationStr += wxT("<br>\n");
 
    #ifdef USE_LIBID3TAG
-   informationStr += _("ID3 tag exporting enabled");
+   informationStr += wxT("LibID3Tag") + enabled;
    #else
-   informationStr += _("ID3 tag exporting disabled");
+   informationStr += wxT("LibID3Tag") + disabled;
    #endif
    informationStr += wxT("<br>\n");
 
    # if USE_LIBFLAC
-   informationStr += _("FLAC plug-in support enabled");
+   informationStr += wxT("FLAC") + enabled;
    # else
-   informationStr += _("FLAC plug-in support disabled");
+   informationStr += wxT("FLAC") + disabled;
    # endif
    informationStr += wxT("<br>\n");
 
    # if USE_LADSPA
-   informationStr += _("LADSPA plug-in support enabled");
+   informationStr += wxT("LADSPA") + enabled;
    # else
-   informationStr += _("LADSPA plug-in support disabled");
+   informationStr += wxT("LADSPA") + disabled;
    # endif
    informationStr += wxT("<br>\n");
 
    #if USE_LIBRESAMPLE
-   informationStr += _("Libresample support enabled");
+   informationStr += wxT("Libresample") + enabled;
    #elif USE_LIBSAMPLERATE
-   informationStr += _("Libsamplerate support enabled");
-   #else
-   informationStr += _("No resampling support enabled");
+   informationStr += wxT("Libsamplerate") + enabled;
    #endif
    informationStr += wxT("<br>\n");
 
    #if USE_PORTAUDIO_V19
-   informationStr += _("PortAudio v19");
+   informationStr += wxT("PortAudio v19");
    #else
-   informationStr += _("PortAudio v18");
+   informationStr += wxT("PortAudio v18");
    #endif
    informationStr += wxT("<br>\n");
 
@@ -198,7 +214,7 @@ AboutDialog::AboutDialog(wxWindow * parent)
    wxString par1Str = _(
      "Audacity is a free program written by a team of volunteer developers around the world. Coordination happens thanks to SourceForge.net, an online service that provides free tools to open-source software projects. Audacity is available for Windows 98 and newer, Mac OS X, Linux, and other Unix-like operating systems. Older versions of Audacity are available for Mac OS 9.");
 
-   #if 1 // Not beta anymore
+   #if 1 // Is this beta or not?
    wxString par2Str = _(
      "This is a beta version of the program.  It may contain bugs and unfinished features.  We depend on your feedback, so please visit our website and give us your bug reports and feature requests." );
    #else
@@ -226,27 +242,50 @@ AboutDialog::AboutDialog(wxWindow * parent)
       wxT("<html><head><META http-equiv=\"Content-Type\" content=\"text/html; charset=") + 
          localeStr + 
          wxT("\"></head>") + 
-      wxT("<body bgcolor=\"#ffffff\"><font size=1><center>") + 
+      wxT("<body bgcolor=\"#ffffff\"><center>") + 
       wxT("<h3>Audacity &reg; ") + versionStr + wxT(" " ) + csetStr + wxT("</h3>")+ 
       _("A Free Digital Audio Editor") + 
       wxT("</center><p>") + par1Str +
       wxT("<p>") + par2Str +
-      wxT("<p>http://audacity.sourceforge.net/") + 
+      wxT("<p><center><a href=\"http://audacity.sourceforge.net/\">http://audacity.sourceforge.net/</a></center>") + 
       wxT("<p><center><b>") + _("Information") + wxT("</b></center>") + 
       wxT("<p><br>") + informationStr + 
       wxT("<p><center><b>") + _("Credits") + wxT("</b></center>")
       + translatorCredits +
-      wxT("<p><center><table border=0>") + 
-      GetCreditsByRole(roleProjectLeader) +
-      GetCreditsByRole(roleLeadDeveloper) +
-      wxT("</table></center>") +
-      wxT("<p><center><b>") + _("Developers:") + wxT("</b><p><br>") +
-      GetCreditsByRole(roleDeveloper) +
-      wxT("<br><p><center><b>") + _("Contributors:") + wxT("</b><p><br>") +
+      wxT("<p><center><b>") +
+
+      wxString::Format(_("Audacity %s Development Team"), versionStr.c_str()) +
+      wxT("</b><p><br>") +
+      GetCreditsByRole(roleTeamDeveloper) +
+      wxT("<p><br><b>") +
+
+      wxString::Format(_("Audacity %s Support Team"), versionStr.c_str()) +
+      wxT("</b><p><br>") +
+      GetCreditsByRole(roleTeamSupport) +
+      wxT("<p><br><b>") +
+
+      _("Emeritus Developers") +
+      wxT("</b><p><br>") +
+      GetCreditsByRole(roleEmeritusDeveloper) +
+      wxT("<p><br><b>") +
+
+      _("Other Emeritus Team Members") +
+      wxT("</b><p><br>") +
+      GetCreditsByRole(roleEmeritusSupport) +
+      wxT("<p><br><b>") +
+
+      _("Other Contributors") +
+      wxT("</b><p><br>") +
       GetCreditsByRole(roleContributor) +
-      wxT("<br><p><center><b>") + _("Audacity is based on code from the following projects:") + wxT("</b><p><br>") +
+      wxT("<p><br><b>") +
+
+      _("Audacity is based on code from the following projects:") +
+      wxT("</b><p><br>") +
       GetCreditsByRole(roleLibrary) +
-      wxT("<p><b>") + _("Special thanks:") + wxT("</b><p><br>") + 
+      wxT("<p><br><b>") +
+
+      _("Special thanks:") +
+      wxT("</b><p><br>") +
       GetCreditsByRole(roleThanks) +
       wxT("</center></font></body></html>");
    
@@ -260,13 +299,17 @@ AboutDialog::AboutDialog(wxWindow * parent)
                           wxSize(215, 190));
    pBoxSizer->Add(icon, 0, wxALIGN_CENTER | wxALL, 8);
 
-   wxHtmlWindow *html = new wxHtmlWindow(this, -1,
-                                         wxDefaultPosition,
-                                         wxSize(480, 240),
-                                         wxHW_SCROLLBAR_AUTO | wxSUNKEN_BORDER);
+   wxHtmlWindow *html = 
+      new LinkingHtmlWindow(this, -1,
+                             wxDefaultPosition,
+                             wxSize(480, 240),
+                             wxHW_SCROLLBAR_AUTO | wxSUNKEN_BORDER);
    html->SetPage(creditStr);
    pBoxSizer->Add(html, 0, wxALIGN_CENTER | wxALL, 8);
 
+   /* i18n-hint: Rather than a literal translation, consider adding the
+   appropriate suffix or prefix to create a word meaning 'something which
+   has Audacity' */
    wxButton *ok = new wxButton(this, wxID_OK, _("Audacious!"));
    ok->SetDefault();
    ok->SetFocus();
@@ -297,17 +340,8 @@ wxString AboutDialog::GetCreditsByRole(AboutDialog::Role role)
       AboutDialogCreditItem* item = p->GetData();
       if (item->role == role)
       {
-         if (item->role == roleProjectLeader || item->role == roleLeadDeveloper || item->role == roleDeveloper)
-            s += wxT("<tr><td>");
-            
          s += item->description;
-         
-         if (item->role == roleProjectLeader)
-            s += wxString(wxT("</td><td>")) + _("Project leader and primary programmer") + wxT("</td></tr>");
-         else if (item->role == roleLeadDeveloper)
-            s += wxString(wxT("</td><td>")) + _("Lead Developer") + wxT("</td></tr>");
-         else
-            s += wxT("<br>");
+         s += wxT("<br>");
       }
    }
    
