@@ -6,23 +6,35 @@
 
   Dominic Mazzoni
 
-**********************************************************************/
+******************************************************************//**
+
+\class AudacityApp
+\brief AudacityApp is the 'main' class for Audacity 
+
+It handles initialization and termination by subclassing wxApp.
+
+*//*******************************************************************/
+
 
 #include "Audacity.h" // This should always be included first
 
 #include <wx/defs.h>
 #include <wx/app.h>
 #include <wx/docview.h>
+#include <wx/event.h>
+#include <wx/ipc.h>
 #include <wx/log.h>
 #include <wx/window.h>
 #include <wx/intl.h>
 #include <wx/menu.h>
 #include <wx/msgdlg.h>
 #include <wx/snglinst.h>
+#include <wx/fontmap.h>
 
 #include <wx/fs_zip.h>
 #include <wx/image.h>
 
+#include <wx/dir.h>
 #include <wx/file.h>
 #include <wx/filename.h>
 
@@ -39,23 +51,121 @@
 #include "AudacityApp.h"
 
 #include "AboutDialog.h"
+#include "AColor.h"
 #include "AudioIO.h"
 #include "Benchmark.h"
 #include "DirManager.h"
 #include "effects/LoadEffects.h"
 #include "FreqWindow.h"
-#include "Help.h"
 #include "Internat.h"
 #include "LangChoice.h"
 #include "Prefs.h"
 #include "Project.h"
+#include "Screenshot.h"
 #include "Sequence.h"
 #include "WaveTrack.h"
 #include "Internat.h"
 #include "prefs/PrefsDialog.h"
+#include "Theme.h"
+#include "Experimental.h"
+#include "PlatformCompatibility.h"
+#include "FileNames.h"
+#include "AutoRecovery.h"
 
-class ToolBar;
-class ControlToolBar;
+#ifdef USE_QUICKTIME
+#include "import/ImportQT.h"
+#endif
+
+// Windows specific linker control...only needed once so
+// this is a good place (unless we want to add another file).
+#if defined(__WXMSW__)
+
+// These lines ensure that Audacity gets WindowsXP themes.
+// Without them we get the old-style Windows98/2000 look under XP.
+#  if !defined(__WXWINCE__)
+#     pragma comment(linker, "\"/manifestdependency:type='win32' name='Microsoft.Windows.Common-Controls' version='6.0.0.0' processorArchitecture='X86' publicKeyToken='6595b64144ccf1df'\"")
+#  endif
+
+// These lines allows conditional inclusion of the various libraries
+// that Audacity can use.
+#  if wxUSE_UNICODE
+#     define U "u"
+#  else
+#     define U ""
+#  endif
+
+#  if DEBUG
+#     define D "d"
+#  else
+#     define D ""
+#  endif
+
+#  if defined(USE_LIBFLAC)
+#     pragma comment(lib, "libFLAC++" U D)
+#     pragma comment(lib, "libFLAC" U D)
+#  endif
+
+#  if defined(USE_LIBID3TAG)
+#     pragma comment(lib, "libid3tag" U D)
+#  endif
+
+#  if defined(USE_LIBMAD)
+#     pragma comment(lib, "mad" U D)
+#  endif
+
+#  if defined(USE_LIBRESAMPLE)
+#     pragma comment(lib, "libresample" U D)
+#  endif
+
+#  if defined(USE_LIBSAMPLERATE)
+#     pragma comment(lib, "libsamplerate" U D)
+#  endif
+
+#  if defined(USE_LIBVORBIS)
+#     pragma comment(lib, "ogg_static" U D)
+#     pragma comment(lib, "vorbis_static" U D)
+#     pragma comment(lib, "vorbisfile_static" U D)
+#  endif
+
+#  if defined(USE_NYQUIST)
+#     pragma comment(lib, "libnyquist" U D)
+#  endif
+
+#  if defined(USE_SOUNDTOUCH)
+#     pragma comment(lib, "soundtouch" U D)
+#  endif
+
+#  if defined(USE_LIBTWOLAME)
+#     pragma comment(lib, "libtwolame_static" U D)
+#  endif
+
+#  undef U
+#  undef D
+
+#endif 
+
+#if wxUSE_ACCESSIBILITY
+#if wxCHECK_VERSION(2, 8, 0)
+const wxChar overrideTextCtrlNameStr[] = wxT("");
+const wxChar overrideChoiceNameStr[] = wxT("");
+const wxChar overrideComboBoxNameStr[] = wxT("");
+const wxChar overrideSliderNameStr[] = wxT("");
+#else
+const wxChar *overrideTextCtrlNameStr = wxT("");
+const wxChar *overrideChoiceNameStr = wxT("");
+const wxChar *overrideComboBoxNameStr = wxT("");
+const wxChar *overrideSliderNameStr = wxT("");
+#endif
+#endif
+
+////////////////////////////////////////////////////////////
+/// Custom events
+////////////////////////////////////////////////////////////
+
+DEFINE_EVENT_TYPE(EVT_OPEN_AUDIO_FILE);
+DEFINE_EVENT_TYPE(EVT_CAPTURE_KEYBOARD);
+DEFINE_EVENT_TYPE(EVT_RELEASE_KEYBOARD);
+DEFINE_EVENT_TYPE(EVT_CAPTURE_KEY);
 
 #ifdef __WXGTK__
 void wxOnAssert(const wxChar *fileName, int lineNumber, const wxChar *msg)
@@ -76,22 +186,8 @@ void wxOnAssert(const wxChar *fileName, int lineNumber, const wxChar *msg)
 
 wxFrame *gParentFrame = NULL;
 wxWindow *gParentWindow = NULL;
-ToolBarStub *gControlToolBarStub = NULL;
-ToolBarStub *gMixerToolBarStub = NULL;
-ToolBarStub *gEditToolBarStub = NULL;
-ToolBarStub *gMeterToolBarStub = NULL;
-ToolBarStub *gTranscriptionToolBarStub = NULL;
-
-//This array holds pointers to the toolbar stub pointers..
-ToolBarStub **gToolBarStubArray[ nToolBars ]=
-{
-   &gControlToolBarStub,
-   &gMixerToolBarStub,
-   &gEditToolBarStub,
-   &gMeterToolBarStub,
-   &gTranscriptionToolBarStub
-};
 
+bool gInited = false;
 bool gIsQuitting = false;
 
 void SaveWindowSize()
@@ -116,62 +212,45 @@ void QuitAudacity(bool bForce)
 
    gIsQuitting = true;
 
-   if(bForce)
-   {
-      wxMessageBox(_("WARNING: You may be prompted to save your work. Clicking cancel will have the same effect as clicking no."));
-   }
-
    // Try to close each open window.  If the user hits Cancel
    // in a Save Changes dialog, don't continue.
    // BG: unless force is true
 
-   SaveWindowSize();
-
    // BG: Are there any projects open?
-   if(!gAudacityProjects.IsEmpty())
+   if (!gAudacityProjects.IsEmpty())
    {
-      size_t len = gAudacityProjects.Count();
-      for (size_t i = 0; i < len; i++) {
-         if(bForce)
+      while (gAudacityProjects.Count())
+      {
+         if (bForce)
          {
-            gAudacityProjects[i]->Close(true);
+            gAudacityProjects[0]->Close(true);
          }
          else
          {
-	   if (!gAudacityProjects[i]->Close()){
-	       gIsQuitting = false;
+            if (!gAudacityProjects[0]->Close())
+            {
+               gIsQuitting = false;
                return;
+            }
          }
       }
    }
-   }
-
+   
    if (gFreqWindow)
       gFreqWindow->Destroy();
 
-
    if (gParentFrame)
       gParentFrame->Destroy();
 
    gFreqWindow = NULL;
    gParentFrame = NULL;
 
-   // Delete all the toolbars...
-   for(int i=0;i<nToolBars;i++)
-   {
-      if( *gToolBarStubArray[i] )
-      {
-         delete *gToolBarStubArray[i];
-         *gToolBarStubArray[i]=NULL;
-      }
-   }
+   CloseScreenshotTools();
 
-   //Delete the clipboard
+      //Delete the clipboard
    AudacityProject::DeleteClipboard();
 
-   QuitHelp();
-
-   if(bForce)
+   if (bForce)
    {
       wxExit();
    }
@@ -182,6 +261,80 @@ void QuitAudacity()
    QuitAudacity(false);
 }
 
+#if defined(__WXMSW__)
+
+//
+// DDE support for opening multiple files with one instance
+// of Audacity.
+//
+
+#define IPC_APPL wxT("audacity")
+#define IPC_TOPIC wxT("System")
+
+class IPCConn : public wxConnection
+{
+public:
+   IPCConn()
+   : wxConnection()
+   {
+   };
+
+   ~IPCConn()
+   {
+   };
+
+   bool OnExecute(const wxString & topic,
+                  wxChar *data,
+                  int size,
+                  wxIPCFormat format)
+   {
+      if (!gInited) {
+         return false;
+      }
+
+      AudacityProject *project = CreateNewAudacityProject(gParentWindow);
+
+      wxString cmd(data);
+
+      // We queue a command event to the project responsible for 
+      // opening the file since it can be a long process and we
+      // only have 5 seconds to return the Execute message to the
+      // client.
+      if (!cmd.IsEmpty()) {
+         wxCommandEvent e(EVT_OPEN_AUDIO_FILE);
+         e.SetString(data);
+         project->AddPendingEvent(e);
+      }
+
+      return true;
+   };
+};
+
+class IPCServ : public wxServer
+{
+public:
+   IPCServ()
+   : wxServer()
+   {
+      Create(wxT("audacity"));
+   };
+
+   ~IPCServ()
+   {
+   };
+
+   wxConnectionBase *OnAcceptConnection(const wxString & topic)
+   {
+      if (topic != IPC_TOPIC) {
+         return NULL;
+      }
+
+      return new IPCConn();
+   };
+};
+
+#endif
+
 IMPLEMENT_APP(AudacityApp)
 
 #ifdef __WXMAC__
@@ -189,6 +342,9 @@ IMPLEMENT_APP(AudacityApp)
 // in response of an open-document apple event
 void AudacityApp::MacOpenFile(const wxString &fileName)
 {
+   if (!gInited)
+      return;
+
    AudacityProject *project = GetActiveProject();
    if (project == NULL || !project->GetTracks()->IsEmpty()) {
       project = CreateNewAudacityProject(gParentWindow);
@@ -199,6 +355,9 @@ void AudacityApp::MacOpenFile(const wxString &fileName)
 // in response of a print-document apple event
 void AudacityApp::MacPrintFile(const wxString &fileName)
 {
+   if (!gInited)
+      return;
+
    AudacityProject *project = GetActiveProject();
    if (project == NULL || !project->GetTracks()->IsEmpty()) {
       project = CreateNewAudacityProject(gParentWindow);
@@ -209,6 +368,9 @@ void AudacityApp::MacPrintFile(const wxString &fileName)
 // in response of a open-application apple event
 void AudacityApp::MacNewFile()
 {
+   if (!gInited)
+      return;
+
    // This method should only be used on the Mac platform
    // when no project windows are open.
  
@@ -225,12 +387,10 @@ void AudacityApp::MacReopenApp()
 
 typedef int (AudacityApp::*SPECIALKEYEVENT)(wxKeyEvent&);
 
-#ifndef wxID_PREFERENCES
-#define wxID_PREFERENCES 1
-#endif
-
 BEGIN_EVENT_TABLE(AudacityApp, wxApp)
-   EVT_CHAR(AudacityApp::OnKeyEvent)
+   EVT_KEY_DOWN(AudacityApp::OnKeyDown)
+   EVT_CHAR(AudacityApp::OnChar)
+   EVT_KEY_UP(AudacityApp::OnKeyUp)
 #ifdef __WXMAC__
    EVT_MENU(wxID_NEW, AudacityApp::OnMenuNew)
    EVT_MENU(wxID_OPEN, AudacityApp::OnMenuOpen)
@@ -243,20 +403,6 @@ BEGIN_EVENT_TABLE(AudacityApp, wxApp)
 // EVT_MENU_RANGE(6050, 6060, AudacityApp::OnMRUProject)
 END_EVENT_TABLE()
 
-ToolBarStub * AudacityApp::LoadToolBar( const wxString Name, bool bDefault, 
-   wxWindow * pParent, enum ToolBarType tbt )
-{
-   bool bLoadToolBar=bDefault;
-   if( !Name.IsEmpty() )
-   {
-      gPrefs->Read(Name, &bLoadToolBar, bDefault);
-   }
-   if(bLoadToolBar)
-      return  new ToolBarStub(pParent, tbt);
-   
-   return NULL;
-}
-
 // Backend for OnMRUFile and OnMRUProject
 bool AudacityApp::MRUOpen(wxString fileName) {
    // Most of the checks below are copied from AudacityProject::ShowFileDialog
@@ -356,6 +502,10 @@ void AudacityApp::OnMRUProject(wxCommandEvent& event) {
 // main frame
 bool AudacityApp::OnInit()
 {
+   #if USE_QUICKTIME
+   ::InitQuicktime();
+   #endif
+
    // Unused strings that we want to be translated, even though
    // we're not using them yet...
    wxString future1 = _("Master Gain Control");
@@ -378,6 +528,10 @@ bool AudacityApp::OnInit()
 
    wxString home = wxGetHomeDir();
    mAppHomeDir = home;
+   theTheme.EnsureInitialised();
+
+   // AColor depends on theTheme.
+   AColor::Init(); 
 
    // On Unix systems, the default temp dir is in /tmp.
    // Search path (in this order):
@@ -390,7 +544,7 @@ bool AudacityApp::OnInit()
    wxString pathVar = wxGetenv(wxT("AUDACITY_PATH"));
    if (pathVar != wxT(""))
       AddMultiPathsToPathList(pathVar, audacityPathList);
-   AddUniquePathToPathList(FROMFILENAME(::wxGetCwd()), audacityPathList);
+   AddUniquePathToPathList(::wxGetCwd(), audacityPathList);
    AddUniquePathToPathList(wxString::Format(wxT("%s/.audacity-files"),
                                             home.c_str()),
                            audacityPathList);
@@ -419,7 +573,7 @@ bool AudacityApp::OnInit()
    wxFileName tmpFile;
    tmpFile.AssignTempFileName(wxT("nn"));
    wxString tmpDirLoc = tmpFile.GetPath(wxPATH_GET_VOLUME);
-   ::wxRemoveFile(FILENAME(tmpFile.GetFullPath()));
+   ::wxRemoveFile(tmpFile.GetFullPath());
 
    // On Mac and Windows systems, use the directory which contains Audacity.
    #ifdef __WXMSW__
@@ -523,11 +677,12 @@ bool AudacityApp::OnInit()
    // be visible, but when all other windows are closed, this menu bar should
    // become visible.
 
-   gParentFrame = new wxFrame(NULL, -1, wxT("invisible"), wxPoint(5000, 5000), wxSize(100, 100));
+   gParentFrame = new wxFrame(NULL, -1, wxT("invisible"), wxPoint(5000, 5000), wxSize(100, 100), wxFRAME_NO_TASKBAR);
 
    wxMenu *fileMenu = new wxMenu();
    fileMenu->Append(wxID_NEW, wxT("&New\tCtrl+N"));
    fileMenu->Append(wxID_OPEN, wxT("&Open...\tCtrl+O"));
+   fileMenu->Append(wxID_ABOUT, _("&About Audacity..."));
    /* i18n-hint: Mac OS X shortcut should be Ctrl+, */
    fileMenu->Append(wxID_PREFERENCES, _("&Preferences...\tCtrl+,"));
 
@@ -536,59 +691,73 @@ bool AudacityApp::OnInit()
 
    gParentFrame->SetMenuBar(menuBar);
 
-   gParentFrame->Show();
+   gParentFrame->Hide();
 
    SetTopWindow(gParentFrame);
-
 #endif
 
    SetExitOnFrameDelete(true);
 
-
-   ///////////////////////////////////////////////////////////////////
-   //////////////////////////////////////////////////////////////////
-   //Initiate pointers to toolbars here, and create 
-   //the toolbars that should be loaded at startup.
-
-   gControlToolBarStub = 
-      LoadToolBar( wxT(""),true,
-      gParentWindow,ControlToolBarID);
-   gMixerToolBarStub = 
-      LoadToolBar( wxT("/GUI/EnableMixerToolBar"),true,
-      gParentWindow,MixerToolBarID);
-   gMeterToolBarStub = 
-      LoadToolBar( wxT("/GUI/EnableMeterToolBar"),true,
-      gParentWindow,MeterToolBarID);
-   gEditToolBarStub = 
-      LoadToolBar( wxT("/GUI/EnableEditToolBar"),true,
-      gParentWindow,EditToolBarID);
-   gTranscriptionToolBarStub = 
-      LoadToolBar( wxT("/GUI/EnableTranscriptionToolBar"),false,
-      gParentWindow,TranscriptionToolBarID);
-
-   /// ToolBar Initiation Complete.
-   ////////////////////////////////////////////////////////////////
-   ////////////////////////////////////////////////////////////////
-
    AudacityProject *project = CreateNewAudacityProject(gParentWindow);
+
+#ifdef EXPERIMENTAL_TRACK_PANEL_FROM_THE_START
+   // Parasite Track Panel.
+   // Takes over the display...
+   wxWindow * pWnd = new TrackPanel2(NULL) ;
+   SetTopWindow(pWnd);
+   pWnd->Show( true );
+   project->Show( false );
+#else
    SetTopWindow(project);
+#endif
+
 
    delete temporarywindow;
 
-   // Can't handle command-line args on Mac OS X yet...
-   // Cygwin command-line parser below...
-   #if !defined(__MACOSX__) && !defined(__CYGWIN__)
+   //
+   // Auto-recovery
+   //
+
+   bool didRecoverAnything = false;
+   if (!ShowAutoRecoveryDialogIfNeeded(&project, &didRecoverAnything))
+   {
+      // Important: Prevent deleting any temporary files!
+      DirManager::SetDontDeleteTempFiles();
+      QuitAudacity(true);
+   }
+
+   //
+   // Command-line parsing, but only if we didn't recover
+   //
+
+#if !defined(__CYGWIN__)
+
    // Parse command-line arguments
-   if (argc > 1) {
+   if (argc > 1 && !didRecoverAnything) {
       for (int option = 1; option < argc; option++) {
          if (!argv[option])
             continue;
          bool handled = false;
 
+// JKC: In order to have long strings that span multiple lines,
+// and for this to work with i18n, use the magic formula
+// of wrapping continuations with wxT() but not the first
+// part of the string.
+// Why does it work?  Because in unicode _(s) and wxT(s) both
+// prepend an L to s.  That's why you don't want a wxT on the 
+// first string.
+
          if (!wxString(wxT("-help")).CmpNoCase(argv[option])) {
-            wxPrintf(/* i18n-hint: '-help', '-test' and
+            wxPrintf(/* i18n-hint: '-help', '-version', '-test' and
                       '-blocksize' need to stay in English. */
-                   _("Command-line options supported:\n  -help (this message)\n  -test (run self diagnostics)\n  -blocksize ### (set max disk block size in bytes)\n\nIn addition, specify the name of an audio file or Audacity project\nto open it.\n\n"));
+                   _(
+                    "Command-line options supported:\n"
+                    wxT("  -help (this message)\n")
+                    wxT("  -version (display Audacity version)\n")
+                    wxT("  -test (run self diagnostics)\n")
+                    wxT("  -blocksize ### (set max disk block size in bytes)\n\n")
+                    wxT("In addition, specify the name of an audio file or Audacity project\n")
+                    wxT("to open it.\n\n")));
             exit(0);
          }
 
@@ -612,21 +781,36 @@ bool AudacityApp::OnInit()
             exit(0);
          }
 
+         if (!handled && !wxString(wxT("-version")).CmpNoCase(argv[option])) {
+            wxPrintf(wxT("Audacity v%s (%s)\n"),
+                     AUDACITY_VERSION_STRING,
+                     (wxUSE_UNICODE ? wxT("Unicode") : wxT("ANSI")));
+            exit(0);
+         }
+
          if (argv[option][0] == wxT('-') && !handled) {
             wxPrintf(_("Unknown command line option: %s\n"), argv[option]);
             exit(0);
          }
 
          if (!handled)
+         {
+            if (!project)
+            {
+               // Create new window for project
+               project = CreateNewAudacityProject(gParentWindow);
+            }
             project->OpenFile(argv[option]);
+            project = NULL; // don't reuse this project for other file
+         }
 
       }                         // for option...
    }                            // if (argc>1)
-   #endif // not Mac OS X
+
+#else
 	
    // Cygwin command line parser (by Dave Fancella)
-   #if defined(__CYGWIN__)
-   if (argc > 1) {
+   if (argc > 1 && !didRecoverAnything) {
       int optionstart = 1;
       bool startAtOffset = false;
 		
@@ -696,19 +880,28 @@ bool AudacityApp::OnInit()
             openThisFile = true;
          if(openThisFile) {
             openThisFile = false;
+            if (!project)
+            {
+               // Create new window for project
+               project = CreateNewAudacityProject(gParentWindow);
+            }
             project->OpenFile(fileToOpen);
+            project = NULL; // don't reuse this project for other file
          }
 
       }                         // for option...
    }                            // if (argc>1)
-   #endif // Cygwin command-line parser
 
+#endif // Cygwin command-line parser
+
+   gInited = true;
+   
    return TRUE;
 }
 
 bool AudacityApp::InitCleanSpeech()
 {
-   wxString cwd = FROMFILENAME(::wxGetCwd());
+   wxString userdatadir = FileNames::DataDir();
    wxString presetsFromPrefs = gPrefs->Read(wxT("/Directories/PresetsDir"), wxT(""));
    wxString presets = wxT("");
 
@@ -717,41 +910,38 @@ bool AudacityApp::InitCleanSpeech()
       presetsFromPrefs = wxT("");
    #endif
 
-   #ifdef __WXMSW__
-   wxString presetsDefaultLoc = cwd + wxT("\\presets");
-   #else
-   wxString presetsDefaultLoc = cwd + wxT("/presets");
-   #endif
+   wxString presetsDefaultLoc =
+      wxFileName(userdatadir, wxT("presets")).GetFullPath();
 
    // Stop wxWindows from printing its own error messages (not used ... does this really do anything?)
    wxLogNull logNo;
-
+   
    // Try temp dir that was stored in prefs first
    if (presetsFromPrefs != wxT("")) {
-      if (wxDirExists(FILENAME(presetsFromPrefs)))
+      if (wxDirExists(presetsFromPrefs))
          presets = presetsFromPrefs;
-      else if (wxMkdir(FILENAME(presetsFromPrefs)))
+      else if (wxMkdir(presetsFromPrefs))
          presets = presetsFromPrefs;
    }
 
    // If that didn't work, try the default location
    if ((presets == wxT("")) && (presetsDefaultLoc != wxT(""))) {
-      if (wxDirExists(FILENAME(presetsDefaultLoc)))
+      if (wxDirExists(presetsDefaultLoc))
          presets = presetsDefaultLoc;
-      else if (wxMkdir(FILENAME(presetsDefaultLoc)))
+      else if (wxMkdir(presetsDefaultLoc))
          presets = presetsDefaultLoc;
    }
 
    if (presets == wxT("")) {
       // Failed
-      wxMessageBox(_("Audacity could not find a place to store\n.csp CleanSpeech preset files\nAudacity is now going to exit. \nInstallation may be corrupt."));
+      wxMessageBox(wxT("Audacity could not find a place to store\n.csp CleanSpeech preset files\nAudacity is now going to exit. \nInstallation may be corrupt."));
       return false;
    }
 
    // The permissions don't always seem to be set on
    // some platforms.  Hopefully this fixes it...
    #ifdef __UNIX__
-   chmod(FILENAME(presets).fn_str(), 0755);
+   chmod(OSFILENAME(presets), 0755);
    #endif
 
    gPrefs->Write(wxT("/Directories/PresetsDir"), presets);
@@ -779,18 +969,18 @@ bool AudacityApp::InitTempDir()
    // Try temp dir that was stored in prefs first
 
    if (tempFromPrefs != wxT("")) {
-      if (wxDirExists(FILENAME(tempFromPrefs)))
+      if (wxDirExists(tempFromPrefs))
          temp = tempFromPrefs;
-      else if (wxMkdir(FILENAME(tempFromPrefs)))
+      else if (wxMkdir(tempFromPrefs))
          temp = tempFromPrefs;
    }
 
    // If that didn't work, try the default location
 
    if (temp==wxT("") && tempDefaultLoc != wxT("")) {
-      if (wxDirExists(FILENAME(tempDefaultLoc)))
+      if (wxDirExists(tempDefaultLoc))
          temp = tempDefaultLoc;
-      else if (wxMkdir(FILENAME(tempDefaultLoc)))
+      else if (wxMkdir(tempDefaultLoc))
          temp = tempDefaultLoc;
    }
 
@@ -809,7 +999,7 @@ bool AudacityApp::InitTempDir()
    // The permissions don't always seem to be set on
    // some platforms.  Hopefully this fixes it...
    #ifdef __UNIX__
-   chmod(FILENAME(temp).fn_str(), 0755);
+   chmod(OSFILENAME(temp), 0755);
    #endif
 
    gPrefs->Write(wxT("/Directories/TempDir"), temp);
@@ -819,8 +1009,6 @@ bool AudacityApp::InitTempDir()
    if (!CreateSingleInstanceChecker(temp))
       return false;
 
-   DirManager::CleanTempDir(true);
-
    return true;
 }
 
@@ -836,12 +1024,16 @@ bool AudacityApp::CreateSingleInstanceChecker(wxString dir)
    wxString name = wxString::Format(wxT("audacity-lock-%s"), wxGetUserId().c_str());
    mChecker = new wxSingleInstanceChecker();
 
-   if (!mChecker->Create(FILENAME(name), FILENAME(dir))) {
+   wxString runningTwoCopiesStr = _("Running two copies of Audacity simultaneously may cause\ndata loss or cause your system to crash.\n\n");
+
+   if (!mChecker->Create(name, dir)) {
       // Error initializing the wxSingleInstanceChecker.  We don't know
       // whether there is another instance running or not.
 
       wxString prompt =
-         _("Audacity was not able to lock the temporary files directory.\nThis folder may be in use by another copy of Audacity.\nRunning two copies of Audacity simultaneously may cause\ndata loss or cause your system to crash.\n\nDo you still want to start Audacity?");
+         _("Audacity was not able to lock the temporary files directory.\nThis folder may be in use by another copy of Audacity.\n") +
+         runningTwoCopiesStr +
+         _("Do you still want to start Audacity?");
       int action = wxMessageBox(prompt,
                                 _("Error locking temporary folder"),
                                 wxYES_NO | wxICON_EXCLAMATION,
@@ -852,16 +1044,49 @@ bool AudacityApp::CreateSingleInstanceChecker(wxString dir)
       }
    }
    else if ( mChecker->IsAnotherRunning() ) {
+#if defined(__WXMSW__)
+      //
+      // On Windows (and possibly Linux?), we attempt to make
+      // a DDE connection to an already active Audacity.  If
+      // successful, we send the first command line argument
+      // (the audio file name) to that Audacity for processing.
+      wxClient client;
+      wxConnectionBase *conn;
+
+      // We try up to 10 times since there's a small window
+      // where the DDE server may not have been fully initialized
+      // yet.
+      for (int i = 0; i < 10; i++) {
+         conn = client.MakeConnection(wxEmptyString,
+                                      IPC_APPL,
+                                      IPC_TOPIC);
+         if (conn) {
+            if (conn->Execute(argv[1])) {
+               // Command was successfully queued so exit quietly
+               delete mChecker;
+               return false;
+            }
+         }
+         wxMilliSleep(100);
+      }
+#endif
       // There is another copy of Audacity running.  Force quit.
       
       wxString prompt =
-         _("The system has detected that another copy of Audacity is running.\nRunning two copies of Audacity simultaneously may lead to\ndata loss or cause your system to crash, so is not allowed.\n\nUse the New or Open commands in the currently running Audacity\nprocess to open multiple projects simultaneously.\n");
+         _("The system has detected that another copy of Audacity is running.\n") +
+         runningTwoCopiesStr +
+         _("Use the New or Open commands in the currently running Audacity\nprocess to open multiple projects simultaneously.\n");
       wxMessageBox(prompt, _("Audacity is already running"),
             wxOK | wxICON_ERROR);
       delete mChecker;
       return false;
    }
 
+#if defined(__WXMSW__)
+   // Create the DDE server
+   mIPCServ = new IPCServ();
+#endif
+
    return true;
 }
 
@@ -903,70 +1128,77 @@ void AudacityApp::FindFilesInPathList(wxString pattern,
    if (pattern == wxT(""))
       return;
 
+   wxFileName f;
+
    for(unsigned i=0; i<pathList.GetCount(); i++) {
-      wxString path = pathList[i];
-
-      wxString fname = 
-         wxFindFirstFile(path + wxFILE_SEP_PATH + pattern);
-
-      while(fname != wxT("")) {
-         results.Add(fname);
-         fname = wxFindNextFile();
-      }
+      f = pathList[i] + wxFILE_SEP_PATH + pattern;
+      wxDir::GetAllFiles(f.GetPath(), &results, f.GetFullName(), wxDIR_FILES);
    }
 }
 
-void AudacityApp::OnKeyEvent(wxKeyEvent & event )
+void AudacityApp::OnKeyDown(wxKeyEvent & event)
 {
-   event.Skip();
+   // Not handled
+   event.Skip(true);
+
+   // Make sure this event is destined for a project window
+   AudacityProject *prj = GetActiveProject();
+
+   // TODO: I don't know how it can happen, but it did on 2006-07-06.
+   // I was switching between apps fairly quickly so maybe that has something
+   // to do with it.
+   if (!prj)
+      return;
+
+   if (prj != wxGetTopLevelParent(wxWindow::FindFocus()))
+      return;
+
+   if (prj->HandleKeyDown(event))
+      event.Skip(false);
 }
 
-
-//BG: Filters all events before they are processed
-int AudacityApp::FilterEvent(wxEvent& event)
+void AudacityApp::OnChar(wxKeyEvent & event)
 {
-   //Send key events to the commands code
-   if(event.GetEventType() == wxEVT_KEY_DOWN ||
-      event.GetEventType() == wxEVT_KEY_UP)
-      return (this->*((SPECIALKEYEVENT) (&AudacityApp::OnAllKeys)))
-         ((wxKeyEvent&)event);
-   return -1;
+   // Not handled
+   event.Skip(true);
+
+   // Make sure this event is destined for a project window
+   AudacityProject *prj = GetActiveProject();
+
+   // TODO: I don't know how it can happen, but it did on 2006-07-06.
+   // I was switching between apps fairly quickly so maybe that has something
+   // to do with it.
+   if (!prj)
+      return;
+
+   if (prj != wxGetTopLevelParent(wxWindow::FindFocus()))
+      return;
+
+   if (prj->HandleChar(event))
+      event.Skip(false);
 }
 
-//BG: THIS IS NOT A NORMAL KEY CALLBACK
-//BG: it gets called before everything else
-//BG: return -1 to allow other things to process keydown events
-//BG: return TRUE to signify that the event has been processed, but do not allow anything else to process this event
-//BG: return FALSE to signigy that the event has not been processed and should not be processed
-//BG: do not call event.skip, I do not know what would happen
-
-
-int AudacityApp::OnAllKeys(wxKeyEvent& event)
+void AudacityApp::OnKeyUp(wxKeyEvent & event)
 {
-   AudacityProject *audacityPrj = GetActiveProject();
+   // Not handled
+   event.Skip(true);
 
-   if (!audacityPrj) {
-      return -1;
-   }
+   // Make sure this event is destined for a project window
+   AudacityProject *prj = GetActiveProject();
 
-   if(audacityPrj->IsActive())
-   {
-      if (event.GetEventType() == wxEVT_KEY_DOWN) {
-         if (audacityPrj->HandleKeyDown(event))
-            return true;
-      }
-      if (event.GetEventType() == wxEVT_KEY_UP) {
-         if (audacityPrj->HandleKeyUp(event))
-            return true;
-      }
-   }
+   // TODO: I don't know how it can happen, but it did on 2006-07-06.
+   // I was switching between apps fairly quickly so maybe that has something
+   // to do with it.
+   if (!prj)
+      return;
 
-   return -1;
+   if (prj != wxGetTopLevelParent(wxWindow::FindFocus()))
+      return;
+
+   if (prj->HandleKeyUp(event))
+      event.Skip(false);
 }
 
-
-
-
 int AudacityApp::OnExit()
 {
    gIsQuitting = true;
@@ -975,6 +1207,10 @@ int AudacityApp::OnExit()
       Dispatch();
    }
 
+#if defined(__WXMSW__)
+   delete mIPCServ;
+#endif
+
    if(gPrefs)
    {
       bool bFalse = false;
@@ -1016,8 +1252,8 @@ void AudacityApp::OnMenuAbout(wxCommandEvent & event)
    // this happens, and enable the same code to be present on
    // all platforms.
    if(gAudacityProjects.GetCount() == 0) {
-   AboutDialog dlog(NULL);
-   dlog.ShowModal();
+      AboutDialog dlog(NULL);
+      dlog.ShowModal();
    }
    else
       event.Skip();
@@ -1080,10 +1316,13 @@ void AudacityApp::OnMenuExit(wxCommandEvent & event)
    // this happens, and enable the same code to be present on
    // all platforms.
 
-   if(gAudacityProjects.GetCount() == 0)
+   // LL:  Removed "if" to allow closing based on final project count.
+   // if(gAudacityProjects.GetCount() == 0)
       QuitAudacity();
-   else
-      event.Skip();
+   
+   // LL:  Veto quit if projects are still open.  This can happen
+   //      if the user selected Cancel in a Save dialog.
+   event.Skip(gAudacityProjects.GetCount() == 0);
    
 }
 
@@ -1098,83 +1337,105 @@ void AudacityApp::OnMenuExit(wxCommandEvent & event)
 #if defined(__WXMSW__) && !defined(__WXUNIVERSAL__) && !defined(__CYGWIN__)
 void AudacityApp::AssociateFileTypes()
 {
-	wxRegKey associateFileTypes;
-	associateFileTypes.SetName(wxT("HKCR\\.AUP"));
-	bool bKeyExists = associateFileTypes.Exists();
-	if (!bKeyExists) {
-		// Not at HKEY_CLASSES_ROOT. Try HKEY_CURRENT_USER.
-		associateFileTypes.SetName(wxT("HKCU\\.AUP"));
-		bKeyExists = associateFileTypes.Exists();
-	}
-	if (!bKeyExists) {	
-		// File types are not currently associated. 
-		// Check pref in case user has already decided against it.
-		bool bWantAssociateFiles = true;
-		if (!gPrefs->Read(wxT("/WantAssociateFiles"), &bWantAssociateFiles) || 
-				bWantAssociateFiles) {
-			// Either there's no pref or user does want associations 
-			// and they got stepped on, so ask.
-			int wantAssoc = 
-				wxMessageBox(
-					_("Audacity project (.AUP) files are not currently \nassociated with Audacity. \n\nAssociate them, so they open on double-click?"), 
-					_("Audacity Project Files"), 
-					wxYES_NO | wxICON_QUESTION);
-			if (wantAssoc == wxYES) {
-				gPrefs->Write(wxT("/WantAssociateFiles"), true);
+   wxRegKey associateFileTypes;
+   associateFileTypes.SetName(wxT("HKCR\\.AUP"));
+   bool bKeyExists = associateFileTypes.Exists();
+   if (!bKeyExists) {
+      // Not at HKEY_CLASSES_ROOT. Try HKEY_CURRENT_USER.
+      associateFileTypes.SetName(wxT("HKCU\\Software\\Classes\\.AUP"));
+      bKeyExists = associateFileTypes.Exists();
+   }
+   if (!bKeyExists) {	
+      // File types are not currently associated. 
+      // Check pref in case user has already decided against it.
+      bool bWantAssociateFiles = true;
+      if (!gPrefs->Read(wxT("/WantAssociateFiles"), &bWantAssociateFiles) || 
+            bWantAssociateFiles) {
+         // Either there's no pref or user does want associations 
+         // and they got stepped on, so ask.
+         int wantAssoc = 
+            wxMessageBox(
+               _("Audacity project (.AUP) files are not currently \nassociated with Audacity. \n\nAssociate them, so they open on double-click?"), 
+               _("Audacity Project Files"), 
+               wxYES_NO | wxICON_QUESTION);
+         if (wantAssoc == wxYES) {
+            gPrefs->Write(wxT("/WantAssociateFiles"), true);
 
-				wxString root_key = wxT("HKCR");
-				associateFileTypes.SetName(wxT("HKCR\\.AUP")); // Start again with HKEY_CLASSES_ROOT.
-				if (!associateFileTypes.Create(true)) {
-					// Not at HKEY_CLASSES_ROOT. Try HKEY_CURRENT_USER.
-					associateFileTypes.SetName(wxT("HKCU\\.AUP"));
-					if (!associateFileTypes.Create(true)) { 
-						// Actually, can't create keys. Empty root_key to flag failure.
-						root_key.Empty();
-					} else {
-						// User must not have privileges to create at HKEY_CLASSES_ROOT.
-						// Succeeded setting key for HKEY_CURRENT_USER. Continue that way.
-						root_key = wxT("HKCU");
-					}
-				}
-				if (root_key.IsEmpty()) {
-					//v Warn that we can't set keys. Ask whether to set pref for no retry?
-				} else {
-					associateFileTypes = wxT("Audacity.Project"); // Finally set value for .AUP key
+            wxString root_key;
 
-					associateFileTypes.SetName(root_key + wxT("\\Audacity.Project"));
-					if(!associateFileTypes.Exists()) {
-						associateFileTypes.Create(true);
-						associateFileTypes = wxT("Audacity Project File");
-					}
+            root_key = wxT("HKCU\\Software\\Classes\\");
+            associateFileTypes.SetName(root_key + wxT(".AUP")); // Start again with HKEY_CLASSES_ROOT.
+            if (!associateFileTypes.Create(true)) {
+               // Not at HKEY_CLASSES_USER. Try HKEY_CURRENT_ROOT.
+               root_key = wxT("HKCR\\");
+               associateFileTypes.SetName(root_key + wxT(".AUP"));
+               if (!associateFileTypes.Create(true)) { 
+                  // Actually, can't create keys. Empty root_key to flag failure.
+                  root_key.Empty();
+               }
+            }
+            if (root_key.IsEmpty()) {
+               //v Warn that we can't set keys. Ask whether to set pref for no retry?
+            } else {
+               associateFileTypes = wxT("Audacity.Project"); // Finally set value for .AUP key
 
-					associateFileTypes.SetName(root_key + wxT("\\Audacity.Project\\shell"));
-					if(!associateFileTypes.Exists()) {
-						associateFileTypes.Create(true);
-						associateFileTypes = wxT("");
-					}
+               associateFileTypes.SetName(root_key + wxT("Audacity.Project"));
+               if(!associateFileTypes.Exists()) {
+                  associateFileTypes.Create(true);
+                  associateFileTypes = wxT("Audacity Project File");
+               }
 
-					associateFileTypes.SetName(root_key + wxT("\\Audacity.Project\\shell\\open"));
-					if(!associateFileTypes.Exists()) {
-						associateFileTypes.Create(true);
-					}
+               associateFileTypes.SetName(root_key + wxT("Audacity.Project\\shell"));
+               if(!associateFileTypes.Exists()) {
+                  associateFileTypes.Create(true);
+                  associateFileTypes = wxT("");
+               }
 
-					associateFileTypes.SetName(root_key + wxT("\\Audacity.Project\\shell\\open\\command"));
-					wxString tmpRegAudPath;
-					if(associateFileTypes.Exists()) {
-						tmpRegAudPath = wxString(associateFileTypes).Lower();
-					}
-					if (!associateFileTypes.Exists() || 
-							(tmpRegAudPath.Find(wxT("\\audacity.exe")) >= 0)) {
-						associateFileTypes.Create(true);
-						associateFileTypes = (wxString)argv[0] + (wxString)wxT(" \"%1\"");
-					}
-				}
-			} else {
-				// User said no. Set a pref so we don't keep asking.
-				gPrefs->Write(wxT("/WantAssociateFiles"), false);
-			}
-		}
-	}
+               associateFileTypes.SetName(root_key + wxT("Audacity.Project\\shell\\open"));
+               if(!associateFileTypes.Exists()) {
+                  associateFileTypes.Create(true);
+               }
+
+               associateFileTypes.SetName(root_key + wxT("Audacity.Project\\shell\\open\\command"));
+               wxString tmpRegAudPath;
+               if(associateFileTypes.Exists()) {
+                  tmpRegAudPath = wxString(associateFileTypes).Lower();
+               }
+               if (!associateFileTypes.Exists() || 
+                     (tmpRegAudPath.Find(wxT("audacity.exe")) >= 0)) {
+                  associateFileTypes.Create(true);
+                  associateFileTypes = (wxString)argv[0] + (wxString)wxT(" \"%1\"");
+               }
+
+#if 0
+               // These can be use later to support more startup messages
+               // like maybe "Import into existing project" or some such.
+               // Leaving here for an example...
+               associateFileTypes.SetName(root_key + wxT("Audacity.Project\\shell\\open\\ddeexec"));
+               if(!associateFileTypes.Exists()) {
+                  associateFileTypes.Create(true);
+                  associateFileTypes = wxT("%1");
+               }
+
+               associateFileTypes.SetName(root_key + wxT("Audacity.Project\\shell\\open\\ddeexec\\Application"));
+               if(!associateFileTypes.Exists()) {
+                  associateFileTypes.Create(true);
+                  associateFileTypes = IPC_APPL;
+               }
+
+               associateFileTypes.SetName(root_key + wxT("Audacity.Project\\shell\\open\\ddeexec\\Topic"));
+               if(!associateFileTypes.Exists()) {
+                  associateFileTypes.Create(true);
+                  associateFileTypes = IPC_TOPIC;
+               }
+#endif
+            }
+         } else {
+            // User said no. Set a pref so we don't keep asking.
+            gPrefs->Write(wxT("/WantAssociateFiles"), false);
+         }
+      }
+   }
 }
 #endif
 
