@@ -51,11 +51,11 @@ enum eCommandType { CtEffect, CtMenu, CtSpecial };
 // TIDY-ME: Not currently translated, 
 // but there are issues to address if we do.  
 wxString SpecialCommands[] = {
-   wxT("No Action"),
+   wxT("NoAction"),
    wxT("Import"),
-   wxT("Save Hq Master1"),
-   wxT("Save Hq Master2"),
-   wxT("Stereo To Mono"),
+   wxT("SaveHqMaster1"),
+   wxT("SaveHqMaster2"),
+   wxT("StereoToMono"),
    wxT("ExportFlac"),
    wxT("ExportMp3"),
    wxT("ExportOgg"),
@@ -131,8 +131,9 @@ bool BatchCommands::ReadChain(const wxString & chain)
 
          // Find the command name terminator...ingore line if not found
          int splitAt = tf[i].Find(wxT(':'));
-         if (splitAt < 0)
+         if (splitAt < 0) {
             continue;
+         }
 
          // Parse and clean
          wxString cmd = tf[i].Left(splitAt).Strip(wxString::both);
@@ -229,13 +230,13 @@ void BatchCommands::SetCleanSpeechChain()
 // Commands (at least currently) don't.  Messy.
 
 /* i18n-hint: Effect name translations must agree with those used elsewhere, or batch won't find them */
-   AddToChain(   _("Stereo To Mono") );
-   AddToChain(   _("Normalize") );
-   AddToChain( wxT("Save Hq Master1") );
-   AddToChain(   _("Noise Removal") );
-   AddToChain(   _("Truncate Silence") );
-   AddToChain(   _("Leveller") );
-   AddToChain(   _("Normalize") );
+   AddToChain( wxT("StereoToMono") );
+   AddToChain( wxT("Normalize") );
+   AddToChain( wxT("SaveHqMaster1") );
+   AddToChain( wxT("NoiseRemoval") );
+   AddToChain( wxT("TruncateSilence") );
+   AddToChain( wxT("Leveller") );
+   AddToChain( wxT("Normalize") );
    AddToChain( wxT("ExportMp3") );
 }
 
@@ -243,14 +244,14 @@ void BatchCommands::SetWavToMp3Chain()
 {
    ResetChain();
  
-   AddToChain(   _("Normalize") );
+   AddToChain( wxT("Normalize") );
    AddToChain( wxT("ExportMp3") );
 }
 
 // Gets all commands that are valid for this mode.
 wxArrayString BatchCommands::GetAllCommands()
 {
-   wxArrayString commands;
+   wxArrayString commands(true);
    wxString command;
    commands.Clear();
 
@@ -271,9 +272,10 @@ wxArrayString BatchCommands::GetAllCommands()
        additionalEffects = 0;
    effects = Effect::GetEffects(PROCESS_EFFECT | BUILTIN_EFFECT | additionalEffects);
    for(i=0; i<effects->GetCount(); i++) {
-      command=(*effects)[i]->GetEffectName();
-      command.Replace( wxT("..."), wxT(""));
-      commands.Add( command);
+      command=(*effects)[i]->GetEffectIdentifier();
+      if (!command.IsEmpty()) {
+         commands.Add( command);
+      }
    }
    delete effects;
 
@@ -298,8 +300,7 @@ Effect * BatchCommands::GetEffectFromCommandName(wxString inCommand)
    EffectArray * effects = Effect::GetEffects( ALL_EFFECTS );
    for(i=0; i<effects->GetCount(); i++) {
       f = (*effects)[i];
-      command=f->GetEffectName();
-      command.Replace( wxT("..."), wxT(""));
+      command=f->GetEffectIdentifier();
       if( command.IsSameAs( inCommand ))
       {  
          delete effects;
@@ -345,13 +346,13 @@ double BatchCommands::GetEndTime()
    AudacityProject *project = GetActiveProject();
    if( project == NULL )
    {
-      wxMessageBox( _("No project and no Audio to process!") );
+      //wxMessageBox( wxT("No project to process!") );
       return -1.0;
    }
    TrackList * tracks = project->GetTracks();
    if( tracks == NULL )
    {
-      wxMessageBox( _("No tracks to process!") );
+      //wxMessageBox( wxT("No tracks to process!") );
       return -1.0;
    }
 
@@ -364,14 +365,14 @@ bool BatchCommands::IsMono()
    AudacityProject *project = GetActiveProject();
    if( project == NULL )
    {
-      wxMessageBox( _("No project and no Audio to process!") );
+      //wxMessageBox( wxT("No project and no Audio to process!") );
       return false;
    }
 
    TrackList * tracks = project->GetTracks();
    if( tracks == NULL )
    {
-      wxMessageBox( _("No tracks to process!") );
+      //wxMessageBox( wxT("No tracks to process!") );
       return false;
    }
 
@@ -391,8 +392,10 @@ bool BatchCommands::IsMono()
 
 bool BatchCommands::WriteMp3File( const wxString Name, int bitrate )
 {  //check if current project is mono or stereo
-   bool Mp3Stereo = true;
-   if (IsMono()) {Mp3Stereo = false;};
+   int numChannels = 2;
+   if (IsMono()) {
+      numChannels = 1;
+   }
 
    double endTime = GetEndTime();
    if( endTime <= 0.0f )
@@ -402,14 +405,15 @@ bool BatchCommands::WriteMp3File( const wxString Name, int bitrate )
    {
       // 'No' bitrate given, use the current default.
       // Use Mp3Stereo to control if export is to a stereo or mono file
-      return ::ExportMP3(project, Mp3Stereo, Name, false, 0.0, endTime);
+      return mExporter.Process(project, numChannels, wxT("MP3"), Name, false, 0.0, endTime);
    }
 
+
    bool rc;
    long prevBitRate = gPrefs->Read(wxT("/FileFormats/MP3Bitrate"), 128);
    gPrefs->Write(wxT("/FileFormats/MP3Bitrate"), bitrate);
    // Use Mp3Stereo to control if export is to a stereo or mono file
-   rc = ::ExportMP3(project, Mp3Stereo, Name, false, 0.0, endTime);
+   rc = mExporter.Process(project, numChannels, wxT("MP3"), Name, false, 0.0, endTime);
    gPrefs->Write(wxT("/FileFormats/MP3Bitrate"), prevBitRate);
    return rc;
 }
@@ -430,8 +434,11 @@ bool BatchCommands::WriteMp3File( const wxString Name, int bitrate )
 // ======= IMPORTANT ========
 bool BatchCommands::ApplySpecialCommand(int iCommand, const wxString command,const wxString params)
 {
+   if (ReportAndSkip(command, params))
+      return true;
+
    AudacityProject *project = GetActiveProject();
-   wxString filename;
+
    int numChannels = 1;		//used to switch between mono and stereo export
    if (IsMono()) {
       numChannels = 1;	//export in mono
@@ -439,9 +446,7 @@ bool BatchCommands::ApplySpecialCommand(int iCommand, const wxString command,con
       numChannels = 2;	//export in stereo
    }
 
-   if( ReportAndSkip(command, params))
-      return true;
-
+   wxString filename;
    if (mFileName.IsEmpty()) {   
       filename = project->BuildCleanFileName(project->GetFileName());
    }
@@ -452,56 +457,60 @@ bool BatchCommands::ApplySpecialCommand(int iCommand, const wxString command,con
    // We have a command index, but we don't use it!
    // TODO: Make this special-batch-command code use the menu item code....
    // FIX-ME: No error reporting on write file failure in batch mode.
-   if( command == wxT("No Action")){
+   if (command == wxT("NoAction")) {
       return true;
-   } else if (!mFileName.IsEmpty() && command == wxT("Import") ){
+   } else if (!mFileName.IsEmpty() && command == wxT("Import")) {
       // historically this was in use, now ignored if there
       return true;
-   } else if (command == wxT("Save Hq Master1")){
+   } else if (command == wxT("SaveHqMaster1")) {
       filename.Replace(wxT("cleaned/"), wxT("cleaned/MasterBefore_"), false);
-      return WriteMp3File( filename, 56 );
-   } else if (command == wxT("Save Hq Master2")){
+      return WriteMp3File(filename, 56);
+   } else if (command == wxT("SaveHqMaster2")) {
       filename.Replace(wxT("cleaned/"), wxT("cleaned/MasterAfter_"), false);
-      return WriteMp3File ( filename, 56 );
-   } else if (command == wxT("Stereo To Mono")){
+      return WriteMp3File(filename, 56);
+   } else if (command == wxT("StereoToMono")) {
       // StereoToMono is an effect masquerading as a menu item.
-      Effect * f=GetEffectFromCommandName( _("Stereo To Mono") );
-      if( f!=NULL )
-         return ApplyEffectCommand( f, command, params );
-      wxMessageBox( _("Stereo To Mono Effect not found"));
+      Effect * f = GetEffectFromCommandName(wxT("StereoToMono"));
+      if (f != NULL) {
+         return ApplyEffectCommand(f, command, params);
+      }
+      wxMessageBox(_("Stereo To Mono Effect not found"));
       return false;
-   } else if (command == wxT("ExportMp3") ){
-      return WriteMp3File ( filename, 0 ); // 0 bitrate means use default/current
-   } else if (command == wxT("ExportWav") ){
+   } else if (command == wxT("ExportMp3")) {
+      return WriteMp3File(filename, 0); // 0 bitrate means use default/current
+   } else if (command == wxT("ExportWav")) {
       filename.Replace(wxT(".mp3"), wxT(".wav"), false);
       double endTime = GetEndTime();
-      if( endTime <= 0.0f )
+      if (endTime <= 0.0f) {
          return false;
-      return ::ExportPCM(project, numChannels, filename, false, 0.0, endTime);
-   } else if (command == wxT("ExportOgg")){
+      }
+      return mExporter.Process(project, numChannels, wxT("WAV"), filename, false, 0.0, endTime);
+   } else if (command == wxT("ExportOgg")) {
 #ifdef USE_LIBVORBIS
       filename.Replace(wxT(".mp3"), wxT(".ogg"), false);
       double endTime = GetEndTime();
-      if( endTime <= 0.0f )
+      if (endTime <= 0.0f) {
          return false;
-      return ::ExportOGG(project, numChannels, filename, false, 0.0, endTime);
+      }
+      return mExporter.Process(project, numChannels, wxT("OGG"), filename, false, 0.0, endTime);
 #else
       wxMessageBox(_("Ogg Vorbis support is not included in this build of Audacity"));
       return false;
 #endif
-   } else if (command == wxT("ExportFlac")){
+   } else if (command == wxT("ExportFlac")) {
 #ifdef USE_LIBFLAC
       filename.Replace(wxT(".mp3"), wxT(".flac"), false);
       double endTime = GetEndTime();
-      if( endTime <= 0.0f )
+      if (endTime <= 0.0f) {
          return false;
-      return ::ExportFLAC(project, numChannels, filename, false, 0.0, endTime);
+      }
+      return mExporter.Process(project, numChannels, wxT("FLAC"), filename, false, 0.0, endTime);
 #else
       wxMessageBox(_("FLAC support is not included in this build of Audacity"));
       return false;
 #endif
    } 
-   wxMessageBox( wxString::Format(_("Command %s not implemented yet"),command.c_str()) );
+   wxMessageBox(wxString::Format(_("Command %s not implemented yet"),command.c_str()));
    return false;
 }
 
