@@ -16,7 +16,7 @@
   This class manages an envelope - i.e. a piecewise linear funtion
   that the user can edit by dragging control points around.  The
   envelope is most commonly used to control the amplitude of a
-  waveform, but it is also used to shape a general FFT filter.
+  waveform, but it is also used to shape the Equalization curve.
 
 *//****************************************************************//**
 
@@ -226,13 +226,21 @@ bool Envelope::HandleXMLTag(const wxChar *tag, const wxChar **attrs)
 {
    if (!wxStrcmp(tag, wxT("envelope"))) {
       int numPoints = 0;
+      long nValue = -1;
 
       while (*attrs) {
          const wxChar *attr = *attrs++;
          const wxChar *value = *attrs++;
-         if (!wxStrcmp(attr, wxT("numpoints")))
-            numPoints = wxAtoi(value);
+         if (!value)
+            break;
+         const wxString strValue = value;
+         if( !wxStrcmp(attr, wxT("numpoints")) && 
+               XMLValueChecker::IsGoodInt(strValue) && strValue.ToLong(&nValue)) 
+            numPoints = nValue;
       }
+      if (numPoints < 0)
+         return false;
+
       WX_CLEAR_ARRAY(mEnv);
       mEnv.Alloc(numPoints);
       return true;
@@ -793,11 +801,10 @@ void Envelope::GetValues(double *buffer, int bufferLen,
    double tprev, vprev, tnext = 0, vnext, vstep = 0;
 
    for (int b = 0; b < bufferLen; b++) {
-
       if (len <= 0) {
-	 buffer[b] = mDefaultValue;
-	 t += tstep;
-	 continue;
+         buffer[b] = mDefaultValue;
+         t += tstep;
+         continue;
       }
       if (t <= mEnv[0]->t) {
          buffer[b] = mEnv[0]->val;
@@ -940,8 +947,7 @@ double Envelope::Average( double t0, double t1 )
 //
 // The functions below are used by the TimeTrack and possibly for
 // other debugging.  They do not affect normal amplitude envelopes
-// for waveforms, not frequency envelopes for the FFT Filter or
-// equalization.
+// for waveforms, not frequency envelopes for equalization.
 //
 
 // We should be able to write a very efficient memoizer for this
