@@ -9,10 +9,10 @@
 *******************************************************************//**
 
 \class FreqWindow
-\brief Displays a spectrum plot of the waveform.  Has options for 
+\brief Displays a spectrum plot of the waveform.  Has options for
 selecting parameters of the plot.
 
-Has a feature that finds peaks and rerports their value as you move 
+Has a feature that finds peaks and reports their value as you move
 the mouse around.
 
 *//****************************************************************//**
@@ -21,11 +21,16 @@ the mouse around.
 \brief Works with FreqWindow to dsplay a spectrum plot of the waveform.
 This class actually does the graph display.
 
-Has a feature that finds peaks and rerports their value as you move 
+Has a feature that finds peaks and reports their value as you move
 the mouse around.
 
 *//*******************************************************************/
 
+/*
+  Salvo Ventura - November 2006
+  Extended range check for additional FFT windows
+*/
+
 
 #include "Audacity.h"
 
@@ -66,6 +71,8 @@ the mouse around.
 #include "Theme.h"
 #include "AllThemeResources.h"
 
+#include "FileDialog.h"
+
 enum {
    FirstID = 7000,
 
@@ -114,7 +121,7 @@ FreqWindow::FreqWindow(wxWindow * parent, wxWindowID id,
                            const wxString & title,
                            const wxPoint & pos):
   wxDialog(parent, id, title, pos, wxDefaultSize,
-     wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER ),
+     wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER | wxMAXIMIZE_BOX ),
   mData(NULL), mProcessed(NULL), mBitmap(NULL)
 {
    mMouseX = 0;
@@ -143,7 +150,7 @@ FreqWindow::FreqWindow(wxWindow * parent, wxWindowID id,
    wxStaticText *algLabel = new wxStaticText(this, wxID_ANY, _("Algorithm:"));
    mAlgChoice = new wxChoice(this, FreqAlgChoiceID,
                              wxDefaultPosition, wxDefaultSize,
-                             4, algChoiceStrings);
+                             5, algChoiceStrings);
 
    mAlgChoice->SetSelection(0);
 
@@ -157,7 +164,8 @@ FreqWindow::FreqWindow(wxWindow * parent, wxWindowID id,
       wxT("16384")
    };
 
-   wxStaticText *sizeLabel = new wxStaticText(this, wxID_ANY, _("Size:"));
+   wxStaticText *sizeLabel = new wxStaticText(this, wxID_ANY,
+                                              wxString(_("Size")) + wxT(":"));
    mSizeChoice = new wxChoice(this, FreqSizeChoiceID,
                               wxDefaultPosition, wxDefaultSize,
                               8, sizeChoiceStrings);
@@ -195,7 +203,7 @@ FreqWindow::FreqWindow(wxWindow * parent, wxWindowID id,
    mLogAxis = false;
 
    mExportButton = new wxButton(this, FreqExportButtonID,
-                                _("Export..."));
+                                _("&Export..."));
 
    mCloseButton = new wxButton(this, FreqCloseButtonID,
                                _("Close"));
@@ -245,7 +253,7 @@ FreqWindow::FreqWindow(wxWindow * parent, wxWindowID id,
    r.Union( mCloseButton->GetRect() );
    mExportButton->SetMinSize( r.GetSize() );
    mCloseButton->SetMinSize( r.GetSize() );
-   
+
    // Add everything to the sizers
 
    wxBoxSizer *vs = new wxBoxSizer( wxVERTICAL );
@@ -341,9 +349,7 @@ void FreqWindow::OnSize(wxSizeEvent & event)
 
    DrawPlot();
 
-#if defined(__WXMSW__)
    Refresh(true);
-#endif
 }
 
 void FreqWindow::DrawPlot()
@@ -541,7 +547,7 @@ void FreqWindow::OnAxisChoice(wxCommandEvent & event)
 
 /* Declare Static functions */
 static float CubicInterpolate(float y0, float y1, float y2, float y3, float x);
-static float CubicMaximize(float y0, float y1, float y2, float y3);
+static float CubicMaximize(float y0, float y1, float y2, float y3, float * max);
 
 float CubicInterpolate(float y0, float y1, float y2, float y3, float x)
 {
@@ -558,7 +564,7 @@ float CubicInterpolate(float y0, float y1, float y2, float y3, float x)
    return (a * xxx + b * xx + c * x + d);
 }
 
-float CubicMaximize(float y0, float y1, float y2, float y3)
+float CubicMaximize(float y0, float y1, float y2, float y3, float * max)
 {
    // Find coefficients of cubic
 
@@ -594,9 +600,15 @@ float CubicMaximize(float y0, float y1, float y2, float y3)
    float ddb = db;
 
    if (dda * x1 + ddb < 0)
+   {
+      *max = a*x1*x1*x1+b*x1*x1+c*x1+d;
       return x1;
+   }
    else
+   {
+      *max = a*x2*x2*x2+b*x2*x2+c*x2+d;
       return x2;
+   }
 }
 
 float FreqWindow::GetProcessedValue(float freq0, float freq1)
@@ -692,6 +704,7 @@ void FreqWindow::PlotPaint(wxPaintEvent & evt)
 
       bool up = (mProcessed[1] > mProcessed[0]);
       float bestdist = 1000000;
+      float bestValue = 0.0;
       for (int bin = 2; bin < mProcessedSize; bin++) {
          bool nowUp = mProcessed[bin] > mProcessed[bin - 1];
          if (!nowUp && up) {
@@ -699,10 +712,11 @@ void FreqWindow::PlotPaint(wxPaintEvent & evt)
             int leftbin = bin - 2;
             if (leftbin < 1)
                leftbin = 1;
+            float valueAtMax = 0.0;
             float max = leftbin + CubicMaximize(mProcessed[leftbin],
                                                 mProcessed[leftbin + 1],
                                                 mProcessed[leftbin + 2],
-                                                mProcessed[leftbin + 3]);
+                                                mProcessed[leftbin + 3], &valueAtMax);
 
             float thispeak;
             if (alg == 0)
@@ -713,6 +727,7 @@ void FreqWindow::PlotPaint(wxPaintEvent & evt)
             if (fabs(thispeak - xPos) < bestdist) {
                bestpeak = thispeak;
                bestdist = fabs(thispeak - xPos);
+               bestValue = valueAtMax;
                if (thispeak > xPos)
                   break;
             }
@@ -752,18 +767,20 @@ void FreqWindow::PlotPaint(wxPaintEvent & evt)
          peakpitch = PitchName_Absolute(Freq2Pitch(bestpeak));
          xp = xpitch.c_str();
          pp = peakpitch.c_str();
-         info.Printf(_("Cursor: %d Hz (%s) = %d dB    Peak: %d Hz (%s)"),
-                     int (xPos + 0.5),
-                     xp, int (value + 0.5), int (bestpeak + 0.5), pp);
+
+         info.Printf(_("Cursor: %d Hz (%s) = %d dB    Peak: %d Hz (%s) = %.1f dB"),
+               int (xPos + 0.5), xp,
+               int (value + 0.5), int (bestpeak + 0.5),
+               pp, bestValue);
       } else if (xPos > 0.0 && bestpeak > 0.0) {
          xpitch = PitchName_Absolute(Freq2Pitch(1.0 / xPos));
          peakpitch = PitchName_Absolute(Freq2Pitch(1.0 / bestpeak));
          xp = xpitch.c_str();
          pp = peakpitch.c_str();
-         info.Printf(_("Cursor: %.4f sec (%d Hz) (%s) = %f,    Peak: %.4f sec (%d Hz) (%s)"),
+         info.Printf(_("Cursor: %.4f sec (%d Hz) (%s) = %f,    Peak: %.4f sec (%d Hz) (%s) = %.3f"),
                      xPos,
                      int (1.0 / xPos + 0.5),
-                     xp, value, bestpeak, int (1.0 / bestpeak + 0.5), pp);
+                     xp, value, bestpeak, int (1.0 / bestpeak + 0.5), pp, bestValue);
       }
       mInfoText->SetLabel(info);
    }
@@ -818,7 +835,7 @@ void FreqWindow::Recalc()
    (mSizeChoice->GetStringSelection()).ToLong(&windowSize);
 
    if (!(windowSize >= 32 && windowSize <= 65536 &&
-         alg >= 0 && alg <= 3 && windowFunc >= 0 && windowFunc <= 3)) {
+         alg >= 0 && alg <= 4 && windowFunc >= 0 && windowFunc <= 9)) {
       mFreqPlot->Refresh(true);
       return;
    }
@@ -841,14 +858,18 @@ void FreqWindow::Recalc()
    float *in2 = new float[mWindowSize];
    float *out = new float[mWindowSize];
    float *out2 = new float[mWindowSize];
+   float *win = new float[mWindowSize];
+
+   // initialize the window
+   for(int i=0; i<mWindowSize; i++)
+      win[i] = 1.0;
+   WindowFunc(windowFunc, mWindowSize, win);
 
    int start = 0;
    int windows = 0;
    while (start + mWindowSize <= mDataLen) {
       for (i = 0; i < mWindowSize; i++)
-         in[i] = mData[start + i];
-
-      WindowFunc(windowFunc, mWindowSize, in);
+         in[i] = win[i] * mData[start + i];
 
       switch (alg) {
       case 0:                  // Spectrum
@@ -892,9 +913,15 @@ void FreqWindow::Recalc()
          FFT(mWindowSize, false, in, NULL, out, out2);
 
          // Compute log power
+         float power;
          for (i = 0; i < mWindowSize; i++)
-            in[i] = log((out[i] * out[i]) + (out2[i] * out2[i]));
-
+         {
+            power = (out[i] * out[i]) + (out2[i] * out2[i]);
+            if(power <= 0.)
+               in[i] = -100000.;
+            else
+               in[i] = log((out[i] * out[i]) + (out2[i] * out2[i]));
+         }
          // Take IFFT
          FFT(mWindowSize, true, in, NULL, out, out2);
 
@@ -912,12 +939,21 @@ void FreqWindow::Recalc()
    switch (alg) {
    case 0:                     // Spectrum
       // Convert to decibels
+      mYMin = 1000000.;
+      mYMax = -1000000.;
       for (i = 0; i < half; i++)
+      {
          mProcessed[i] = 10 * log10(mProcessed[i] / mWindowSize / windows);
+         if(mProcessed[i] > mYMax)
+            mYMax = mProcessed[i];
+         else if(mProcessed[i] < mYMin)
+            mYMin = mProcessed[i];
+      }
+      if(mYMin < -90.)
+         mYMin = -90.;
+      mYMax += .5;
 
       mProcessedSize = half;
-      mYMin = -90;
-      mYMax = 10;
       mYStep = 10;
       break;
 
@@ -1004,6 +1040,7 @@ void FreqWindow::Recalc()
    delete[]in2;
    delete[]out;
    delete[]out2;
+   delete[]win;
 
    DrawPlot();
    mFreqPlot->Refresh(true);
@@ -1013,8 +1050,8 @@ void FreqWindow::OnExport(wxCommandEvent & WXUNUSED(event))
 {
    wxString fName = _("spectrum.txt");
 
-   fName = wxFileSelector(_("Export Spectral Data As:"),
-                          NULL, fName, wxT("txt"), wxT("*.txt"), wxSAVE, this);
+   fName = FileSelector(_("Export Spectral Data As:"),
+                        NULL, fName, wxT("txt"), wxT("*.txt"), wxSAVE, this);
 
    if (fName == wxT(""))
       return;
@@ -1043,7 +1080,7 @@ void FreqWindow::OnExport(wxCommandEvent & WXUNUSED(event))
       f.AddLine(_("Lag (seconds)\tFrequency (Hz)\tLevel"));
       for (int i = 1; i < mProcessedSize; i++)
          f.AddLine(wxString::Format(wxT("%f\t%f\t%f"),
-                                    i / mRate, 1.0 / i, mProcessed[i]));
+                                    i / mRate, mRate / i, mProcessed[i]));
    }
 
 #ifdef __WXMAC__
