@@ -36,6 +36,8 @@
 #include "ViewInfo.h"
 #include "widgets/TimeTextCtrl.h"
 
+#include "FileDialog.h"
+
 enum Column
 {
    Col_Track,
@@ -49,7 +51,7 @@ static const wxChar *headers[Col_Max] =
 {
    _("Track"),
    _("Label"),
-   _("Start Time"),
+   _NoAcc("&Start Time"),
    _("End Time")
 };
 
@@ -95,7 +97,7 @@ LabelDialog::LabelDialog(wxWindow *parent,
                          double rate)
 : wxDialog(parent,
            wxID_ANY,
-           _("Edit Labels"),
+           _NoAcc("&Edit Labels"),
            wxDefaultPosition,
            wxSize(800, 600),
            wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER),
@@ -344,9 +346,7 @@ bool LabelDialog::Validate()
 
    for (i = 0; i < cnt; i++) {
       if (mData[i]->title.IsEmpty()) {
-         int id = wxMessageBox(_("You have left blank label names.  These will be\n"
-                             wxT("skipped when repopulating the Label Tracks.\n\n")
-                             wxT("Would you like to go back and provide names?")),
+         int id = wxMessageBox(_("You have left blank label names.  These will be\nskipped when repopulating the Label Tracks.\n\nWould you like to go back and provide names?"),
                                _("Confirm"),
                                wxYES_NO | wxICON_EXCLAMATION);
          if (id == wxYES) {
@@ -502,13 +502,13 @@ void LabelDialog::OnImport(wxCommandEvent &event)
 
    // Ask user for a filename
    wxString fileName =
-       wxFileSelector(_("Select a text file containing labels..."),
-                      path,     // Path
-                      wxT(""),       // Name
-                      wxT(".txt"),   // Extension
-                      _("Text files (*.txt)|*.txt|All files (*.*)|*.*"),
-                      0,        // Flags
-                      this);    // Parent
+       FileSelector(_("Select a text file containing labels..."),
+                    path,     // Path
+                    wxT(""),       // Name
+                    wxT(".txt"),   // Extension
+                    _("Text files (*.txt)|*.txt|All files (*.*)|*.*"),
+                    0,        // Flags
+                    this);    // Parent
 
    // They gave us one...
    if (fileName != wxT("")) {
@@ -552,11 +552,11 @@ void LabelDialog::OnExport(wxCommandEvent &event)
 
    wxString fName;
 
-   fName = wxFileSelector(_("Export Labels As:"),
-                          NULL,
-                          _("labels.txt"),
-                          wxT("txt"),
-                          wxT("*.txt"), wxSAVE | wxOVERWRITE_PROMPT, this);
+   fName = FileSelector(_("Export Labels As:"),
+                        NULL,
+                        _("labels.txt"),
+                        wxT("txt"),
+                        wxT("*.txt"), wxSAVE | wxOVERWRITE_PROMPT, this);
 
    if (fName == wxT(""))
       return;
@@ -689,7 +689,7 @@ void LabelDialog::OnChangeTrack(wxGridEvent &event, int row, RowData *rd)
       wxTextEntryDialog d(this,
                           _("New Label Track"),
                           _("Enter track name"),
-                          _("Label Track"));
+                          _NoAcc("&Label Track"));
 
       // User canceled so repopulating the grid will set the track
       // name to the orignal value
@@ -736,3 +736,21 @@ void LabelDialog::OnChangeEtime(wxGridEvent &event, int row, RowData *rd)
 
    return;
 }
+
+void LabelDialog::OnOK(wxCommandEvent &event)
+{
+   // Standard handling
+   if (Validate() && TransferDataFromWindow()) {
+      EndModal(wxID_OK);
+   }
+
+   return;
+}
+
+void LabelDialog::OnCancel(wxCommandEvent &event)
+{
+   // Standard handling
+   EndModal(wxID_CANCEL);
+
+   return;
+}
