@@ -8,18 +8,40 @@
   James Crook
   Jun Wan
 
-**********************************************************************/
+*******************************************************************//**
+
+\class LabelTrack
+\brief A LabelTrack is a Track that holds labels (LabelStruct).
+
+These are used to annotate a waveform.
+Each label has a start time and an end time.
+The text of the labels is editable and the
+positions of the end points are draggable.
+
+*//****************************************************************//**
+
+\class LabelStruct
+\brief A LabelStruct holds information for ONE label in a LabelTrack
+
+LabelStruct also has label specific functions, mostly functions
+for drawing different aspects of the label and its text box.
+
+*//*******************************************************************/
+
+#include "Audacity.h"
 
 #include <stdio.h>
 
+#include <wx/bitmap.h>
 #include <wx/brush.h>
 #include <wx/clipbrd.h>
 #include <wx/dataobj.h>
 #include <wx/dc.h>
+#include <wx/dcclient.h>
 #include <wx/event.h>
-#include <wx/icon.h>
 #include <wx/intl.h>
 #include <wx/log.h>
+#include <wx/msgdlg.h>
 #include <wx/pen.h>
 #include <wx/string.h>
 #include <wx/textfile.h>
@@ -29,17 +51,21 @@
 #include "DirManager.h"
 #include "Internat.h"
 #include "Prefs.h"
+#include "Theme.h"
+#include "AllThemeResources.h"
+#include "AColor.h"
 
 wxFont LabelTrack::msFont;
 
 // static member variables.
 bool LabelTrack::mbGlyphsReady=false;
-// We have several variants of the icons (highlighting).
-// The icons are draggable, and you can drag one boundary
-// or all boundaries at the same timecode depending on whether you 
-// click the centre (for all) or the arrow part (for one).
-// Currently we have twelve variants but we're only using six.
-wxIcon LabelTrack::mBoundaryGlyphs[ NUM_GLYPH_CONFIGS * NUM_GLYPH_HIGHLIGHTS ];
+
+/// We have several variants of the icons (highlighting).
+/// The icons are draggable, and you can drag one boundary
+/// or all boundaries at the same timecode depending on whether you 
+/// click the centre (for all) or the arrow part (for one).
+/// Currently we have twelve variants but we're only using six.
+wxBitmap LabelTrack::mBoundaryGlyphs[ NUM_GLYPH_CONFIGS * NUM_GLYPH_HIGHLIGHTS ];
 int LabelTrack::mIconHeight;
 int LabelTrack::mIconWidth;
 int LabelTrack::mTextHeight;
@@ -52,27 +78,27 @@ LabelTrack *TrackFactory::NewLabelTrack()
    return new LabelTrack(mDirManager);
 }
 
+
 LabelTrack::LabelTrack(DirManager * projDirManager):
    Track(projDirManager),
    mMouseOverLabelLeft(-1),
    mMouseOverLabelRight(-1),
    mIsAdjustingLabel(false)
 {
-   InitColours();
-   SetName(_("Label Track"));
+   SetDefaultName(_NoAcc("&Label Track"));
+   SetName(GetDefaultName());
 
    ResetFont();
 
    // Label tracks are narrow
    // Default is to allow two rows so that new users get the
    // idea that labels can 'stack' when they would overlap.
-   mHeight = 62;     
+   mHeight = 73;     
    CreateCustomGlyphs();
    mSelIndex = -1;
 
-   // reset falgs
+   // reset flags
    ResetFlags();
-
 }
 
 LabelTrack::LabelTrack(const LabelTrack &orig) :
@@ -81,8 +107,6 @@ LabelTrack::LabelTrack(const LabelTrack &orig) :
    mMouseOverLabelRight(-1),
    mIsAdjustingLabel(false)
 {
-   InitColours();
-
    int len = orig.mLabels.Count();
 
    for (int i = 0; i < len; i++) {
@@ -113,7 +137,6 @@ void LabelTrack::ResetFlags()
    mCurrentCursorPos = 1;
    mResetCursorPos = false;
    mRightDragging = false;
-   mKeyOn = false;
    mDrawCursor = false;
 }
 
@@ -126,6 +149,10 @@ void LabelTrack::ResetFont()
                       wxFONTWEIGHT_NORMAL, FALSE, facename,
                       wxFONTENCODING_SYSTEM);
    }
+   else {
+      msFont = wxFont(12, wxFONTFAMILY_DEFAULT, wxFONTSTYLE_NORMAL,
+                      wxFONTWEIGHT_NORMAL);
+   }
 }
 
 /// ComputeTextPosition is 'smart' about where to display 
@@ -349,6 +376,7 @@ void LabelTrack::ComputeLayout( wxRect & r, double h, double pps)
 LabelStruct::LabelStruct() {
    changeInitialMouseXPos = true;
    highlighted = false;
+   updated = false;
 }
 
 /// Draw vertical lines that go exactly through the position
@@ -361,7 +389,7 @@ void LabelStruct::DrawLines( wxDC & dc, wxRect & r)
    // start, i.e. what is the y position of the icon?
    // We addjust this slightly so that the line encroaches on 
    // the icon slightly (there is white space in the design).
-   const int yIconStart = y - (LabelTrack::mIconHeight /2)+1;
+   const int yIconStart = y - (LabelTrack::mIconHeight /2)+1+(LabelTrack::mTextHeight+3)/2;
    const int yIconEnd   = yIconStart + LabelTrack::mIconHeight-2;
 
    if (y<0) 
@@ -390,14 +418,14 @@ void LabelStruct::DrawGlyphs( wxDC & dc, wxRect & r, int GlyphLeft, int GlyphRig
    if (y<0) 
       return;
    const int xHalfWidth=LabelTrack::mIconWidth/2;
-   const int yStart=y-LabelTrack::mIconHeight/2;
+   const int yStart=y-LabelTrack::mIconHeight/2+(LabelTrack::mTextHeight+3)/2;
 
    if((x  >= r.x) && (x  <= (r.x+r.width)))
-      dc.DrawIcon( LabelTrack::mBoundaryGlyphs[GlyphLeft], x-xHalfWidth,yStart );
+      dc.DrawBitmap(LabelTrack::GetGlyph(GlyphLeft), x-xHalfWidth,yStart, true);
    // The extra test here suppresses right hand markers when they overlap
    // the left hand marker (e.g. zoomed out) or are to the left.
    if((x1 >= r.x) && (x1 <= (r.x+r.width)) && (x1>x+LabelTrack::mIconWidth))
-      dc.DrawIcon( LabelTrack::mBoundaryGlyphs[GlyphRight], x1-xHalfWidth,yStart );
+      dc.DrawBitmap(LabelTrack::GetGlyph(GlyphRight), x1-xHalfWidth,yStart, true);
 }
 
 /// Draw the text of the label and also draw
@@ -446,7 +474,7 @@ void LabelStruct::DrawTextBox( wxDC & dc, wxRect & r)
       if( (xStart < (r.x+r.width)) && (xEnd > r.x) && (xWidth>0))
       {
          
-         wxRect bar( xStart,y-yBarHeight/2, 
+         wxRect bar( xStart,y-yBarHeight/2+yFrameHeight/2, 
             xWidth,yBarHeight);
          if( x1 > x+xBarShorten )
             dc.DrawRectangle(bar);
@@ -487,7 +515,10 @@ void LabelStruct::DrawHighlight( wxDC & dc, int xPos1, int xPos2, int charHeight
    curPen.SetColour(wxString(wxT("BLUE")));
    wxBrush curBrush = dc.GetBrush();
    curBrush.SetColour(wxString(wxT("BLUE")));
-   dc.DrawRectangle(xPos1, y-charHeight/2, (int) (xPos2-xPos1+0.5), charHeight);
+   if (xPos1 < xPos2)
+      dc.DrawRectangle(xPos1-1, y-charHeight/2, xPos2-xPos1+1, charHeight);
+   else
+      dc.DrawRectangle(xPos2-1, y-charHeight/2, xPos1-xPos2+1, charHeight);
 }
 
 void LabelStruct::getXPos( wxDC & dc, int * xPos1, int cursorPos) 
@@ -502,6 +533,23 @@ void LabelStruct::getXPos( wxDC & dc, int * xPos1, int cursorPos)
    }
 }
 
+bool LabelTrack::CalcCursorX(wxWindow * parent, int * x)
+{
+   if (mSelIndex >= 0) {
+      wxClientDC dc(parent);
+
+      if (msFont.Ok()) {
+         dc.SetFont(msFont);
+      }
+
+      mLabels[mSelIndex]->getXPos(dc, x, mCurrentCursorPos);
+      *x += LabelTrack::mIconWidth / 2;
+      return true;
+   }
+
+   return false;
+}
+
 /// Draw calls other functions to draw the LabelTrack.
 ///   @param  dc the device context
 ///   @param  r  the LabelTrack rectangle.
@@ -528,15 +576,15 @@ void LabelTrack::Draw(wxDC & dc, wxRect & r, double h, double pps,
 
    wxRect before = r;
    before.width = int ((dsel0 - h) * pps);
-   dc.SetBrush(mUnselectedBrush);
-   dc.SetPen(mUnselectedPen);
+   dc.SetBrush(AColor::labelUnselectedBrush);
+   dc.SetPen(AColor::labelUnselectedPen);
    dc.DrawRectangle(before);
 
    wxRect selr = r;
    selr.x += before.width;
    selr.width = int ((dsel1 - dsel0) * pps);
-   dc.SetBrush(mSelectedBrush);
-   dc.SetPen(mSelectedPen);
+   dc.SetBrush(AColor::labelSelectedBrush);
+   dc.SetPen(AColor::labelSelectedPen);
    dc.DrawRectangle(selr);
 
    wxRect after = r;
@@ -548,8 +596,8 @@ void LabelTrack::Draw(wxDC & dc, wxRect & r, double h, double pps,
       after.x = r.x;
    }
 
-   dc.SetBrush(mUnselectedBrush);
-   dc.SetPen(mUnselectedPen);
+   dc.SetBrush(AColor::labelUnselectedBrush);
+   dc.SetPen(AColor::labelUnselectedPen);
    dc.DrawRectangle(after);
 
    int i;
@@ -578,10 +626,10 @@ void LabelTrack::Draw(wxDC & dc, wxRect & r, double h, double pps,
    dc.GetTextExtent(wxT("Demo Text x^y"), &textWidth, &textHeight);
    mTextHeight = (int)textHeight;   
    ComputeLayout( r, h , pps );
-   dc.SetTextForeground(wxColour(0, 0, 0));
+   dc.SetTextForeground(theTheme.Colour( clrLabelTrackText));
    dc.SetBackgroundMode(wxTRANSPARENT);
-   dc.SetBrush(mTextNormalBrush);
-   dc.SetPen(mLabelSurroundPen);
+   dc.SetBrush(AColor::labelTextNormalBrush);
+   dc.SetPen(AColor::labelSurroundPen);
    const int nLabels = (int)mLabels.Count();
    int GlyphLeft;
    int GlyphRight;
@@ -609,9 +657,9 @@ void LabelTrack::Draw(wxDC & dc, wxRect & r, double h, double pps,
    // Draw the label boxes.
    for (i = 0; i < nLabels; i++)
    {
-      if( mSelIndex==i) dc.SetBrush(mTextEditBrush);
+      if( mSelIndex==i) dc.SetBrush(AColor::labelTextEditBrush);
       mLabels[i]->DrawTextBox( dc, r );
-      if( mSelIndex==i) dc.SetBrush(mTextNormalBrush);
+      if( mSelIndex==i) dc.SetBrush(AColor::labelTextNormalBrush);
    }
 
    // Draw highlights
@@ -633,9 +681,9 @@ void LabelTrack::Draw(wxDC & dc, wxRect & r, double h, double pps,
    // Draw the text and the label boxes.
    for (i = 0; i < nLabels; i++)
    {
-      if( mSelIndex==i) dc.SetBrush(mTextEditBrush);
+      if( mSelIndex==i) dc.SetBrush(AColor::labelTextEditBrush);
       mLabels[i]->DrawText( dc, r );
-      if( mSelIndex==i) dc.SetBrush(mTextNormalBrush);
+      if( mSelIndex==i) dc.SetBrush(AColor::labelTextNormalBrush);
    }
 
    // Draw the cursor, if there is one.
@@ -670,7 +718,7 @@ void LabelTrack::Draw(wxDC & dc, wxRect & r, double h, double pps,
       const int CursorWidth=2;
       if (mDrawCursor) {
          currentPen.SetWidth(CursorWidth);
-         dc.DrawLine(xPos, mLabels[i]->y - mFontHeight/2, xPos, mLabels[i]->y + mFontHeight/2);
+         dc.DrawLine(xPos-1, mLabels[i]->y - mFontHeight/2 + 1, xPos-1, mLabels[i]->y + mFontHeight/2);
          currentPen.SetWidth(1);
       }
    }
@@ -762,7 +810,7 @@ bool LabelTrack::CutSelectedText()
    wxString left=wxT("");
    wxString right=wxT("");
    wxString text = mLabels[mSelIndex]->title;
-   
+
    // swapping to make sure currentCursorPos > initialCursorPos always
    if (mInitialCursorPos > mCurrentCursorPos) {
       int temp = mCurrentCursorPos;
@@ -771,8 +819,8 @@ bool LabelTrack::CutSelectedText()
    }
 
    // data for cutting
-   wxString data = mLabels[mSelIndex]->title.Mid(mInitialCursorPos, mCurrentCursorPos-mInitialCursorPos);
-   
+   wxString data = text.Mid(mInitialCursorPos, mCurrentCursorPos-mInitialCursorPos);
+
    // get left-remaining text
    if (mInitialCursorPos > 0) {
       left = text.Mid(0, mInitialCursorPos);
@@ -830,61 +878,67 @@ bool LabelTrack::CopySelectedText()
 
 /// Paste the text on the clipboard to text box
 ///  @return true if mouse is clicked in text box, false otherwise
-bool LabelTrack::PasteSelectedText() 
+bool LabelTrack::PasteSelectedText(double sel0, double sel1)
 {
-   if (mSelIndex == -1)
-      return false;
+   if (mSelIndex == -1) {
+      AddLabel(sel0, sel1, wxT(""));
+   }
 
-   wxTextDataObject data;
+   wxString text;
    wxString left=wxT("");
    wxString right=wxT("");
    
-   // if the mouse is clicked in text box
-   if (mInBox) {
-      // if text data is available
-      if (wxTheClipboard->IsSupported(wxDF_TEXT))
-      {
-         if (wxTheClipboard->Open()) {
-            wxTheClipboard->GetData( data );
-            wxTheClipboard->Close();
-         }
+   // if text data is available
+   if (wxTheClipboard->IsSupported(wxDF_TEXT)) {
+      if (wxTheClipboard->Open()) {
+         wxTextDataObject data;
+         wxTheClipboard->GetData( data );
+         wxTheClipboard->Close();
+         text = data.GetText();
       }
 
-      // if there is some highlighted text
-      if (mLabels[mSelIndex]->highlighted) {
-         // swapping to make sure currentCursorPos > mInitialCursorPos always
-         if (mInitialCursorPos > mCurrentCursorPos) {
-            int temp = mCurrentCursorPos;
-            mCurrentCursorPos = mInitialCursorPos;
-            mInitialCursorPos = temp;
+      // Convert control characters to blanks
+      int i;
+      for (i = 0; i < (int)text.Length(); i++) {
+         if (wxIscntrl(text[i])) {
+            text[i] = wxT(' ');
          }
-
-         // same as cutting
-         if (mInitialCursorPos > 0) {
-            left = (mLabels[mSelIndex]->title).Mid(0, mInitialCursorPos);
-         }
-         if (mCurrentCursorPos < (int)(mLabels[mSelIndex]->title).Length()) {
-            right = (mLabels[mSelIndex]->title).Mid(mCurrentCursorPos, (mLabels[mSelIndex]->title).Length()-mCurrentCursorPos);
-         }
-         mLabels[mSelIndex]->title = left + data.GetText() + right;            
-         mCurrentCursorPos = left.Length() + data.GetTextLength() - 1;
-      } 
-      else
-      {
-         // insert the data on the clipboard from the cursor position
-         if (mCurrentCursorPos < (int)(mLabels[mSelIndex]->title).Length()) {
-            right = (mLabels[mSelIndex]->title).Mid(mCurrentCursorPos);
-         }
-         mLabels[mSelIndex]->title = (mLabels[mSelIndex]->title).Left(mCurrentCursorPos);
-         mLabels[mSelIndex]->title += data.GetText();
-         mLabels[mSelIndex]->title += right;
-         mCurrentCursorPos += data.GetTextLength() - 1;
       }
-      // set mInitialCursorPos equal to currentCursorPos
-      mInitialCursorPos = mCurrentCursorPos;
-      return true;
    }
-   return false;
+
+   // if there is some highlighted text
+   if (mLabels[mSelIndex]->highlighted) {
+      // swapping to make sure currentCursorPos > mInitialCursorPos always
+      if (mInitialCursorPos > mCurrentCursorPos) {
+         int temp = mCurrentCursorPos;
+         mCurrentCursorPos = mInitialCursorPos;
+         mInitialCursorPos = temp;
+      }
+
+      // same as cutting
+      if (mInitialCursorPos > 0) {
+         left = (mLabels[mSelIndex]->title).Mid(0, mInitialCursorPos);
+      }
+      if (mCurrentCursorPos < (int)(mLabels[mSelIndex]->title).Length()) {
+         right = (mLabels[mSelIndex]->title).Mid(mCurrentCursorPos, (mLabels[mSelIndex]->title).Length()-mCurrentCursorPos);
+      }
+      mLabels[mSelIndex]->title = left + text + right;            
+      mCurrentCursorPos = left.Length() + text.Length();
+   } 
+   else
+   {
+      // insert the data on the clipboard from the cursor position
+      if (mCurrentCursorPos < (int)(mLabels[mSelIndex]->title).Length()) {
+         right = (mLabels[mSelIndex]->title).Mid(mCurrentCursorPos);
+      }
+      mLabels[mSelIndex]->title = (mLabels[mSelIndex]->title).Left(mCurrentCursorPos);
+      mLabels[mSelIndex]->title += text;
+      mLabels[mSelIndex]->title += right;
+      mCurrentCursorPos += text.Length();
+   }
+   // set mInitialCursorPos equal to currentCursorPos
+   mInitialCursorPos = mCurrentCursorPos;
+   return true;
 }
 
 
@@ -953,7 +1007,7 @@ int LabelTrack::OverGlyph(int x, int y)
       pLabel = mLabels[i];
       
       //over left or right selection bound
-      if(   abs(pLabel->y - y) < d1 &&
+      if(   abs(pLabel->y - (y - (LabelTrack::mTextHeight+3)/2)) < d1 &&
             abs(pLabel->x + d2 - x) < d1 )
       {
          mMouseOverLabelLeft = i;
@@ -964,7 +1018,7 @@ int LabelTrack::OverGlyph(int x, int y)
       }
       // use else-if so that we don't detect left and right 
       // of the same label.
-      else if( abs(pLabel->y-y ) < d1 &&
+      else if( abs(pLabel->y - (y - (LabelTrack::mTextHeight+3)/2)) < d1 &&
                abs(pLabel->x1 - d2 -x) < d1)
       {
          mMouseOverLabelRight = i;
@@ -998,16 +1052,27 @@ bool LabelTrack::OverTextBox(const LabelStruct *pLabel, int x, int y)
 
 /// HandleMouse gets called with every mouse move or click.
 /// 
-void LabelTrack::HandleMouse(const wxMouseEvent & evt,
+bool LabelTrack::HandleMouse(const wxMouseEvent & evt,
                              wxRect & r, double h, double pps,
                              double *newSel0, double *newSel1)
 {
-   if(evt.ButtonUp(1) )
+   if(evt.ButtonUp(1))
    {
+      bool lupd = false, rupd = false;
+      if(mIsAdjustingLabel) {
+         if(mMouseOverLabelLeft>=0) {
+            lupd = mLabels[mMouseOverLabelLeft]->updated;
+            mLabels[mMouseOverLabelLeft]->updated = false;
+         }
+         if(mMouseOverLabelRight>=0) {
+            rupd = mLabels[mMouseOverLabelRight]->updated;
+            mLabels[mMouseOverLabelRight]->updated = false;
+         }
+      }
       mIsAdjustingLabel = false;
       mMouseOverLabelLeft  = -1;
       mMouseOverLabelRight = -1;
-      return;
+      return lupd || rupd;
    }
 
    if(evt.Dragging())
@@ -1040,6 +1105,7 @@ void LabelTrack::HandleMouse(const wxMouseEvent & evt,
             {
                mLabels[mMouseOverLabelLeft]->t1  = mLabels[mMouseOverLabelLeft]->t;
             }
+            mLabels[mMouseOverLabelLeft]->updated = true;
          }
          if (mMouseOverLabelRight>=0)
          {
@@ -1048,6 +1114,7 @@ void LabelTrack::HandleMouse(const wxMouseEvent & evt,
             {
                mLabels[mMouseOverLabelRight]->t  = mLabels[mMouseOverLabelRight]->t1;
             }
+            mLabels[mMouseOverLabelRight]->updated = true;
          }
 
          if( mSelIndex >=0 )
@@ -1060,7 +1127,7 @@ void LabelTrack::HandleMouse(const wxMouseEvent & evt,
          SortLabels();
       }
 
-      return;
+      return false;
    }
 
    if( evt.ButtonDown())
@@ -1068,7 +1135,7 @@ void LabelTrack::HandleMouse(const wxMouseEvent & evt,
       //OverGlyph sets mMouseOverLabel to be the chosen label.         
       mIsAdjustingLabel = OverGlyph(evt.m_x, evt.m_y) != 0;   
       
-      // reset mouseXPos if the mouse is pressed in thte text box
+      // reset mouseXPos if the mouse is pressed in the text box
       mMouseXPos = -1;
       mInBox = false;
       bool changeCursor = true;
@@ -1076,7 +1143,7 @@ void LabelTrack::HandleMouse(const wxMouseEvent & evt,
       // reset the highlight indicator
       wxRect highlightedRect;
       if (mSelIndex != -1) {
-         // the rectangle  of highlighted area
+         // the rectangle of highlighted area
          if (mXPos1 < mXPos2)
             highlightedRect = wxRect(mXPos1, mLabels[mSelIndex]->y - mFontHeight/2, (int) (mXPos2-mXPos1+0.5), mFontHeight);
          else
@@ -1099,7 +1166,7 @@ void LabelTrack::HandleMouse(const wxMouseEvent & evt,
          mLabels[mSelIndex]->changeInitialMouseXPos = true;
       }
 
-      if(mIsAdjustingLabel )
+      if (mIsAdjustingLabel)
       {
          float t = 0.0;
          // When we start dragging the label(s) we don't want them to jump.
@@ -1125,15 +1192,15 @@ void LabelTrack::HandleMouse(const wxMouseEvent & evt,
             t = mLabels[mMouseOverLabelLeft]->t;
          }
          mxMouseDisplacement = (int)((((t-h) * pps) + r.x )-evt.m_x);
-         return;
+         return false;
       } 
 
       // disable displaying if left button is down
       if (evt.LeftDown())
          mDragXPos = -1;
-         
+
       // disable displaying if right button is down outside text box
-      if( mSelIndex >0)
+      if(mSelIndex != -1)
       {
          if (evt.RightDown())
          {
@@ -1146,7 +1213,8 @@ void LabelTrack::HandleMouse(const wxMouseEvent & evt,
                changeCursor = false;
          }
       }
-      
+
+      mSelIndex = -1;
       LabelStruct * pLabel;
       for (int i = 0; i < (int)mLabels.Count(); i++) {
          pLabel = mLabels[i];
@@ -1160,35 +1228,66 @@ void LabelTrack::HandleMouse(const wxMouseEvent & evt,
                mMouseXPos = evt.m_x;
             // set mInBox flag
             mInBox = true;
-            return;
+            return false;
          }
       }
    }
+
+   return false;
 }
 
-#ifdef __WXMAC__
- #if ((wxMAJOR_VERSION == 2) && (wxMINOR_VERSION <= 4))
-  // HACK: Workaround wxMac <=2.4 bug
-  bool gMacRepeat = false;
- #endif
+// Check for keys that we will process
+bool LabelTrack::CaptureKey(wxKeyEvent & event)
+{
+   // Cache the keycode
+   int keyCode = event.GetKeyCode();
+   wxChar charCode = keyCode;
+#if wxUSE_UNICODE
+   charCode = event.GetUnicodeKey();
 #endif
 
+   if (mSelIndex >= 0) {
+      switch (keyCode)
+      {
+         case WXK_BACK:
+         case WXK_DELETE:
+         case WXK_HOME:
+         case WXK_END:
+         case WXK_LEFT:
+         case WXK_RIGHT:
+         case WXK_RETURN:
+         case WXK_ESCAPE:
+         case WXK_TAB:
+            return true;
+         break;
+      }
+
+      if (IsGoodLabelCharacter(keyCode, charCode) && !event.CmdDown()) {
+         return true;
+      }
+   }
+
+   if (IsGoodLabelFirstCharacter(keyCode, charCode) && !event.CmdDown()) {
+      return true;
+   }
+
+   return false;
+}
+
 /// KeyEvent is called for every keypress when over the label track.
-/// TODO: Modify so that it can pass unused characters on to 
-/// higher levels of the interface.
-void LabelTrack::KeyEvent(double & newSel0, double & newSel1, wxKeyEvent & event)
-{ 
-#ifdef __WXMAC__
- #if ((wxMAJOR_VERSION == 2) && (wxMINOR_VERSION <= 4))
-  // HACK: Workaround wxMac <=2.4 bug
-   gMacRepeat = !gMacRepeat;
-   if (gMacRepeat)
-      return;
- #endif
+bool LabelTrack::OnKeyDown(double & newSel0, double & newSel1, wxKeyEvent & event)
+{
+   // Only track true changes to the label
+   bool updated = false;
+
+   // Cache the keycode
+   int keyCode = event.GetKeyCode();
+   wxChar charCode = keyCode;
+#if wxUSE_UNICODE
+   charCode = event.GetUnicodeKey();
 #endif
 
-   long keyCode = event.GetKeyCode();
-
+   // All editing keys are only active if we're currently editing a label
    if (mSelIndex >= 0) {
       switch (keyCode) {
 
@@ -1221,6 +1320,7 @@ void LabelTrack::KeyEvent(double & newSel0, double & newSel1, wxKeyEvent & event
                mCurrentCursorPos = 1;
             }
             mInitialCursorPos = mCurrentCursorPos;
+            updated = true;
          }
          break;
 
@@ -1252,6 +1352,33 @@ void LabelTrack::KeyEvent(double & newSel0, double & newSel1, wxKeyEvent & event
                mCurrentCursorPos = 1;
             }
             mInitialCursorPos = mCurrentCursorPos;
+            updated = true;
+         }
+         break;
+
+      case WXK_HOME:
+         // Move cursor to beginning of label
+         mCurrentCursorPos = 0;
+         if (event.ShiftDown()) {
+            mDragXPos = 0;
+         }
+         else
+         {
+            mDragXPos = -1;
+            mInitialCursorPos = mCurrentCursorPos;
+         }
+         break;
+
+      case WXK_END:
+         // Move cursor to end of label
+         mCurrentCursorPos = (int)mLabels[mSelIndex]->title.length();
+         if (event.ShiftDown()) {
+            mDragXPos = 0;
+         }
+         else
+         {
+            mDragXPos = -1;
+            mInitialCursorPos = mCurrentCursorPos;
          }
          break;
 
@@ -1287,7 +1414,6 @@ void LabelTrack::KeyEvent(double & newSel0, double & newSel1, wxKeyEvent & event
 
       case WXK_RETURN:
 
-
       case WXK_ESCAPE:
          if (mLabels[mSelIndex]->title == wxT("")) {
             delete mLabels[mSelIndex];
@@ -1299,56 +1425,106 @@ void LabelTrack::KeyEvent(double & newSel0, double & newSel1, wxKeyEvent & event
          
       case WXK_TAB:
          if (event.ShiftDown()) {
-            if (mSelIndex > 0)
                mSelIndex--;
          } else {
-            if (mSelIndex < (int)mLabels.Count() - 1) 
                mSelIndex++;
          }
          
+         if (mSelIndex >= 0 && mSelIndex < (int)mLabels.Count()) {
+            mCurrentCursorPos = mLabels[mSelIndex]->title.Length();
+            //Set the selection region to be equal to the selection bounds of the tabbed-to label.
+            newSel0 = mLabels[mSelIndex]->t;
+            newSel1 = mLabels[mSelIndex]->t1;
+         }
+         else {
+            mSelIndex = -1;
+         }
+         break;
+
+      default:
+         event.Skip();
+         break;
+      }
+   }
+   else
+   {
+      switch (keyCode) {
+
+      case WXK_TAB:
+         if (event.ShiftDown()) {
+            mSelIndex = (int)mLabels.Count() - 1;
+         } else {
+            mSelIndex = 0;
+         }
+         
          mCurrentCursorPos = mLabels[mSelIndex]->title.Length();
          //Set the selection region to be equal to the selection bounds of the tabbed-to label.
          newSel0 = mLabels[mSelIndex]->t;
          newSel1 = mLabels[mSelIndex]->t1;
          break;
-      
-      default:
-         if( IsGoodLabelCharacter( keyCode )) {
-            // Test if cursor is in the end of string or not
-            if (mLabels[mSelIndex]->highlighted) {
-               RemoveSelectedText();
-            }
 
-            if (mCurrentCursorPos < (int)mLabels[mSelIndex]->title.length()) {
-               // Get substring on the righthand side of cursor
-               wxString rightPart = mLabels[mSelIndex]->title.Mid(mCurrentCursorPos);
-               // Set title to substring on the lefthand side of cursor
-               mLabels[mSelIndex]->title = mLabels[mSelIndex]->title.Left(mCurrentCursorPos);
-               //append keycode
-               mLabels[mSelIndex]->title += keyCode;
-               //append the right part substring
-               mLabels[mSelIndex]->title += rightPart;
-            }
-            else
-            {
-               //append keycode
-               mLabels[mSelIndex]->title += keyCode;
-            }
-            //moving cursor position forward
-            mCurrentCursorPos++;
-            mInitialCursorPos = mCurrentCursorPos;
-         }
+      default:
+         event.Skip();
          break;
       }
-   } 
-   else if( !IsGoodLabelFirstCharacter( keyCode ))
+   }
+
+   // Make sure the caret is visible
+   mDrawCursor = true;
+
+   return updated;
+}
+
+/// KeyEvent is called for every keypress when over the label track.
+bool LabelTrack::OnChar(double & newSel0, double & newSel1, wxKeyEvent & event)
+{
+   // Only track true changes to the label
+   bool updated = false;
+
+   // Cache the keycode
+   int keyCode = event.GetKeyCode();
+   wxChar charCode = keyCode;
+#if wxUSE_UNICODE
+   charCode = event.GetUnicodeKey();
+#endif
+
+   if (mSelIndex >= 0) {
+      if (IsGoodLabelCharacter(keyCode, charCode)) {
+         // Test if cursor is in the end of string or not
+         if (mLabels[mSelIndex]->highlighted) {
+            RemoveSelectedText();
+         }
+
+         if (mCurrentCursorPos < (int)mLabels[mSelIndex]->title.length()) {
+            // Get substring on the righthand side of cursor
+            wxString rightPart = mLabels[mSelIndex]->title.Mid(mCurrentCursorPos);
+            // Set title to substring on the lefthand side of cursor
+            mLabels[mSelIndex]->title = mLabels[mSelIndex]->title.Left(mCurrentCursorPos);
+            //append charcode
+            mLabels[mSelIndex]->title += charCode;
+            //append the right part substring
+            mLabels[mSelIndex]->title += rightPart;
+         }
+         else
+         {
+            //append charCode
+            mLabels[mSelIndex]->title += charCode;
+         }
+         //moving cursor position forward
+         mCurrentCursorPos++;
+         mInitialCursorPos = mCurrentCursorPos;
+         updated = true;
+      }
+      else
+         event.Skip();
+   }
+   else if( !IsGoodLabelFirstCharacter(keyCode, charCode))
    {
       // Don't automatically start a label unless its one of the accepted 
       // characters.
       // We can always create the label and then type the forbidden character
       // as our first character.
-      // TODO: (Possibly) pass the unused characters on to other keycode
-      // handlers, 
+      event.Skip();
    }
    else
    {
@@ -1357,7 +1533,7 @@ void LabelTrack::KeyEvent(double & newSel0, double & newSel1, wxKeyEvent & event
       LabelStruct *l = new LabelStruct();
       l->t =  newSel0;
       l->t1 = newSel1;
-      l->title += wxChar(keyCode);
+      l->title += wxChar(charCode);
       mCurrentCursorPos = 1;
       mInitialCursorPos = mCurrentCursorPos;
       
@@ -1370,7 +1546,13 @@ void LabelTrack::KeyEvent(double & newSel0, double & newSel1, wxKeyEvent & event
       mLabels.Insert(l, pos);
       
       mSelIndex = pos;
+      updated = true;
    }
+
+   // Make sure the caret is visible
+   mDrawCursor = true;
+
+   return updated;
 }
 
 void LabelTrack::RemoveSelectedText() 
@@ -1514,6 +1696,7 @@ bool LabelTrack::HandleXMLTag(const wxChar *tag, const wxChar **attrs)
       // loop through attrs, which is a null-terminated list of
       // attribute-value pairs
       bool has_t1 = false;
+      double dblValue;
       while(*attrs) {
          const wxChar *attr = *attrs++;
          const wxChar *value = *attrs++;
@@ -1521,14 +1704,22 @@ bool LabelTrack::HandleXMLTag(const wxChar *tag, const wxChar **attrs)
          if (!value)
             break;
          
-         if (!wxStrcmp(attr, wxT("t")))
-            Internat::CompatibleToDouble(value, &l->t);
-         else if (!wxStrcmp(attr, wxT("t1"))) {
+         const wxString strValue = value;
+         if (!XMLValueChecker::IsGoodString(strValue))
+         {
+            delete l;
+            return false;
+         }
+        
+         if (!wxStrcmp(attr, wxT("t")) && Internat::CompatibleToDouble(strValue, &dblValue))
+            l->t = dblValue;
+         else if (!wxStrcmp(attr, wxT("t1")) && Internat::CompatibleToDouble(strValue, &dblValue))
+         {
             has_t1 = true;
-            Internat::CompatibleToDouble(value, &l->t1);
+            l->t1 = dblValue;
          }
          else if (!wxStrcmp(attr, wxT("title")))
-            l->title = value;
+            l->title = strValue;
 
       } // while
 
@@ -1542,19 +1733,27 @@ bool LabelTrack::HandleXMLTag(const wxChar *tag, const wxChar **attrs)
       return true;
    }
    else if (!wxStrcmp(tag, wxT("labeltrack"))) {
-      if (*attrs) {
+      long nValue = -1;
+      while (*attrs) {
          const wxChar *attr = *attrs++;
          const wxChar *value = *attrs++;
          
          if (!value)
             return true;
 
-         if (!wxStrcmp(attr, wxT("name")))
-            mName = value;
-         else if (!wxStrcmp(attr, wxT("numlabels"))) {
-            int len = wxAtoi(value);
+         const wxString strValue = value;
+         if (!wxStrcmp(attr, wxT("name")) && XMLValueChecker::IsGoodString(strValue))
+            mName = strValue;
+         else if (!wxStrcmp(attr, wxT("numlabels")) && 
+                     XMLValueChecker::IsGoodInt(strValue) && strValue.ToLong(&nValue)) 
+         {
+            if (nValue < 0)
+            {
+               wxLogWarning(wxT("Project shows negative number of labels: %d"), nValue);
+               return false;
+            }
             mLabels.Clear();
-            mLabels.Alloc(len);
+            mLabels.Alloc(nValue);
          }
       }
 
@@ -1572,28 +1771,24 @@ XMLTagHandler *LabelTrack::HandleXMLChild(const wxChar *tag)
       return NULL;
 }
 
-void LabelTrack::WriteXML(int depth, FILE *fp)
+void LabelTrack::WriteXML(XMLWriter &xmlFile)
 {
    int len = mLabels.Count();
-   int i, j;
+   int i;
 
-   for(j=0; j<depth; j++)
-      fprintf(fp, "\t");
-   fprintf(fp, "<labeltrack ");
-   fprintf(fp, "name=\"%s\" ", (const char *)XMLEsc(mName).mb_str());
-   fprintf(fp, "numlabels=\"%d\">\n", len);
+   xmlFile.StartTag(wxT("labeltrack"));
+   xmlFile.WriteAttr(wxT("name"), mName);
+   xmlFile.WriteAttr(wxT("numlabels"), len);
 
    for (i = 0; i < len; i++) {
-      for(j=0; j<depth+1; j++)
-         fprintf(fp, "\t");
-      fprintf(fp, "<label t=\"%s\" t1=\"%s\" title=\"%s\"/>\n",
-            (const char *)Internat::ToString(mLabels[i]->t, 8).mb_str(),
-            (const char *)Internat::ToString(mLabels[i]->t1, 8).mb_str(),
-            (const char *)XMLEsc(mLabels[i]->title).mb_str());
+      xmlFile.StartTag(wxT("label"));
+      xmlFile.WriteAttr(wxT("t"), mLabels[i]->t, 8);
+      xmlFile.WriteAttr(wxT("t1"), mLabels[i]->t1, 8);
+      xmlFile.WriteAttr(wxT("title"), mLabels[i]->title);
+      xmlFile.EndTag(wxT("label"));
    }
-   for(j=0; j<depth; j++)
-      fprintf(fp, "\t");
-   fprintf(fp, "</labeltrack>\n");
+
+   xmlFile.EndTag(wxT("labeltrack"));
 }
 
 #if LEGACY_PROJECT_FILE_SUPPORT
@@ -1787,6 +1982,8 @@ int LabelTrack::AddLabel(double t, double t1, const wxString &title)
    l->t = t;
    l->t1 = t1;
    l->title = title;
+   mCurrentCursorPos = title.length();
+   mInitialCursorPos = mCurrentCursorPos;
 
    int len = mLabels.Count();
    int pos = 0;
@@ -1798,28 +1995,30 @@ int LabelTrack::AddLabel(double t, double t1, const wxString &title)
 
    mSelIndex = pos;
 
+   // Make sure the caret is visible
+   //
+   // LLL: The cursor will not be drawn when the first label
+   //      is added since mDrawCursor will be false.  Presumably,
+   //      if the user adds a label, then a cursor should be drawn
+   //      to indicate that typing is expected.
+   //
+   //      If the label is added during actions like import, then the
+   //      mDrawCursor flag will be reset once the action is complete.
+   mDrawCursor = true;
+
    return pos;
 }
 
-// Private method called from the constructor
-void LabelTrack::InitColours()
+void LabelTrack::DeleteLabel(int index)
 {
-// JKC: Standard Audacity colours are grays and shades of blue.
-// To keep interface 'unified' only use special colors
-// where there is a good/established reason, e.g.
-//   Red   - Record.
-//   Green - Playback.
-   mLabelSurroundPen.SetColour( 0, 0, 0);
-   mTextEditBrush.SetColour( 255,255,255);
-   mTextNormalBrush.SetColour( 190,190,240 );// #BEBEF0
-
-   mUnselectedBrush.SetColour(192, 192, 192);
-   mSelectedBrush.SetColour(148, 148, 170);
-
-   mUnselectedPen.SetColour(192, 192, 192);
-   mSelectedPen.SetColour(148, 148, 170);
+   wxASSERT((index < (int)mLabels.GetCount()));
+   mLabels.RemoveAt(index);
 }
 
+wxBitmap & LabelTrack::GetGlyph( int i)
+{
+   return theTheme.Bitmap( i + bmpLabelGlyph0);
+}
 
 // This one XPM spec is used to generate a number of
 // different wxIcons.
@@ -1828,7 +2027,7 @@ static char *GlyphXpmRegionSpec[] = {
 /* columns rows colors chars-per-pixel */
 "15 23 7 1",
 /* Default colors, with first color transparent */
-". c #C0C0C0",
+". c none",
 "2 c black",
 "3 c black",
 "4 c black",
@@ -1890,12 +2089,7 @@ void LabelTrack::CreateCustomGlyphs()
    if( mbGlyphsReady )
       return;
 
-// KLUDGE!  Color #C0C0C0 is transparent for icons in wxWindows 2.4.0
-// Might not be that way in future, in which case we might need to
-// explicitly create a mask for the bitmap dependent on the colors before
-// creating the icon.
-
-// We're about to tweak the basic color spec to get 12 variations.
+   // We're about to tweak the basic color spec to get 12 variations.
    for( iConfig=0;iConfig<NUM_GLYPH_CONFIGS;iConfig++)
    {
       for( iHighlight=0;iHighlight<NUM_GLYPH_HIGHLIGHTS;iHighlight++)
@@ -1909,10 +2103,12 @@ void LabelTrack::CreateCustomGlyphs()
          if( iHighlight==3 ) XmpBmp[7]="7 c #FFFFFF";
          // For left or right arrow the other side of the glyph
          // is the transparent color.
-         if( iConfig==0) { XmpBmp[3]="3 c #C0C0C0"; XmpBmp[5]="5 c #C0C0C0"; }
-         if( iConfig==1) { XmpBmp[4]="4 c #C0C0C0"; XmpBmp[7]="7 c #C0C0C0"; }
+         if( iConfig==0) { XmpBmp[3]="3 c none"; XmpBmp[5]="5 c none"; }
+         if( iConfig==1) { XmpBmp[4]="4 c none"; XmpBmp[7]="7 c none"; }
          // Create the icon from the tweaked spec.
-         mBoundaryGlyphs[index] = wxIcon(XmpBmp);
+         mBoundaryGlyphs[index] = wxBitmap(XmpBmp);
+         // Create the mask
+         mBoundaryGlyphs[index].SetMask(new wxMask(mBoundaryGlyphs[index], wxColour(192, 192, 192)));
       }
    }
 
@@ -1927,27 +2123,19 @@ void LabelTrack::CreateCustomGlyphs()
 }
 
 /// The 'typing at selection creates label' feature
-/// will only accept these keyCodes as a first label 
+/// will only accept these charCodes as a first label 
 /// character.
-bool LabelTrack::IsGoodLabelFirstCharacter(long keyCode)
+bool LabelTrack::IsGoodLabelFirstCharacter(int keyCode, wxChar charCode)
 {
-   if(( 'a' <= keyCode ) && (keyCode <='z' ))
-      return true;
-   if(( 'A' <= keyCode ) && (keyCode <='Z' ))
-      return true;
-   if(( '0' <= keyCode ) && (keyCode <='9' ))
-      return true;
-   return false;
+   return (keyCode < WXK_START && !wxIscntrl(charCode) && (keyCode != WXK_SPACE)) ||
+          keyCode > WXK_COMMAND;
 }
 
 /// We'll only accept these characters within a label
-bool LabelTrack::IsGoodLabelCharacter(long keyCode)
+bool LabelTrack::IsGoodLabelCharacter(int keyCode, wxChar charCode)
 {
-// if( IsGoodLabelFirstCharacter( keyCode ))
-//    return true;
-   if( keyCode >= WXK_START )
-      return false;
-   return true;
+   return (keyCode < WXK_START && !wxIscntrl(charCode)) ||
+          keyCode > WXK_COMMAND;
 }
 
 /// Sorts the labels in order of their starting times.
