@@ -73,10 +73,6 @@ const int MAX_NUM_ROWS =80;
 
 class LabelTrack:public Track {
    friend class LabelStruct;
-   friend class BouncePane;
-   friend bool ExportPCM(AudacityProject *project,
-               wxString format, bool stereo, wxString fName,
-               bool selectionOnly, double t0, double t1);
 
  public:
 	bool IsGoodLabelCharacter(int keyCode, wxChar charCode);
@@ -165,6 +161,7 @@ class LabelTrack:public Track {
    void DeleteLabel(int index);
 
    //get current cursor position
+   bool CalcCursorX(wxWindow * parent, int * x);
    int getCurrentCursorPosition() const { return mCurrentCursorPos; };
 
  public:
@@ -182,15 +179,6 @@ class LabelTrack:public Track {
    int mxMouseDisplacement;    /// Displacement of mouse cursor from the centre being dragged.
    LabelArray mLabels;
 
-   wxBrush mUnselectedBrush;
-   wxBrush mSelectedBrush;
-   wxBrush mTextNormalBrush;
-   wxBrush mTextEditBrush;
-
-   wxPen mLabelSurroundPen;
-   wxPen mUnselectedPen;
-   wxPen mSelectedPen;
-
    static int mIconHeight;
    static int mIconWidth;
    static int mTextHeight;
@@ -214,7 +202,6 @@ class LabelTrack:public Track {
    // Used only for a LabelTrack on the clipboard
    double mClipLen;
 
-   void InitColours();
    void ComputeLayout(wxRect & r, double h, double pps);
    void ComputeTextPosition(wxRect & r, int index);
    void SetCurrentCursorPosition(wxDC & dc, int xPos);
