@@ -18,7 +18,6 @@ void CommandManagerListCallback(void *fptr, int index);
 void CreateMenusAndCommands();
 
 void CreateRecentFilesMenu(CommandManager *c);
-void ModifyExportMenus();
 void ModifyUndoMenus();
 void ModifyToolbarMenus();
 
@@ -125,19 +124,9 @@ void OnSaveAs();
 
 void OnCheckDependencies();
 
-void OnExportMix();
+void OnExport();
 void OnExportSelection();
-void OnExportMP3Mix();
-void OnExportMP3Selection();
-void OnExportOggMix();
-void OnExportOggSelection();
-void OnExportFLACMix();
-void OnExportFLACSelection();
-void OnExportMP2Mix();
-void OnExportMP2Selection();
-
 void OnExportMultiple();
-
 void OnExportLabels();
 
 void OnUpload();
@@ -169,6 +158,7 @@ void OnSplitDelete();
 void OnSilence();
 
 void OnSplit();
+void OnSplitNew();
 void OnSplitLabelsToTracks();
 void OnJoin();
 void OnDisjoin();
@@ -185,6 +175,7 @@ void OnJoinLabels();
 void OnDisjoinLabels();
 
 void OnSelectAll();
+void OnSelectNone();
 void OnSelectCursorEnd();
 void OnSelectStartCursor();
 
@@ -216,6 +207,7 @@ void OnShowMixerToolBar();
 void OnShowSelectionToolBar();
 void OnShowToolsToolBar();
 void OnShowTranscriptionToolBar();
+void OnResetToolBars();
 
 
         // Project Menu
@@ -279,10 +271,8 @@ int  mStereoToMonoIndex;
         // Help Menu
 
 void OnAbout();
-void OnHelp();
-void OnHelpIndex();
-void OnHelpSearch();
 void OnBenchmark();
+void OnScreenshot();
 
        //
 
