@@ -179,7 +179,7 @@ void InitPreferences()
       gPrefs->Write(wxT("/NewPrefsInitialized"), true);
    }
    
-   gPrefs->Write(wxT("/Version"), wxString(wxT(AUDACITY_VERSION_STRING)));
+   gPrefs->Write(wxT("/Version"), wxString(AUDACITY_VERSION_STRING));
 
    // BG: Make sure the users prefs are up to date
    // BG: Otherwise reset some of them to their defaults
@@ -206,17 +206,6 @@ void FinishPreferences()
    }
 }
 
-int ReadExportFormatPref()
-{
-   return gPrefs->Read(wxT("/FileFormats/ExportFormat_SF1"),
-                       (long int)(SF_FORMAT_WAV | SF_FORMAT_PCM_16));
-}
-
-void WriteExportFormatPref(int format)
-{
-   gPrefs->Write(wxT("/FileFormats/ExportFormat_SF1"), (long int)format);
-}
-
 // Indentation settings for Vim and Emacs and unique identifier for Arch, a
 // version control system. Please do not modify past this point.
 //
