@@ -98,6 +98,7 @@ for registering for changes.
 #include <wx/wx.h>
 #include <wx/wxprec.h>
 #include <wx/listctrl.h>
+#include <wx/notebook.h>
 #include <wx/treectrl.h>
 #include "Internat.h"
 #include "Experimental.h"
@@ -361,7 +362,7 @@ void ShuttleGuiBase::AddFixedText(const wxString &Str, bool bCenter)
       UpdateSizers();
 }
 
-wxStaticText * ShuttleGuiBase::AddVariableText(const wxString &Str, bool bCenter)
+wxStaticText * ShuttleGuiBase::AddVariableText(const wxString &Str, bool bCenter, int PositionFlags)
 {
    UseUpId();
    if( mShuttleMode != eIsCreating )
@@ -373,10 +374,16 @@ wxStaticText * ShuttleGuiBase::AddVariableText(const wxString &Str, bool bCenter
    if( bCenter )
    {
       miProp=1;
-      UpdateSizersC();
+      if( PositionFlags )
+         UpdateSizersCore( false, PositionFlags );
+      else
+         UpdateSizersC();
    }
    else
-      UpdateSizers();
+      if( PositionFlags )
+         UpdateSizersCore( false, PositionFlags );
+      else
+         UpdateSizers();
    return pStatic;
 }
 
@@ -1307,33 +1314,39 @@ void ShuttleGuiBase::TieCheckBox(
 
 /// Variant of the standard TieTextBox which does the two step exchange 
 /// between gui and stack variable and stack variable and shuttle.
-void ShuttleGuiBase::TieTextBox(
+wxTextCtrl * ShuttleGuiBase::TieTextBox(
    const wxString & Prompt, 
    const wxString & SettingName, 
    const wxString & Default,
    const int nChars)
 {
+   wxTextCtrl * pText=(wxTextCtrl*)NULL;
+
    wxString Temp = Default;
    WrappedType WrappedRef( Temp );
    if( DoStep(1) ) DoDataShuttle( SettingName, WrappedRef );
-   if( DoStep(2) ) TieTextBox( Prompt, WrappedRef, nChars );
+   if( DoStep(2) ) pText = TieTextBox( Prompt, WrappedRef, nChars );
    if( DoStep(3) ) DoDataShuttle( SettingName, WrappedRef );
+   return pText;
 }
 
 /// Variant of the standard TieTextBox which does the two step exchange 
 /// between gui and stack variable and stack variable and shuttle.
 /// This one does it for double values...
-void ShuttleGuiBase::TieTextBox(
+wxTextCtrl * ShuttleGuiBase::TieTextBox(
    const wxString & Prompt, 
    const wxString & SettingName, 
    const double & Default,
    const int nChars)
 {
+   wxTextCtrl * pText=(wxTextCtrl*)NULL;
+
    double Temp = Default;
    WrappedType WrappedRef( Temp );
    if( DoStep(1) ) DoDataShuttle( SettingName, WrappedRef );
-   if( DoStep(2) ) TieTextBox( Prompt, WrappedRef, nChars );
+   if( DoStep(2) ) pText = TieTextBox( Prompt, WrappedRef, nChars );
    if( DoStep(3) ) DoDataShuttle( SettingName, WrappedRef );
+   return pText;
 }
 
 /// Variant of the standard TieChoice which does the two step exchange 
@@ -1421,16 +1434,16 @@ void ShuttleGuiBase::StartRadioButtonGroup( const wxString & SettingName, const
 
 
 /// This function must be within a StartRadioButtonGroup - EndRadioButtonGroup pair.
-void ShuttleGuiBase::TieRadioButton( 
+wxRadioButton * ShuttleGuiBase::TieRadioButton( 
    const wxString &Prompt, 
    int iValue)
 {
    WrappedType WrappedRef( iValue );
-   TieRadioButton( Prompt, WrappedRef );
+   return TieRadioButton( Prompt, WrappedRef );
 }
 
 /// This function must be within a StartRadioButtonGroup - EndRadioButtonGroup pair.
-void ShuttleGuiBase::TieRadioButton( 
+wxRadioButton * ShuttleGuiBase::TieRadioButton( 
    const wxString &Prompt, 
    const wxString &Value)
 {
@@ -1438,7 +1451,7 @@ void ShuttleGuiBase::TieRadioButton(
    // don't have a 'read-only' version, so we copy to deal with the constness.
    wxString Temp = Value;
    WrappedType WrappedRef( Temp ); 
-   TieRadioButton( Prompt, WrappedRef );
+   return TieRadioButton( Prompt, WrappedRef );
 }
 
 //------------------------------------------------------------------//
