@@ -227,7 +227,6 @@ void Theme::RegisterImages()
    if( mbInitialised )
       return;
    mbInitialised = true;
-   int i=0;
 
 // This initialises the variables e.g
 // RegisterImage( bmpRecordButton, some image, wxT("RecordButton"));
@@ -334,7 +333,7 @@ wxImage ThemeBase::MaskedImage( char const ** pXpm, char const ** pMask )
    wxImage Img1( Bmp1.ConvertToImage());
    wxImage Img2( Bmp2.ConvertToImage());
 
-   unsigned char *src = Img1.GetData();
+//   unsigned char *src = Img1.GetData();
    unsigned char *mk = Img2.GetData();
    unsigned char *alpha = (unsigned char*)malloc( nBytes );
 
@@ -617,8 +616,7 @@ void ThemeBase::CreateImageCache( bool bBinarySave )
       {
          wxMessageBox(
             wxString::Format( 
-            _("Theme cache file:\r\n  %s\r\nalready exists.\r\n"
-               wxT("Are you sure you want to replace it?")),
+            wxT("Theme cache file:\n  %s\nalready exists.\nAre you sure you want to replace it?"),
                FileName.c_str() ));
          return;
       }
@@ -627,13 +625,13 @@ void ThemeBase::CreateImageCache( bool bBinarySave )
       {
          wxMessageBox(
             wxString::Format( 
-            _("Audacity could not write file:\r\n  %s."),
+            _("Audacity could not write file:\n  %s."),
                FileName.c_str() ));
          return;
       }
       wxMessageBox(
          wxString::Format( 
-            wxT("Theme written to:\r\n  %s."),
+            wxT("Theme written to:\n  %s."),
             FileName.c_str() ));
    }
    // ELSE saving to a C code textual version.
@@ -645,7 +643,7 @@ void ThemeBase::CreateImageCache( bool bBinarySave )
       {
          wxMessageBox(
             wxString::Format( 
-            _("Audacity could not open file:\r\n  %s\r\nfor writing."),
+            _("Audacity could not open file:\n  %s\nfor writing."),
             FileName.c_str() ));
          return;
       }
@@ -653,13 +651,13 @@ void ThemeBase::CreateImageCache( bool bBinarySave )
       {
          wxMessageBox(
             wxString::Format( 
-            _("Audacity could not write images to file:\r\n  %s."),
+            _("Audacity could not write images to file:\n  %s."),
             FileName.c_str() ));
          return;
       }
       wxMessageBox(
          wxString::Format( 
-            wxT("Theme as Cee code written to:\r\n  %s."),
+            wxT("Theme as Cee code written to:\n  %s."),
             FileName.c_str() ));
    }
 }
@@ -788,7 +786,7 @@ bool ThemeBase::ReadImageCache( bool bBinaryRead, bool bOkIfNotFound)
             return false; // did not load the images, so return false.
          wxMessageBox(
             wxString::Format( 
-            _("Audacity could not find file:\r\n  %s.\r\nTheme not loaded."),
+            _("Audacity could not find file:\n  %s.\nTheme not loaded."),
                FileName.c_str() ));
          return false;
       }
@@ -796,7 +794,7 @@ bool ThemeBase::ReadImageCache( bool bBinaryRead, bool bOkIfNotFound)
       {
          wxMessageBox(
             wxString::Format( 
-            _("Audacity could not load file:\r\n  %s.\r\nBad png format perhaps?"),
+            _("Audacity could not load file:\n  %s.\nBad png format perhaps?"),
                FileName.c_str() ));
          return false;
       }
@@ -812,12 +810,7 @@ bool ThemeBase::ReadImageCache( bool bBinaryRead, bool bOkIfNotFound)
          // was not a valid png image.
          // Most likely someone edited it by mistake, 
          // Or some experiment is being tried with new formats for it.
-         wxMessageBox(
-            wxString::Format( 
-               _("Audacity could not read its default theme.\r\n"
-               wxT("We're not sure how this happened.\r\n")
-               wxT("Please report the problem.")))
-               );
+         wxMessageBox(_("Audacity could not read its default theme.\nPlease report the problem."));
          return false;
       }
    }
@@ -889,7 +882,7 @@ void ThemeBase::LoadComponents( bool bOkIfNotFound )
             {
                wxMessageBox(
                   wxString::Format( 
-                  _("Audacity could not load file:\r\n  %s.\r\nBad png format perhaps?"),
+                  _("Audacity could not load file:\n  %s.\nBad png format perhaps?"),
                      FileName.c_str() ));
                return;
             }
@@ -911,11 +904,8 @@ void ThemeBase::LoadComponents( bool bOkIfNotFound )
    {
       if( bOkIfNotFound )
          return;
-      wxMessageBox(
-         wxString::Format( 
-            _("None of the expected theme component files\r\n"
-            wxT("were found in:\r\n  %s.")),
-            FileNames::ThemeComponentsDir().c_str() ));
+      wxMessageBox(wxString::Format(_("None of the expected theme component files\n were found in:\n  %s."),
+                                    FileNames::ThemeComponentsDir().c_str()));
    }
 }
 
@@ -938,7 +928,7 @@ void ThemeBase::SaveComponents()
       {
          wxMessageBox(
             wxString::Format( 
-            _("Could not create directory:\r\n  %s"),
+            _("Could not create directory:\n  %s"),
                FileNames::ThemeComponentsDir().c_str() ));
          return;
       }
@@ -959,7 +949,7 @@ void ThemeBase::SaveComponents()
             {
                wxMessageBox(
                   wxString::Format( 
-                  _("Audacity could not save file:\r\n  %s"),
+                  _("Audacity could not save file:\n  %s"),
                      FileName.c_str() ));
                return;
             }
@@ -971,13 +961,13 @@ void ThemeBase::SaveComponents()
    {
       wxMessageBox(
          wxString::Format( 
-         _("All required files in:\r\n  %s\r\nwere already present."),
+         _("All required files in:\n  %s\nwere already present."),
             FileNames::ThemeComponentsDir().c_str() ));
       return;
    }
    wxMessageBox(
       wxString::Format( 
-         wxT("Theme written to:\r\n  %s."),
+         wxT("Theme written to:\n  %s."),
          FileNames::ThemeComponentsDir().c_str() ));
 }
 
