@@ -32,6 +32,7 @@ class TrackArtist;
 class WaveTrack;
 class LabelTrack;
 class Ruler;
+class SnapManager;
 class AdornedRulerPanel;
 class LWSlider;
 class ControlToolBar; //Needed because state of controls can affect what gets drawn.
@@ -193,6 +194,7 @@ class TrackPanel:public wxPanel {
    void OnCursorLeft(bool shift, bool ctrl);
    void OnCursorRight(bool shift, bool ctrl);
    void ScrollIntoView(double pos);
+   void ScrollIntoView(int x);
 
    void OnTrackPan();
    void OnTrackPanLeft();
@@ -234,11 +236,12 @@ class TrackPanel:public wxPanel {
 
    // AS: Selection handling
    void HandleSelect(wxMouseEvent & event);
-   void SelectionHandleDrag(wxMouseEvent &event);
+   void SelectionHandleDrag(wxMouseEvent &event, Track *pTrack);
    void SelectionHandleClick(wxMouseEvent &event, 
 			     Track* pTrack, wxRect r, int num);
-   void StartSelection (int, int);
-   void ExtendSelection(int, int);
+   void StartSelection (int mouseXCoordinate, int trackLeftEdge);
+   void ExtendSelection(int mouseXCoordinate, int trackLeftEdge,
+                        Track *pTrack);
    void SelectTracksByLabel( LabelTrack *t );
 
 
@@ -368,6 +371,7 @@ private:
                     const int vrul, const wxRect trackRect, int index);
    void DrawZooming(wxDC* dc, const wxRect clip);
 
+   void HighlightFocusedTrack (wxDC* dc, const wxRect r);
    void DrawShadow            (Track *t, wxDC* dc, const wxRect r);
    void DrawBordersAroundTrack(Track *t, wxDC* dc, const wxRect r, const int labelw, const int vrul);
    void DrawOutsideOfTrack    (Track *t, wxDC* dc, const wxRect r);
@@ -464,17 +468,27 @@ private:
    int mZoomStart;
    int mZoomEnd;
 
+   // Handles snapping the selection boundaries or track boundaries to
+   // line up with existing tracks or labels.  mSnapLeft and mSnapRight
+   // are the horizontal index of pixels to display user feedback
+   // guidelines so the user knows when such snapping is taking place.
+   SnapManager *mSnapManager;
+   wxInt64 mSnapLeft;
+   wxInt64 mSnapRight;
+   bool mSnapPreferRightEdge;
+
    Track * mDrawingTrack;          // Keeps track of which track you are drawing on between events cf. HandleDraw()
    int mDrawingTrackTop;           // Keeps track of the top position of the drawing track.
    sampleCount mDrawingStartSample;   // sample of last click-down
    float mDrawingStartSampleValue;    // value of last click-down
    sampleCount mDrawingLastDragSample; // sample of last drag-over
    float mDrawingLastDragSampleValue;  // value of last drag-over
+   wxRect mLastDrawnTrackRect;
  
-   double PositionToTime(int mouseXCoordinate,
-                         int trackLeftEdge) const;
-   int TimeToPosition(double time,
-                      int trackLeftEdge) const;
+   double PositionToTime(wxInt64 mouseXCoordinate,
+                         wxInt64 trackLeftEdge) const;
+   wxInt64 TimeToPosition(double time,
+                          wxInt64 trackLeftEdge) const;
 
    int mInitialTrackHeight;
    int mInitialUpperTrackHeight;
@@ -549,6 +563,11 @@ private:
 
    TrackPanelAx *mAx;
 
+ private:
+
+   // The screenshot class needs to access internals
+   friend class ScreenFrame;
+
  public:
 
    DECLARE_EVENT_TABLE()
