@@ -4,13 +4,20 @@
 
   UploadDialog.cpp
 
-  Mike Underwood
-  munderwood@bedheaddesign.com
+  Mike Underwood - munderwood@bedheaddesign.com
 
   Much like the author, this code is crazy at times but it gets the job
   done.
 
-**********************************************************************/
+*******************************************************************//**
+
+\class UploadDialog
+\brief A contributed class for uploading audio via FTP.
+
+*//*******************************************************************/
+
+#include "Audacity.h"
+
 #include <fstream>
 #include <math.h>
 #include <wx/dialog.h>
@@ -36,9 +43,16 @@
 #endif
 
 #include "UploadDialog.h"
-#include "Audacity.h"
+#include "PlatformCompatibility.h"
+#include "FileNames.h"
+#include "Theme.h"
+#include "AllThemeResources.h"
+#include "Project.h"
+
+#include "FileDialog.h"
 
 //#include "../images/AudacityLogo.xpm"
+//#include "../images/UploadImages.h"
 
 #ifdef __WXMSW__
 # define DLOG_HEIGHT 430
@@ -52,156 +66,7 @@
 
 // Icon images
 
-static const char *file_xpm[] = {
-/* columns rows colors chars-per-pixel */
-"16 16 4 1",
-"  c #040404",
-". c #868686",
-"X c #C0C0C0",
-"o c gray100",
-/* pixels */
-"oo..........oooo",
-"oo.ooooooooX.ooo",
-"oo.ooooooooXo.oo",
-"oo.oooooooo    o",
-"oo.ooooooooooX o",
-"oo.ooooooooooX o",
-"oo.ooooooooooX o",
-"oo.ooooooooooX o",
-"oo.ooooooooooX o",
-"oo.ooooooooooX o",
-"oo.ooooooooooX o",
-"oo.ooooooooooX o",
-"oo.ooooooooooX o",
-"oo.ooooooooooX o",
-"oo.XXXXXXXXXXX o",
-"oo             o"
-};
-
-static const char *folder_xpm[] = {
-/* columns rows colors chars-per-pixel */
-"16 16 15 1",
-"  c gray30",
-". c gray40",
-"X c #996600",
-"o c #CC6633",
-"O c #CC9933",
-"+ c #CCCC33",
-"@ c #FFCC66",
-"# c #868686",
-"$ c #FFCC99",
-"% c #FFFF99",
-"& c gray80",
-"* c #D7D7D7",
-"= c #DDDDDD",
-"- c #F1F1F1",
-"; c gray100",
-/* pixels */
-";;;;;;;;;;;;;;;;",
-";;;;;;;;;;;;;;;;",
-";;OOOO=-;;;;;;;;",
-";O;;;;O&-;;;;;;;",
-"O;%%%%;OOOOOo=-;",
-"O%%%%%%;;;;;X#=;",
-"O%$OOOOOOOOOOOO=",
-"O@O;;;;;;;;;$;X#",
-"O@O%%%%%%%%%@%X.",
-"O@O%%%%%%%%%@%X.",
-"O@O%%%%%%%%%+%X.",
-"O@O%@@@@@@@@O%X.",
-"O@O%@@@@@@@@O%X.",
-"=OOoXXXXXXXXXX #",
-"-&#...........#&",
-";-=***********=-"
-};
-
-static const char *mp3_xpm[] = {
-/* columns rows colors chars-per-pixel */
-"16 16 19 1",
-"  c #040404",
-". c gray11",
-"X c #222222",
-"o c gray20",
-"O c gray26",
-"+ c gray30",
-"@ c gray40",
-"# c #777777",
-"$ c #818181",
-"% c gray59",
-"& c #A0A0A4",
-"* c #B2B2B2",
-"= c #C0C0C0",
-"- c gray80",
-"; c #D7D7D7",
-": c gray89",
-"> c #EAEAEA",
-", c #F1F1F1",
-"< c gray1",
-/* pixels */
-"<<<<<<<>=%@@<<<<",
-"<<<-$+. .O##<<<<",
-"<<<$@%=><<<&<<<<",
-"<<<*<<<<<<<&<<<<",
-"<<<*<<<<<<<&<<<<",
-"<<<*<<<<<<<&<<<<",
-"<<<*<<<<<<<&<<<<",
-"<<<*<<<<<<<&<<<<",
-"<<<*<<<<<<<&<<<<",
-"<<<*<<<<<=$#<<<<",
-"<<:*<<<<$  O<<<<",
-";X O<<<<.  *<<<<",
-"o  @<<<<;%;<<<<<",
-"#X@,<<<<<<<<<<<<",
-"<<<<<<<<<<<<<<<<",
-"<<<<<<<<<<<<<<<<"
-};
-
-static const char *up_xpm[] = {
-/* columns rows colors chars-per-pixel */
-"16 16 25 1",
-"  c #040404",
-". c #0C0C0C",
-"X c #161616",
-"o c gray11",
-"O c #222222",
-"+ c gray16",
-"@ c gray20",
-"# c #393939",
-"$ c gray26",
-"% c gray30",
-"& c #555555",
-"* c gray40",
-"= c #777777",
-"- c #848484",
-"; c gray59",
-": c gray60",
-"> c #A0A0A4",
-", c #B2B2B2",
-"< c #C0C0C0",
-"1 c #D7D7D7",
-"2 c #DDDDDD",
-"3 c gray89",
-"4 c #EAEAEA",
-"5 c #F1F1F1",
-"6 c gray1",
-/* pixels */
-"6666666666666666",
-"6666666666666666",
-"6666<<6666666666",
-"6661$-6666666666",
-"666: &5666666666",
-"661#  :666666666",
-"64&   O<66666666",
-"6,=@ X#-36666336",
-"526- X,436664:<6",
-"666; .=66664*=56",
-"666,o O:43,#+<66",
-"6664&  o%%X :666",
-"6666;     X-6666",
-"66664-. .#>56666",
-"6666666666666666",
-"6666666666666666"
-};
+///\todo get these XPMs out of here and into Theme.
 
 // ----------------------------------------------------------------------------
 // icons
@@ -214,7 +79,7 @@ BEGIN_EVENT_TABLE(UploadDialog, wxDialog)
    EVT_BUTTON(wxID_UPLOAD_FILE, UploadDialog::OnUploadFile)
    EVT_BUTTON(wxID_UPLOAD_DIR, UploadDialog::OnUploadDir)
    EVT_BUTTON(wxID_CREATEDIR, UploadDialog::OnCreateDir)
-   EVT_BUTTON(wxID_OK, UploadDialog::OnOK)
+   EVT_BUTTON(wxID_CANCEL, UploadDialog::OnCancel)
    EVT_BUTTON(wxID_CONNECT, UploadDialog::OnConnect)
    EVT_BUTTON(wxID_DISCONNECT, UploadDialog::OnDisconnect)
 
@@ -229,10 +94,10 @@ END_EVENT_TABLE()
 IMPLEMENT_CLASS(UploadDialog, wxDialog)
 
 UploadDialog::UploadDialog(wxWindow * parent)
-:  wxDialog(parent, -1, _("Audacity FTP"),
+:  wxDialog(parent, -1, wxT("Audacity FTP"),
          wxDefaultPosition, wxSize(640, 480), wxDEFAULT_DIALOG_STYLE | wxWANTS_CHARS)
 {
-
+    ftp = NULL;
     dirnameList = new wxArrayString();
     dirpermList = new wxArrayString();
     dirsizeList = new wxArrayString();
@@ -245,60 +110,66 @@ UploadDialog::UploadDialog(wxWindow * parent)
     ftpList = new wxArrayString();
 
     icons = new wxImageList(16, 16, false, 2);
-    wxIcon *folderIcon = new wxIcon(folder_xpm);
-    wxIcon *fileIcon = new wxIcon(file_xpm);
-    wxIcon *mp3Icon = new wxIcon(mp3_xpm);
-    wxIcon *upIcon = new wxIcon(up_xpm);
+    wxIcon *folderIcon = new wxIcon;
+    folderIcon->CopyFromBitmap(theTheme.Bitmap( bmpUploadFolder));
+    wxIcon *fileIcon = new wxIcon;
+    fileIcon->CopyFromBitmap(theTheme.Bitmap( bmpUploadFile));
+    wxIcon *mp3Icon = new wxIcon;
+    mp3Icon->CopyFromBitmap(theTheme.Bitmap( bmpUploadMp3));
+    wxIcon *upIcon = new wxIcon;
+    upIcon->CopyFromBitmap(theTheme.Bitmap( bmpUploadUp));
     icons->Add(*folderIcon);
     icons->Add(*fileIcon);
     icons->Add(*mp3Icon);
     icons->Add(*upIcon);
     
     wxFlexGridSizer *topSizer = new wxFlexGridSizer(2, 1);  
-    wxStaticBoxSizer *connectionBox = new wxStaticBoxSizer(new wxStaticBox(this, -1, _("FTP Connection"), wxDefaultPosition, wxDefaultSize, 0, wxT("")), wxVERTICAL);   
-    wxStaticBoxSizer *fileBox = new wxStaticBoxSizer(new wxStaticBox(this, -1, _("File Manager"), wxDefaultPosition, wxDefaultSize, 0, wxT("")), wxVERTICAL);
-    wxStaticBoxSizer *siteBox = new wxStaticBoxSizer(new wxStaticBox(this, -1, _("Site Manager"), wxDefaultPosition, wxDefaultSize, 0, wxT("")), wxVERTICAL);
+    wxStaticBoxSizer *connectionBox = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("FTP Connection"), wxDefaultPosition, wxDefaultSize, 0, wxT("")), wxVERTICAL);   
+    wxStaticBoxSizer *fileBox = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("File Manager"), wxDefaultPosition, wxDefaultSize, 0, wxT("")), wxVERTICAL);
+    wxStaticBoxSizer *siteBox = new wxStaticBoxSizer(new wxStaticBox(this, -1, wxT("Site Manager"), wxDefaultPosition, wxDefaultSize, 0, wxT("")), wxVERTICAL);
 
     wxFlexGridSizer *connectionSizer = new wxFlexGridSizer(2, 4);
     wxBoxSizer *leftSizer = new wxBoxSizer(wxVERTICAL);
     wxBoxSizer *rightSizer = new wxBoxSizer(wxVERTICAL);
     wxBoxSizer *fileButtonSizer = new wxBoxSizer(wxHORIZONTAL);
     wxBoxSizer *siteButtonSizer = new wxBoxSizer(wxHORIZONTAL);
+    
+    wxStdDialogButtonSizer *stdButtonSizer = new wxStdDialogButtonSizer();
 
     txtFtpName = new wxTextCtrl(this, wxID_FTPNAME, wxT(""), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, wxT(""));
     txtFtpHost = new wxTextCtrl(this, wxID_FTPHOST, wxT(""), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, wxT(""));
     txtFtpUser = new wxTextCtrl(this, wxID_FTPUSER, wxT(""), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, wxT(""));
     txtFtpPass = new wxTextCtrl(this, wxID_FTPPASS, wxT(""), wxDefaultPosition, wxDefaultSize, wxTE_PASSWORD, wxDefaultValidator, wxT(""));
-    btnConnect = new wxButton(this, wxID_CONNECT, _("Connect"), wxDefaultPosition, wxDefaultSize);
-    btnDisconnect = new wxButton(this, wxID_DISCONNECT, _("Disconnect"), wxDefaultPosition, wxDefaultSize);
+    btnConnect = new wxButton(this, wxID_CONNECT, wxT("&Connect"), wxDefaultPosition, wxDefaultSize);
+    btnDisconnect = new wxButton(this, wxID_DISCONNECT, wxT("&Disconnect"), wxDefaultPosition, wxDefaultSize);
     btnDisconnect->Enable(false);
 
-    btnUploadFile = new wxButton(this, wxID_UPLOAD_FILE, _("Upload File"), wxDefaultPosition, wxDefaultSize);
+    btnUploadFile = new wxButton(this, wxID_UPLOAD_FILE, wxT("Upload &File"), wxDefaultPosition, wxDefaultSize);
     btnUploadFile->Enable(false);
-    btnUploadDir = new wxButton(this, wxID_UPLOAD_DIR, _("Upload Folder"), wxDefaultPosition, wxDefaultSize);
+    btnUploadDir = new wxButton(this, wxID_UPLOAD_DIR, wxT("Upload F&older"), wxDefaultPosition, wxDefaultSize);
     btnUploadDir->Enable(false);
-    btnCreateDir = new wxButton(this, wxID_CREATEDIR, _("New Folder"), wxDefaultPosition, wxDefaultSize);
+    btnCreateDir = new wxButton(this, wxID_CREATEDIR, wxT("N&ew Folder"), wxDefaultPosition, wxDefaultSize);
     btnCreateDir->Enable(false);
 
-    btnDelSite = new wxButton(this, wxID_DELSITE, _("Remove Site"), wxDefaultPosition, wxDefaultSize);
+    btnDelSite = new wxButton(this, wxID_DELSITE, wxT("&Remove Site"), wxDefaultPosition, wxDefaultSize);
 
     fileManager = new wxListView(this, wxID_FILEMANAGER, wxDefaultPosition, wxSize(350, 350), wxLC_REPORT, wxDefaultValidator, wxT(""));
     fileManager->AssignImageList(icons, wxIMAGE_LIST_SMALL);
     //fileManager->Show(false);
-    fileManager->InsertColumn(0, _("name"));
-    fileManager->InsertColumn(1, _("size (bytes)"));
-    fileManager->InsertColumn(2, _("permissions"));
+    fileManager->InsertColumn(0, wxT("Name"));
+    fileManager->InsertColumn(1, wxT("Size (bytes)"));
+    fileManager->InsertColumn(2, wxT("Permissions"));
 
     siteList = new wxListView(this, wxID_SITELIST, wxDefaultPosition, wxSize(200, 215), wxLC_REPORT | wxLC_NO_HEADER, wxDefaultValidator, wxT(""));
     siteList->InsertColumn(0, wxT(""));
 
-    connectionSizer->Add(new wxStaticText(this, -1, _("Name:"), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT, wxT("")));
+    connectionSizer->Add(new wxStaticText(this, -1, wxT("Name:"), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT, wxT("")));
     connectionSizer->Add(txtFtpName);
-    connectionSizer->Add(new wxStaticText(this, -1, _("Host:"), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT, wxT("")));
+    connectionSizer->Add(new wxStaticText(this, -1, wxT("Host:"), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT, wxT("")));
     connectionSizer->Add(txtFtpHost);
-    connectionSizer->Add(new wxStaticText(this, -1, _("Username:"), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT, wxT("")));
+    connectionSizer->Add(new wxStaticText(this, -1, wxT("Username:"), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT, wxT("")));
     connectionSizer->Add(txtFtpUser);
-    connectionSizer->Add(new wxStaticText(this, -1, _("Password:"), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT, wxT("")));
+    connectionSizer->Add(new wxStaticText(this, -1, wxT("Password:"), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT, wxT("")));
     connectionSizer->Add(txtFtpPass);
     connectionSizer->Add(btnConnect, 0, wxALIGN_LEFT | wxALL, 5);
     connectionSizer->Add(btnDisconnect, 0, wxALIGN_LEFT | wxALL, 5);
@@ -306,6 +177,9 @@ UploadDialog::UploadDialog(wxWindow * parent)
     connectionBox->Add(connectionSizer, 0, wxALIGN_CENTER);
     connectionBox->Add(200, 0);
 
+    stdButtonSizer->AddButton(new wxButton(this, wxID_CANCEL, _("&Cancel")));
+    stdButtonSizer->Realize();
+
     fileButtonSizer->Add(btnUploadFile, 0, wxALL, 5);
     fileButtonSizer->Add(btnUploadDir, 0, wxALL, 5);
     fileButtonSizer->Add(btnCreateDir, 0, wxALL, 5);
@@ -323,6 +197,7 @@ UploadDialog::UploadDialog(wxWindow * parent)
     leftSizer->Add(connectionBox);
     leftSizer->Add(siteBox, 0, wxTOP, 10);
     rightSizer->Add(fileBox);
+    rightSizer->Add(stdButtonSizer, 0, wxALIGN_RIGHT | wxALIGN_BOTTOM | wxALL, 5 );
 
     topSizer->Add(leftSizer, 0, wxALL, 10);
     topSizer->Add(rightSizer, 0, wxTOP | wxBOTTOM | wxRIGHT, 10);
@@ -339,12 +214,20 @@ UploadDialog::~UploadDialog()
     
 }
 
+void UploadDialog::OnCancel(wxCommandEvent & event)
+{
+   if (ftp)
+      OnDisconnect(event);
+
+   EndModal(0);
+}
+
 void UploadDialog::OnConnect(wxCommandEvent & WXUNUSED(event))
 {
 
     wxString system;
 
-    SetTitle(_("Connecting..."));
+    SetTitle(wxT("Connecting..."));
     SetCursor(wxCURSOR_WAIT);
 
     //int connectionExists = -1;
@@ -366,8 +249,8 @@ void UploadDialog::OnConnect(wxCommandEvent & WXUNUSED(event))
         if (system.Find(wxT("unix"))==-1 && system.Find(wxT("windows"))==-1)
         {
             wxString msg;
-            msg.Printf(_("Unknown FTP Server type: %s\n\nOnly Unix based and Windows systems are supported/tested at this time."), (const wxChar*)system);
-            wxMessageBox(msg, _("Error"), wxOK | wxICON_INFORMATION, this);
+            msg.Printf(wxT("Unknown FTP Server type: %s\n\nOnly Unix based and Windows systems are supported/tested at this time."), (const wxChar*)system);
+            wxMessageBox(msg, wxT("Error"), wxOK | wxICON_INFORMATION, this);
         }
 
         btnConnect->Enable(false);
@@ -386,7 +269,7 @@ void UploadDialog::OnConnect(wxCommandEvent & WXUNUSED(event))
         txtFtpPass->Enable(false);
 
         if (txtFtpName->GetValue().Trim() == wxT(""))
-            txtFtpName->SetValue(_("New Site"));
+            txtFtpName->SetValue(wxT("New Site"));
 
         // save login into to connections file
         if (SaveFtpSite(txtFtpName->GetValue(), txtFtpHost->GetValue(), txtFtpUser->GetValue(), txtFtpPass->GetValue()))
@@ -396,7 +279,7 @@ void UploadDialog::OnConnect(wxCommandEvent & WXUNUSED(event))
             siteList->SetItemData(tmp, 0);
         }
 
-        SetTitle(_("Connected to: ") + txtFtpHost->GetValue());
+        SetTitle(wxT("Connected to: ") + txtFtpHost->GetValue());
     
         RefreshFiles();
         
@@ -404,8 +287,9 @@ void UploadDialog::OnConnect(wxCommandEvent & WXUNUSED(event))
 
     else
     {
-        wxMessageBox(_("Connection cannot be established"), _("FTP Status"), wxOK | wxICON_INFORMATION, this);
-        delete ftp;
+        wxMessageBox(wxT("Connection cannot be established"), wxT("FTP Status"), wxOK | wxICON_INFORMATION, this);
+      delete ftp;
+      ftp = NULL;
     }
 
     SetCursor(wxCURSOR_ARROW);
@@ -415,7 +299,9 @@ void UploadDialog::OnConnect(wxCommandEvent & WXUNUSED(event))
 void UploadDialog::OnDisconnect(wxCommandEvent & WXUNUSED(event))
 {
 
-    delete ftp;
+   delete ftp;
+   ftp = NULL;
+
 
     btnDisconnect->Enable(false);
     btnConnect->Enable(true);
@@ -433,7 +319,7 @@ void UploadDialog::OnDisconnect(wxCommandEvent & WXUNUSED(event))
     txtFtpUser->Enable(true);
     txtFtpPass->Enable(true);
 
-    SetTitle(_("No Connection"));
+    SetTitle(wxT("No Connection"));
     fileManager->Show(false);
 
     LoadFtpSiteList();
@@ -449,17 +335,17 @@ void UploadDialog::OnUploadFile(wxCommandEvent & WXUNUSED(event))
 
     abort = false;
 
-    wxFileDialog *open = new wxFileDialog(this, _("Choose file(s) to upload"),
-                                          ::wxGetCwd(), wxT(""),
-                                          _("All files (*.*)|*.*"),
-                                          wxOPEN | wxMULTIPLE); 
+    FileDialog open(this, wxT("Choose file(s) to upload"),
+                    ::wxGetCwd(), wxT(""),
+                    wxT("All files (*.*)|*.*"),
+                    wxOPEN | wxMULTIPLE); 
     
-    if (open->ShowModal()==wxID_OK)
+    if (open.ShowModal()==wxID_OK)
     {
         SetCursor(wxCURSOR_WAIT);
 
         // get the file(s) the user selected
-        open->GetPaths(files);
+        open.GetPaths(files);
 
         for (count = 0; count < files.GetCount(); count++)
         {   
@@ -486,15 +372,15 @@ void UploadDialog::OnUploadFile(wxCommandEvent & WXUNUSED(event))
 void UploadDialog::OnUploadDir(wxCommandEvent & WXUNUSED(event))
 {
 
-    wxDirDialog *open = new wxDirDialog(this, _("Choose folder to upload"), wxT("")); 
-    int result = open->ShowModal();
+    wxDirDialog open(this, wxT("Choose folder to upload"), wxT("")); 
+    int result = open.ShowModal();
 
     abort = false;
 
     if (result ==  wxID_OK)
     {
         SetCursor(wxCURSOR_WAIT);
-        UploadDir(open->GetPath(), wxT(""));
+        UploadDir(open.GetPath(), wxT(""));
         SetCursor(wxCURSOR_ARROW);
     }
 }
@@ -505,9 +391,9 @@ void UploadDialog::OnListRightClick(wxListEvent &event)
     listIndex = event.m_itemIndex;
 
     wxMenu *menu = new wxMenu();
-    wxMenuItem *del = new wxMenuItem(menu, wxID_POPUP_DELETE, _("Delete"), _("Delete file(s) from server)"));
-    wxMenuItem *rn  = new wxMenuItem(menu, wxID_POPUP_RENAME, _("Rename"), _("Rename file"));
-    wxMenuItem *dl  = new wxMenuItem(menu, wxID_POPUP_DOWNLOAD, _("Download"), _("Download this file(s) to your computer"));
+    wxMenuItem *del = new wxMenuItem(menu, wxID_POPUP_DELETE, wxT("Delete"), wxT("Delete file(s) from server"));
+    wxMenuItem *rn  = new wxMenuItem(menu, wxID_POPUP_RENAME, wxT("Rename"), wxT("Rename file"));
+    wxMenuItem *dl  = new wxMenuItem(menu, wxID_POPUP_DOWNLOAD, wxT("Download"), wxT("Download this file(s) to your computer"));
 
     wxMenuItem *sep  = new wxMenuItem(menu, -1, wxT(""), wxT(""));
 
@@ -562,7 +448,7 @@ void UploadDialog::OnPopupMenu (wxCommandEvent & event)
         if (fileManager->GetSelectedItemCount() > 1)
         {
 
-            int result = wxMessageBox(_("Delete selected items?"), _("Confirm"), wxYES_NO, NULL);
+            int result = wxMessageBox(wxT("Delete selected items?"), wxT("Confirm"), wxYES_NO, NULL);
 
             if (result == wxYES)
             {
@@ -591,8 +477,7 @@ void UploadDialog::OnPopupMenu (wxCommandEvent & event)
                         // if directory
                         if (selectPerm->Item(count).GetChar(0) == wxT('d'))
                         {
-                            progress = new wxProgressDialog(_("Deleting..."), wxT(""), 100, this);
-                            progress->SetCursor(wxCURSOR_ARROWWAIT);
+                            GetActiveProject()->ProgressShow(wxT("Deleting..."), wxT(""));
 
                             deleteFileList = new wxArrayString();
                             deleteDirList = new wxArrayString();
@@ -603,7 +488,7 @@ void UploadDialog::OnPopupMenu (wxCommandEvent & event)
                             deleteFileList->Clear();
                             deleteDirList->Clear();
 
-                            progress->Destroy();
+                            GetActiveProject()->ProgressHide();
                         }
 
                         else
@@ -626,10 +511,10 @@ void UploadDialog::OnPopupMenu (wxCommandEvent & event)
         {
 
             wxString confirm;
-            confirm.Printf(_("Delete %s?"),
+            confirm.Printf(wxT("Delete %s?"),
                            displayNames->Item(listIndex).c_str());
             
-            int result = wxMessageBox(confirm, _("Confirm"), wxYES_NO, NULL);
+            int result = wxMessageBox(confirm, wxT("Confirm"), wxYES_NO, NULL);
 
             if (result == wxYES)
             {
@@ -639,8 +524,7 @@ void UploadDialog::OnPopupMenu (wxCommandEvent & event)
                     if (displayPerm->Item(listIndex).GetChar(0) == 'd')
                     {
                         // create progress bar
-                        progress = new wxProgressDialog(_("Delete"), wxT(""), 100, this);
-                        progress->SetCursor(wxCURSOR_ARROWWAIT);
+                        GetActiveProject()->ProgressShow(wxT("Delete"), wxT(""));
 
                         deleteFileList = new wxArrayString();
                         deleteDirList = new wxArrayString();    
@@ -651,8 +535,7 @@ void UploadDialog::OnPopupMenu (wxCommandEvent & event)
                         deleteFileList->Clear();
                         deleteDirList->Clear();
 
-                        progress->Destroy();
-
+                        GetActiveProject()->ProgressHide();
                     }
                     else
                     {
@@ -669,7 +552,7 @@ void UploadDialog::OnPopupMenu (wxCommandEvent & event)
 
     if (event.GetId() == wxID_POPUP_RENAME)
     {
-        wxString result = wxGetTextFromUser(_("Please enter the new file/folder name:"), _("Rename"), fileManager->GetItemText(listIndex), this);
+        wxString result = wxGetTextFromUser(wxT("Please enter the new file/folder name:"), wxT("Rename"), fileManager->GetItemText(listIndex), this);
 
         if (result != wxT(""))
         {
@@ -678,7 +561,7 @@ void UploadDialog::OnPopupMenu (wxCommandEvent & event)
             if (ftp->Rename(fileManager->GetItemText(listIndex).MakeLower(), result))
                 RefreshFiles();
             else
-                wxMessageBox(_("Error renaming file, check permissions."), _("FTP Status"), wxOK, NULL);
+                wxMessageBox(wxT("Error renaming file, check permissions."), wxT("FTP Status"), wxOK, NULL);
         }
     }
 
@@ -705,12 +588,8 @@ void UploadDialog::GetDeleteList (wxString src)
                 }
             }
 
-            if (progress)
-            {
-                int percent = (int)(((float)count / (float)displayNames->GetCount())*100.0);
-                progress->Update(percent, _("Preparing to delete"));
-            }           
-
+            int percent = (int)(((float)count / (float)displayNames->GetCount())*1000.0);
+            GetActiveProject()->ProgressUpdate(percent, wxT("Preparing to delete"));
         }
 
         deleteDirList->Add(ftp->Pwd());
@@ -729,17 +608,16 @@ void UploadDialog::RemoveItems(wxArrayString *files, wxArrayString *dirs)
     {
         ftp->RmFile(files->Item(count));
         
-        percent = (int)(((float)count / (float)files->GetCount())*100.0);
-        progress->Update(percent, _("Deleting files"));
+        percent = (int)(((float)count / (float)files->GetCount())*1000.0);
+        GetActiveProject()->ProgressUpdate(percent, wxT("Deleting files"));
     }
     
     for (count = 0; count < dirs->GetCount(); count++)
     {
         ftp->RmDir(dirs->Item(count));
 
-        percent = (int)(((float)count / (float)dirs->GetCount())*100.0);
-        progress->Update(percent, _("Deleting directories"));
-
+        percent = (int)(((float)count / (float)dirs->GetCount())*1000.0);
+        GetActiveProject()->ProgressUpdate(percent, wxT("Deleting directories"));
     }
 
 }
@@ -826,7 +704,7 @@ void UploadDialog::GetDirContents (void)
         }
     }
     else
-        wxMessageBox(_("Unable to retrieve directory contents."), _("FTP Status"), wxOK | wxICON_INFORMATION, this);
+        wxMessageBox(wxT("Unable to retrieve directory contents."), wxT("FTP Status"), wxOK | wxICON_INFORMATION, this);
 
 
 }
@@ -983,7 +861,7 @@ void UploadDialog::DownloadFile (wxString src, wxString dest)
 
         if ( !in )
         {
-                wxLogError(_("Could not get file: ")+src);
+                wxLogError(wxT("Could not get file: ")+src);
         }
         else
         {
@@ -995,8 +873,7 @@ void UploadDialog::DownloadFile (wxString src, wxString dest)
             else
                 chunk = new char[size];
 
-            progress = new wxProgressDialog(_("Download Progress"), wxT(""), 100, this, wxPD_CAN_ABORT);
-            progress->SetCursor(wxCURSOR_ARROWWAIT);    
+            GetActiveProject()->ProgressShow(wxT("Download Progress"), wxT(""));
 
             // create output file
             out = new wxFileOutputStream(dest);
@@ -1027,7 +904,7 @@ void UploadDialog::DownloadFile (wxString src, wxString dest)
 
                     int percent = (int)(((float)count / (float)iterations)*100.0);
                         
-                    if (!progress->Update(percent, src))
+                    if (!GetActiveProject()->ProgressUpdate(percent, src))
                     {
                         //wxMessageBox("ABORT", _T("FTP Status"), wxOK | wxICON_INFORMATION, NULL);
 
@@ -1064,7 +941,7 @@ void UploadDialog::DownloadFile (wxString src, wxString dest)
                 ftp->Reconnect();
             }
 
-            progress->Destroy();
+            GetActiveProject()->ProgressHide();
         }       
     }
 }
@@ -1087,8 +964,7 @@ void UploadDialog::UploadFile(wxString src, wxString dest)
         chunk = new char[size];
 
     // create progress bar
-    progress = new wxProgressDialog(_("Upload Progress"), wxT(""), 100, this, wxPD_CAN_ABORT);
-    progress->SetCursor(wxCURSOR_ARROWWAIT);
+    GetActiveProject()->ProgressShow(wxT("Upload Progress"), wxT(""));
 
     // find number of iterations needed
     if (size > CHUNKSIZE)
@@ -1102,7 +978,13 @@ void UploadDialog::UploadFile(wxString src, wxString dest)
         
     wxOutputStream *out_stream = ftp->GetOutputStream(dest);    
 
-    if (size > 0)
+    if (out_stream == NULL)
+    {
+       wxMessageBox(ftp->GetLastResult());
+    }
+    else
+    {
+        if (size > 0)
         {
             for (count = 0; count < iterations; count++)
             {
@@ -1117,10 +999,10 @@ void UploadDialog::UploadFile(wxString src, wxString dest)
                     out_stream->Write(chunk, size);
                 }
 
-                int percent = (int)(((float)count / (float)iterations)*100.0);
+                int percent = (int)(((float)count / (float)iterations)*1000.0);
                     
                 // result is false, user clicked abort
-                if(!progress->Update(percent, dest))
+                if(!GetActiveProject()->ProgressUpdate(percent, dest))
                 {
                     abort = true;
                         break;
@@ -1145,16 +1027,17 @@ void UploadDialog::UploadFile(wxString src, wxString dest)
             }
 
         }
+        delete out_stream;
+    }
 
     delete [] chunk;
     delete in_stream;
-    delete out_stream;
 
     // if aborted, delete incomplete file
     if (abort)
         ftp->RmFile(dest);
 
-    progress->Destroy();
+    GetActiveProject()->ProgressHide();
 }
 
 void UploadDialog::UploadDir (wxString src, wxString dest)
@@ -1275,7 +1158,7 @@ void UploadDialog::DownloadDir (wxString src, wxString dest)
 
     }
     else
-        wxMessageBox(_("Cannot copy folder: ")+src, _("Error"), wxOK, NULL);
+        wxMessageBox(wxT("Cannot copy folder: ")+src, wxT("Error"), wxOK, NULL);
 
     }
 
@@ -1297,7 +1180,7 @@ void UploadDialog::DownloadItem (wxString &src, wxString &dest,
 
         else
         {
-            wxDirDialog *saveDir = new wxDirDialog(this, _("Download Folder"), wxT(""));
+            wxDirDialog *saveDir = new wxDirDialog(this, wxT("Download Folder"), wxT(""));
             int result = saveDir->ShowModal();
 
             if (result == wxID_OK)
@@ -1315,13 +1198,13 @@ void UploadDialog::DownloadItem (wxString &src, wxString &dest,
         }
         else
         {
-            wxFileDialog *saveFile = new wxFileDialog(this, _("Download File"),
-                                                      wxT(""), src,
-                                                      _("All Files (*.*)|*.*"), wxSAVE);
-            int result = saveFile->ShowModal();
+            FileDialog saveFile(this, wxT("Download File"),
+                                wxT(""), src,
+                                wxT("All files (*.*)|*.*"), wxSAVE);
+            int result = saveFile.ShowModal();
     
             if (result == wxID_OK)
-                DownloadFile(src, saveFile->GetPath());
+                DownloadFile(src, saveFile.GetPath());
         }
     }
 }
@@ -1334,21 +1217,21 @@ void UploadDialog::DownloadItem (wxString &src, bool dir)
     if (dir)
     {
         
-        wxDirDialog *saveDir = new wxDirDialog(this, _("Download Folder"), wxT(""));
-        int result = saveDir->ShowModal();
+        wxDirDialog saveDir(this, wxT("Download Folder"), wxT(""));
+        int result = saveDir.ShowModal();
 
         if (result == wxID_OK)
-            DownloadDir(src, saveDir->GetPath());
+            DownloadDir(src, saveDir.GetPath());
 
     }
     else
     {
 
-        wxFileDialog *saveFile = new wxFileDialog(this, _("Download File"), wxT(""), src, _("All Files (*.*)|*.*"), wxSAVE);
-        int result = saveFile->ShowModal();
+        FileDialog saveFile(this, wxT("Download File"), wxT(""), src, wxT("All files (*.*)|*.*"), wxSAVE);
+        int result = saveFile.ShowModal();
     
         if (result == wxID_OK)
-            DownloadFile(src, saveFile->GetPath());
+            DownloadFile(src, saveFile.GetPath());
         
     }
 }
@@ -1359,7 +1242,7 @@ void UploadDialog::DownloadMultipleItems (void)
     long item = -1;
 
     // get save directory
-    wxDirDialog *saveDir = new wxDirDialog(this, _("Download Files"), wxT(""));
+    wxDirDialog *saveDir = new wxDirDialog(this, wxT("Download Files"), wxT(""));
     saveDir->ShowModal();
 
     // download selected items
@@ -1388,7 +1271,7 @@ void UploadDialog::DownloadMultipleItems (void)
 void UploadDialog::OnCreateDir(wxCommandEvent &event)
 {
 
-    wxString result = wxGetTextFromUser(_("Please enter new folder name:"), _("New Folder"), wxT(""), this);
+    wxString result = wxGetTextFromUser(wxT("Please enter new folder name:"), wxT("New Folder"), wxT(""), this);
 
     if (result != wxT(""))
     {
@@ -1396,6 +1279,8 @@ void UploadDialog::OnCreateDir(wxCommandEvent &event)
 
         if (ftp->MkDir(result.MakeLower()))
             RefreshFiles();
+        else
+           wxMessageBox(ftp->GetLastResult());
     }
 
 }
@@ -1406,7 +1291,7 @@ void UploadDialog::OnNewSite (wxCommandEvent &event)
     int index;
     long tmp;
     wxListItem listItem;
-    wxString newSite = _("New Site");
+    wxString newSite = wxT("New Site");
     
     index = siteList->GetItemCount();
 
@@ -1433,7 +1318,9 @@ void UploadDialog::OnDeleteSite (wxCommandEvent &event)
 //  debug.Printf("%d", index);
 //  wxMessageBox(debug, _T("FTP Status"), wxOK | wxICON_INFORMATION, this);
     
-    wxTextFile *file = new wxTextFile(wxT("connections.ini"));
+    wxFileName fn( FileNames::DataDir(), wxT("connections.ini") );
+
+    wxTextFile *file = new wxTextFile(fn.GetFullPath());
 
     if (file->Exists())
     {
@@ -1480,8 +1367,9 @@ void UploadDialog::OnSelectSite (wxListEvent &event)
 
 bool UploadDialog::SaveFtpSite(wxString name, wxString host, wxString user, wxString pass)
 {
+    wxFileName fn( FileNames::DataDir(), wxT("connections.ini") );
 
-    wxTextFile *file = new wxTextFile(wxT("connections.ini"));
+    wxTextFile *file = new wxTextFile(fn.GetFullPath());
 
     if (file->Exists())
         file->Open();
@@ -1546,7 +1434,9 @@ bool UploadDialog::SaveFtpSite(wxString name, wxString host, wxString user, wxSt
 
 void UploadDialog::LoadFtpSiteList (void)
 {
-    wxTextFile *file = new wxTextFile(wxT("connections.ini"));
+    wxFileName fn( FileNames::DataDir(), wxT("connections.ini") );
+
+    wxTextFile *file = new wxTextFile(fn.GetFullPath());
     unsigned int count;
 
     if (file->Exists())
