@@ -2,41 +2,53 @@
 
   Audacity: A Digital Audio Editor
 
-  WaveTrack.h
+  WaveTrack.cpp
 
   Dominic Mazzoni
 
-**********************************************************************/
+*******************************************************************//**
+
+\class WaveTrack
+\brief A Track that contains audio waveform data.
+
+*//****************************************************************//**
+
+\class WaveTrack::Location
+\brief Used only by WaveTrack, a special way to hold location that
+can accommodate merged regions.
+
+*//****************************************************************//**
+
+\class TrackFactory
+\brief Used to create a WaveTrack, or a LabelTrack..  Implementation 
+of the functions of this class are dispersed through the different 
+Track classes.
+
+*//*******************************************************************/
+
 
 #include <wx/defs.h>
 #include <wx/intl.h>
+#include <wx/debug.h>
 
 #include <math.h>
 
+#include "float_cast.h"
+
 #include "WaveTrack.h"
 
 #include "Envelope.h"
 #include "Sequence.h"
 #include "Spectrum.h"
 
-#include "Prefs.h"
+#include "Project.h"
 #include "Internat.h"
 
 #include "AudioIO.h"
+#include "Prefs.h"
 
 WaveTrack *TrackFactory::NewWaveTrack(sampleFormat format, double rate)
 {
-   if (format == (sampleFormat)0) 
-   {
-      format = (sampleFormat) gPrefs->
-         Read(wxT("/SamplingRate/DefaultProjectSampleFormat"), int16Sample);
-   }
-   if (rate == 0) 
-   {
-      rate = (double) gPrefs->
-         Read(wxT("/SamplingRate/DefaultProjectSampleRate"), AudioIO::GetOptimalSupportedSampleRate());
-   }
-
    return new WaveTrack(mDirManager, format, rate);
 }
 
@@ -45,22 +57,23 @@ WaveTrack::WaveTrack(DirManager *projDirManager, sampleFormat format, double rat
 {
    if (format == (sampleFormat)0) 
    {
-      format = (sampleFormat) gPrefs->
-         Read(wxT("/SamplingRate/DefaultProjectSampleFormat"), int16Sample);
+      format = GetActiveProject()->GetDefaultFormat();
    }
    if (rate == 0) 
    {
-      rate = (double) gPrefs->
-         Read(wxT("/SamplingRate/DefaultProjectSampleRate"), AudioIO::GetOptimalSupportedSampleRate());
+      rate = GetActiveProject()->GetRate();
    }
 
    mDisplay = 0; // Move to GUIWaveTrack
 
+   mLegacyProjectFileOffset = 0;
+
    mFormat = format;
    mRate = (int) rate;
    mGain = 1.0;
    mPan = 0.0;
-   SetName(_("Audio Track"));
+   SetDefaultName(_("Audio Track"));
+   SetName(GetDefaultName());
    mDisplayMin = -1.0;
    mDisplayMax = 1.0;
    mDisplayNumLocations = 0;
@@ -73,6 +86,8 @@ WaveTrack::WaveTrack(WaveTrack &orig):
 {
    mDisplay = 0; // Move to GUIWaveTrack
 
+   mLegacyProjectFileOffset = 0;
+
    Init(orig);
 
    for (WaveClipList::Node *node = orig.mClips.GetFirst(); node; node = node->GetNext())
@@ -87,6 +102,7 @@ void WaveTrack::Init(const WaveTrack &orig)
    mRate = orig.mRate;
    mGain = orig.mGain;
    mPan = orig.mPan;
+   SetDefaultName(orig.GetDefaultName());
    SetName(orig.GetName());
    mDisplay = orig.mDisplay;
    mDisplayMin = orig.mDisplayMin;
@@ -396,15 +412,18 @@ bool WaveTrack::Copy(double t0, double t1, Track **dest)
 
 bool WaveTrack::Paste(double t0, Track *src)
 {
+   bool editClipCanMove = true;
+   gPrefs->Read(wxT("/GUI/EditClipCanMove"), &editClipCanMove);
+   
    //printf("paste: entering WaveTrack::Paste\n");
    
    if (src->GetKind() != Track::Wave)
       return false;
-
+      
    //printf("paste: we have a wave track\n");
 
    WaveTrack* other = (WaveTrack*)src;
-
+   
    //
    // Pasting is a bit complicated, because with the existence of multiclip mode,
    // we must guess the behaviour the user wants.
@@ -437,18 +456,11 @@ bool WaveTrack::Paste(double t0, Track *src)
    
    // Make room for the pasted data, unless the space being pasted in is empty of
    // any clips
-   if (!IsEmpty(t0, t0+insertDuration-1.0/mRate)) {
-
-      for (it=GetClipIterator(); it; it=it->GetNext())
-      {
-         WaveClip* clip = it->GetData();
-
-         //printf("paste: offsetting already existing clip %i by %f seconds\n",
-         //(int)clip, insertDuration);
-
-         if (clip->GetStartTime() > t0-(1.0/mRate))
-            clip->Offset(insertDuration);
-      }
+   if (!IsEmpty(t0, t0+insertDuration-1.0/mRate) && editClipCanMove) {
+      Track *tmp = NULL;
+      Cut(t0, GetEndTime()+1.0/mRate, &tmp);
+      Paste(t0 + insertDuration, tmp);
+      delete tmp;
    }
 
    if (other->GetNumClips() == 1)
@@ -466,13 +478,23 @@ bool WaveTrack::Paste(double t0, Track *src)
          // because an overlap of less than a sample should not trigger
          // traditional behaviour.
 
-         if (t0+src->GetEndTime()-1.0/mRate > clip->GetStartTime() &&
-             t0 < clip->GetEndTime() - 1.0/mRate)
+         if (editClipCanMove)
          {
-            //printf("t0=%.6f: inside clip is %.6f ... %.6f\n",
-            //       t0, clip->GetStartTime(), clip->GetEndTime());
-            insideClip = clip;
-            break;
+            if (t0+src->GetEndTime()-1.0/mRate > clip->GetStartTime() &&
+                t0 < clip->GetEndTime() - 1.0/mRate)
+            {
+               //printf("t0=%.6f: inside clip is %.6f ... %.6f\n",
+               //       t0, clip->GetStartTime(), clip->GetEndTime());
+               insideClip = clip;
+               break;
+            }
+         } else
+         {
+            if (t0 >= clip->GetStartTime() && t0 < clip->GetEndTime())
+            {
+               insideClip = clip;
+               break;
+            }
          }
       }
 
@@ -480,6 +502,26 @@ bool WaveTrack::Paste(double t0, Track *src)
       {
          // Exhibit traditional behaviour
          //printf("paste: traditional behaviour\n");
+         if (!editClipCanMove)
+         {
+            // We did not move other clips out of the way already, so
+            // check if we can paste without having to move other clips
+            for (it=GetClipIterator(); it; it=it->GetNext())
+            {
+               WaveClip *clip = it->GetData();
+               
+               if (clip->GetStartTime() > insideClip->GetStartTime() &&
+                   insideClip->GetEndTime() + insertDuration >
+                                                      clip->GetStartTime())
+               {
+                  wxMessageBox(
+                     _("There is not enough room available to paste the selection"),
+                     _("Error"), wxICON_STOP);
+                  return false;
+               }
+            }
+         }
+         
          return insideClip->Paste(t0, other->GetClipByIndex(0));
       }
 
@@ -488,12 +530,21 @@ bool WaveTrack::Paste(double t0, Track *src)
 
    // Insert new clips
    //printf("paste: multi clip mode!\n");
-   
+
+   if (!editClipCanMove && !IsEmpty(t0, t0+insertDuration-1.0/mRate))
+   {
+      wxMessageBox(
+         _("There is not enough room available to paste the selection"),
+         _("Error"), wxICON_STOP);
+      return false;
+   }
+
    for (it=other->GetClipIterator(); it; it=it->GetNext())
    {
       WaveClip* clip = it->GetData();
 
       WaveClip* newClip = new WaveClip(*clip, mDirManager);
+      newClip->Resample(mRate);
       newClip->Offset(t0);
       newClip->MarkChanged();
       mClips.Append(newClip);
@@ -528,9 +579,30 @@ bool WaveTrack::HandleClear(double t0, double t1,
    if (t1 < t0)
       return false;
 
+   bool editClipCanMove = true;
+   gPrefs->Read(wxT("/GUI/EditClipCanMove"), &editClipCanMove);
+
    WaveClipList::Node* it;
    WaveClipList clipsToDelete;
    WaveClipList clipsToAdd;
+   
+   // We only add cut lines when deleting in the middle of a single clip
+   // The cut line code is not really prepared to handle other situations
+   if (addCutLines)
+   {
+      for (it=GetClipIterator(); it; it=it->GetNext())
+      {
+         WaveClip *clip = it->GetData();
+         
+         if (t1 > clip->GetStartTime() && t0 < clip->GetEndTime() &&
+             (t0 + 1.0/mRate < clip->GetStartTime() ||
+              t1 - 1.0/mRate > clip->GetEndTime()))
+         {
+            addCutLines = false;
+            break;
+         }
+      }
+   }   
 
    for (it=GetClipIterator(); it; it=it->GetNext())
    {
@@ -587,7 +659,8 @@ bool WaveTrack::HandleClear(double t0, double t1,
       if (clip->GetStartTime() >= t1)
       {
          // Clip is "behind" the region -- offset it unless we're splitting
-         if (!split)
+         // or we're using the "don't move other clips" mode
+         if (!split && editClipCanMove)
             clip->Offset(-(t1-t0));
       }
    }
@@ -675,6 +748,91 @@ bool WaveTrack::InsertSilence(double t, double len)
    return true;
 }
 
+//Performs the opposite of Join
+//Analyses selected region for possible Joined clips and disjoins them
+bool WaveTrack::Disjoin(double t0, double t1)
+{
+   sampleCount minSamples = TimeToLongSamples( WAVETRACK_MERGE_POINT_TOLERANCE );
+   sampleCount maxAtOnce = 1048576;
+   float *buffer = new float[ maxAtOnce ];
+   Regions regions;
+   
+   wxBusyCursor busy;
+   
+   for( WaveClipList::Node *it = GetClipIterator(); it; it = it->GetNext() )
+   {
+      WaveClip *clip = it->GetData();
+
+      double startTime = clip->GetStartTime();
+      double endTime = clip->GetEndTime();
+
+      if( endTime < t0 || startTime > t1 )
+         continue;
+
+      if( t0 > startTime )
+         startTime = t0;
+      if( t1 < endTime )
+         endTime = t1;
+      
+      //simply look for a sequence of zeroes and if the sequence
+      //is greater than minimum number, split-delete the region
+      
+      longSampleCount seqStart = -1;
+      longSampleCount start, end;
+      clip->TimeToSamplesClip( startTime, &start );
+      clip->TimeToSamplesClip( endTime, &end );
+       
+      longSampleCount len = ( end - start );
+      for( longSampleCount done = 0; done < len; done += maxAtOnce )
+      {
+         sampleCount numSamples = maxAtOnce;
+         if( done + maxAtOnce > len )
+            numSamples = len - done;
+           
+         clip->GetSamples( ( samplePtr )buffer, floatSample, start + done, 
+               numSamples );
+         for( sampleCount i = 0; i < numSamples; i++ )
+         {
+            longSampleCount curSamplePos = start + done + i;
+
+            //start a new sequence
+            if( buffer[ i ] == 0.0 && seqStart == -1 )
+               seqStart = curSamplePos;
+            else if( buffer[ i ] != 0.0 || curSamplePos == end - 1 )
+            {
+               if( seqStart != -1 )
+               {
+                  longSampleCount seqEnd;
+                  
+                  //consider the end case, where selection ends in zeroes
+                  if( curSamplePos == end - 1 && buffer[ i ] == 0.0 )
+                     seqEnd = end - 1;
+                  else
+                     seqEnd = curSamplePos - 1;
+                  if( seqEnd - seqStart + 1 > minSamples )
+                  {
+                     Region *region = new Region;
+                     region->start = seqStart / GetRate() + clip->GetStartTime();
+                     region->end = seqEnd / GetRate() + clip->GetStartTime();
+                     regions.Add( region );
+                  }
+                  seqStart = -1;
+               }
+            }
+         }
+      }
+   }
+
+   for( unsigned int i = 0; i < regions.GetCount(); i++ )
+   {
+      SplitDelete( regions.Item( i )->start, regions.Item( i )->end );
+      delete regions.Item( i );
+   }
+   
+   delete[] buffer;
+   return true;
+}
+
 bool WaveTrack::Join(double t0, double t1)
 {
    // Merge all WaveClips overlapping selection into one
@@ -692,7 +850,7 @@ bool WaveTrack::Join(double t0, double t1)
 
          // Put in sorted order
          int i;
-         for(i=0; i<clipsToDelete.GetCount(); i++)
+         for(i=0; i<(int)clipsToDelete.GetCount(); i++)
             if (clipsToDelete[i]->GetStartTime() > clip->GetStartTime())
                break;
          //printf("Insert clip %.6f at position %d\n", clip->GetStartTime(), i);
@@ -700,6 +858,10 @@ bool WaveTrack::Join(double t0, double t1)
       }
    }
 
+   //if there are no clips to delete, nothing to do
+   if( clipsToDelete.GetCount() == 0 )
+      return true;
+   
    newClip = CreateClip();
    double t = clipsToDelete[0]->GetOffset();
    newClip->SetOffset(t);
@@ -729,9 +891,11 @@ bool WaveTrack::Join(double t0, double t1)
 }
 
 bool WaveTrack::Append(samplePtr buffer, sampleFormat format,
-                       sampleCount len, unsigned int stride /* = 1 */)
+                       sampleCount len, unsigned int stride /* = 1 */,
+                       XMLWriter *blockFileLog /* = NULL */)
 {
-   return GetLastOrCreateClip()->Append(buffer, format, len, stride);
+   return GetLastOrCreateClip()->Append(buffer, format, len, stride,
+                                        blockFileLog);
 }
 
 bool WaveTrack::AppendAlias(wxString fName, sampleCount start,
@@ -796,6 +960,8 @@ bool WaveTrack::Flush()
 bool WaveTrack::HandleXMLTag(const wxChar *tag, const wxChar **attrs)
 {
    if (!wxStrcmp(tag, wxT("wavetrack"))) {
+      double dblValue;
+      long nValue;
       while(*attrs) {
          const wxChar *attr = *attrs++;
          const wxChar *value = *attrs++;
@@ -803,30 +969,46 @@ bool WaveTrack::HandleXMLTag(const wxChar *tag, const wxChar **attrs)
          if (!value)
             break;
          
+         const wxString strValue = value;
          if (!wxStrcmp(attr, wxT("rate")))
-            mRate = wxAtoi(value);
-         else if (!wxStrcmp(attr, wxT("offset"))) {
-            double ofs = 0.0;
-            Internat::CompatibleToDouble(wxString(value), &ofs);
-            GetLastOrCreateClip()->SetOffset(ofs);
+         {
+            // mRate is an int, but "rate" in the project file is a float.
+            if (!XMLValueChecker::IsGoodString(strValue) || 
+                  !Internat::CompatibleToDouble(strValue, &dblValue) ||
+                  (dblValue < 100.0) || (dblValue > 100000.0)) // same bounds as ImportRawDialog::OnOK
+               return false;
+            mRate = lrint(dblValue);
          }
-         else if (!wxStrcmp(attr, wxT("gain"))) {
-            double d;
-            Internat::CompatibleToDouble(wxString(value), &d);
-            mGain = d;
+         else if (!wxStrcmp(attr, wxT("offset")) && 
+                  XMLValueChecker::IsGoodString(strValue) && 
+                  Internat::CompatibleToDouble(strValue, &dblValue))
+         {
+            // Offset is only relevant for legacy project files. The value
+            // is cached until the actual WaveClip containing the legacy
+            // track is created.
+            mLegacyProjectFileOffset = dblValue;
          }
-         else if (!wxStrcmp(attr, wxT("pan"))) {
-            double d;
-            Internat::CompatibleToDouble(wxString(value), &d);
-            if (d >= -1.0 && d <= 1.0)
-               mPan = d;
-         }
-         else if (!wxStrcmp(attr, wxT("name")))
-            mName = value;
+         else if (!wxStrcmp(attr, wxT("gain")) && 
+                  XMLValueChecker::IsGoodString(strValue) && 
+                  Internat::CompatibleToDouble(strValue, &dblValue))
+            mGain = dblValue;
+         else if (!wxStrcmp(attr, wxT("pan")) && 
+                  XMLValueChecker::IsGoodString(strValue) && 
+                  Internat::CompatibleToDouble(strValue, &dblValue) && 
+                  (dblValue >= -1.0) && (dblValue <= 1.0))
+            mPan = dblValue;
+         else if (!wxStrcmp(attr, wxT("name")) && XMLValueChecker::IsGoodString(strValue))
+            mName = strValue;
          else if (!wxStrcmp(attr, wxT("channel")))
-            mChannel = wxAtoi(value);
-         else if (!wxStrcmp(attr, wxT("linked")))
-            mLinked = wxAtoi(value) != 0;
+         {
+            if (!XMLValueChecker::IsGoodInt(strValue) || !strValue.ToLong(&nValue) || 
+                  !XMLValueChecker::IsValidChannel(nValue))
+               return false;
+            mChannel = nValue;
+         }
+         else if (!wxStrcmp(attr, wxT("linked")) && 
+                  XMLValueChecker::IsGoodInt(strValue) && strValue.ToLong(&nValue))
+            mLinked = (nValue != 0);
          
       } // while
       return true;
@@ -847,10 +1029,17 @@ XMLTagHandler *WaveTrack::HandleXMLChild(const wxChar *tag)
    //
    // This is legacy code (1.2 and previous) and is not called for new projects!
    //
-   if (!wxStrcmp(tag, wxT("sequence")))
-      return GetLastOrCreateClip()->GetSequence();
-   else if (!wxStrcmp(tag, wxT("envelope")))
-      return GetLastOrCreateClip()->GetEnvelope();
+   if (!wxStrcmp(tag, wxT("sequence")) || !wxStrcmp(tag, wxT("envelope")))
+   {
+      // This is a legacy project, so set the cached offset
+      GetLastOrCreateClip()->SetOffset(mLegacyProjectFileOffset);
+ 
+      // Legacy project file tracks are imported as one single wave clip
+      if (!wxStrcmp(tag, wxT("sequence")))
+         return GetLastOrCreateClip()->GetSequence();
+      else if (!wxStrcmp(tag, wxT("envelope")))
+         return GetLastOrCreateClip()->GetEnvelope();
+   }
    
    //
    // This is for the new file format (post-1.2)
@@ -861,30 +1050,23 @@ XMLTagHandler *WaveTrack::HandleXMLChild(const wxChar *tag)
       return NULL;
 }
 
-void WaveTrack::WriteXML(int depth, FILE *fp)
+void WaveTrack::WriteXML(XMLWriter &xmlFile)
 {
-   int i;
-
-   for(i=0; i<depth; i++)
-      fprintf(fp, "\t");
-   fprintf(fp, "<wavetrack ");
-   fprintf(fp, "name=\"%s\" ", (const char *)XMLEsc(mName).mb_str());
-   fprintf(fp, "channel=\"%d\" ", mChannel);
-   fprintf(fp, "linked=\"%d\" ", mLinked);
-   fprintf(fp, "offset=\"%s\" ", (const char *)Internat::ToString(mOffset, 8).mb_str());
-   fprintf(fp, "rate=\"%s\" ", (const char *)Internat::ToString(mRate).mb_str());
-   fprintf(fp, "gain=\"%s\" ", (const char *)Internat::ToString((double)mGain).mb_str());
-   fprintf(fp, "pan=\"%s\" ", (const char *)Internat::ToString((double)mPan).mb_str());
-   fprintf(fp, ">\n");
+   xmlFile.StartTag(wxT("wavetrack"));
+   xmlFile.WriteAttr(wxT("name"), mName);
+   xmlFile.WriteAttr(wxT("channel"), mChannel);
+   xmlFile.WriteAttr(wxT("linked"), mLinked);
+   xmlFile.WriteAttr(wxT("offset"), mOffset, 8);
+   xmlFile.WriteAttr(wxT("rate"), mRate);
+   xmlFile.WriteAttr(wxT("gain"), (double)mGain);
+   xmlFile.WriteAttr(wxT("pan"), (double)mPan);
    
    for (WaveClipList::Node* it=GetClipIterator(); it; it=it->GetNext())
    {
-      it->GetData()->WriteXML(depth+1, fp);
+      it->GetData()->WriteXML(xmlFile);
    }
 
-   for(i=0; i<depth; i++)
-      fprintf(fp, "\t");
-   fprintf(fp, "</wavetrack>\n");
+   xmlFile.EndTag(wxT("wavetrack"));
 }
 
 bool WaveTrack::GetErrorOpening()
@@ -899,7 +1081,7 @@ bool WaveTrack::GetErrorOpening()
 bool WaveTrack::Lock()
 {
    for (WaveClipList::Node* it=GetClipIterator(); it; it=it->GetNext())
-      it->GetData()->GetSequence()->Lock();
+      it->GetData()->Lock();
 
    return true;
 }
@@ -907,7 +1089,7 @@ bool WaveTrack::Lock()
 bool WaveTrack::Unlock()
 {
    for (WaveClipList::Node* it=GetClipIterator(); it; it=it->GetNext())
-      it->GetData()->GetSequence()->Unlock();
+      it->GetData()->Unlock();
 
    return true;
 }
@@ -917,6 +1099,11 @@ longSampleCount WaveTrack::TimeToLongSamples(double t0)
    return (longSampleCount)floor(t0 * mRate + 0.5);
 }
 
+double WaveTrack::LongSamplesToTime(longSampleCount pos)
+{
+   return ((double)pos) / mRate;
+}
+
 double WaveTrack::GetStartTime()
 {
    bool found = false;
@@ -1207,6 +1394,7 @@ void WaveTrack::MoveClipToTrack(WaveClip *clip, WaveTrack* dest)
          WaveClip* clip = it->GetData();
          mClips.DeleteNode(it);
          dest->mClips.Append(clip);
+         return; // JKC iterator is now 'defunct' so better return straight away.
       }
    }
 }
@@ -1271,6 +1459,14 @@ bool WaveTrack::CanInsertClip(WaveClip* clip)
    return true;
 }
 
+bool WaveTrack::Split( double t0, double t1 )
+{
+   bool ret = SplitAt( t0 );
+   if( ret && t0 != t1 )
+      ret = SplitAt( t1 );
+   return ret;
+}
+
 bool WaveTrack::SplitAt(double t)
 {
    for (WaveClipList::Node* it=GetClipIterator(); it; it=it->GetNext())
@@ -1289,7 +1485,8 @@ bool WaveTrack::SplitAt(double t)
             delete newClip;
             return false;
          }
-         newClip->Offset(t - c->GetStartTime());
+         longSampleCount here = llrint(floor(((t - c->GetStartTime() - mOffset) * mRate) + 0.5));
+         newClip->Offset((double)here/(double)mRate);
          mClips.Append(newClip);
          return true;
       }
@@ -1300,27 +1497,29 @@ bool WaveTrack::SplitAt(double t)
 
 void WaveTrack::UpdateLocationsCache()
 {
-   WaveClipList::Node *it, *jt;
+   unsigned int i;
+   WaveClipArray clips;
+   
+   FillSortedClipArray(clips);
    
    mDisplayNumLocations = 0;
-
-   for (it=GetClipIterator(); it; it=it->GetNext())
+   
+   // Count number of display locations
+   for (i = 0; i < clips.GetCount(); i++)
    {
-      WaveClip* clip = it->GetData();
+      WaveClip* clip = clips.Item(i);
       
       mDisplayNumLocations += clip->GetCutLines()->GetCount();
-      
-      for (jt=GetClipIterator(); jt; jt=jt->GetNext())
-      {
-         WaveClip* clip2 = jt->GetData();
-         if (clip != clip2 && fabs(clip->GetEndTime()-clip2->GetStartTime()) < WAVETRACK_MERGE_POINT_TOLERANCE)
-            mDisplayNumLocations++;
-      }
+
+      if (i > 0 && fabs(clips.Item(i - 1)->GetEndTime() -
+                  clip->GetStartTime()) < WAVETRACK_MERGE_POINT_TOLERANCE)
+         mDisplayNumLocations++;
    }
 
    if (mDisplayNumLocations == 0)
       return;
 
+   // Alloc necessary number of display locations
    if (mDisplayNumLocations > mDisplayNumLocationsAllocated)
    {
       // Only realloc, if we need more space than before. Otherwise
@@ -1331,41 +1530,103 @@ void WaveTrack::UpdateLocationsCache()
       mDisplayNumLocationsAllocated = mDisplayNumLocations;
    }
 
+   // Add all display locations to cache
    int curpos = 0;
-
-   for (it=GetClipIterator(); it; it=it->GetNext())
+   
+   for (i = 0; i < clips.GetCount(); i++)
    {
-      WaveClip* clip = it->GetData();
+      WaveClip* clip = clips.Item(i);
+
       WaveClipList* cutlines = clip->GetCutLines();
-      for (jt = cutlines->GetFirst(); jt; jt=jt->GetNext())
+      for (WaveClipList::Node* it = cutlines->GetFirst(); it;
+           it = it->GetNext())
       {
+         // Add cut line expander point
          mDisplayLocations[curpos].typ = locationCutLine;
-         mDisplayLocations[curpos].pos = jt->GetData()->GetOffset() + it->GetData()->GetOffset();
+         mDisplayLocations[curpos].pos =
+            clip->GetOffset() + it->GetData()->GetOffset();
          curpos++;
       }
-
-      for (jt=GetClipIterator(); jt; jt=jt->GetNext())
+      
+      if (i > 0)
       {
-         WaveClip* clip2 = jt->GetData();
-         if (clip != clip2 && fabs(clip->GetEndTime()-clip2->GetStartTime()) < WAVETRACK_MERGE_POINT_TOLERANCE)
+         WaveClip* previousClip = clips.Item(i - 1);
+
+         if (fabs(previousClip->GetEndTime() - clip->GetStartTime())
+                                          < WAVETRACK_MERGE_POINT_TOLERANCE)
          {
+            // Add merge point
             mDisplayLocations[curpos].typ = locationMergePoint;
-            mDisplayLocations[curpos].pos = clip->GetEndTime();
-            mDisplayLocations[curpos].clipidx1 = mClips.IndexOf(clip);
-            mDisplayLocations[curpos].clipidx2 = mClips.IndexOf(clip2);
+            mDisplayLocations[curpos].pos = clips.Item(i-1)->GetEndTime();
+            mDisplayLocations[curpos].clipidx1 = mClips.IndexOf(previousClip);
+            mDisplayLocations[curpos].clipidx2 = mClips.IndexOf(clip);
             curpos++;
          }
       }
    }
+   
+   wxASSERT(curpos == mDisplayNumLocations);
 }
 
 // Expand cut line (that is, re-insert audio, then delete audio saved in cut line)
-bool WaveTrack::ExpandCutLine(double cutLinePosition, double* cutlineStart, double* cutlineEnd)
+bool WaveTrack::ExpandCutLine(double cutLinePosition, double* cutlineStart,
+                              double* cutlineEnd)
 {
+   bool editClipCanMove = true;
+   gPrefs->Read(wxT("/GUI/EditClipCanMove"), &editClipCanMove);
+
+   // Find clip which contains this cut line   
    for (WaveClipList::Node* it=GetClipIterator(); it; it=it->GetNext())
    {
-      if (it->GetData()->ExpandCutLine(cutLinePosition, cutlineStart, cutlineEnd))
+      WaveClip* clip = it->GetData();
+      double start = 0, end = 0;
+
+      if (clip->FindCutLine(cutLinePosition, &start, &end))
+      {
+         WaveClipList::Node* it2;
+         
+         if (!editClipCanMove)
+         {
+            // We are not allowed to move the other clips, so see if there
+            // is enough room to expand the cut line
+            for (it2=GetClipIterator(); it2; it2=it2->GetNext())
+            {
+               WaveClip *clip2 = it2->GetData();
+               
+               if (clip2->GetStartTime() > clip->GetStartTime() &&
+                   clip->GetEndTime() + end - start > clip2->GetStartTime())
+               {
+                  wxMessageBox(
+                     _("There is not enough room available to expand the cut line"),
+                     _("Error"), wxICON_STOP);
+                  return false;
+               }
+            }
+         }
+         
+         if (!clip->ExpandCutLine(cutLinePosition))
+            return false;
+
+         if (cutlineStart)
+            *cutlineStart = start;
+         if (cutlineEnd)
+            *cutlineEnd = end;
+
+         // Move clips which are to the right of the cut line
+         if (editClipCanMove)
+         {
+            for (it2=GetClipIterator(); it2;
+                 it2=it2->GetNext())
+            {
+               WaveClip* clip2 = it2->GetData();
+   
+               if (clip2->GetStartTime() > clip->GetStartTime())
+                  clip2->Offset(end - start);
+            }
+         }
+
          return true;
+      }
    }
 
    return false;
@@ -1396,6 +1657,39 @@ bool WaveTrack::MergeClips(int clipidx1, int clipidx2)
    return true;
 }
 
+bool WaveTrack::Resample(int rate, bool progress)
+{
+   for (WaveClipList::Node* it=GetClipIterator(); it; it=it->GetNext())
+      if (!it->GetData()->Resample(rate, progress))
+      {
+         // FIXME: The track is now in an inconsistent state since some
+         //        clips are resampled and some are not
+         return false;
+      }
+      
+   mRate = rate;
+   
+   return true;
+}
+
+static int SortClipArrayCmpFunc(WaveClip** clip1, WaveClip** clip2)
+{
+   if((*clip1)->GetStartTime() < (*clip2)->GetStartTime())
+      return -1;
+   else
+      return 1;
+}
+
+void WaveTrack::FillSortedClipArray(WaveClipArray& clips)
+{
+   clips.Empty();
+   
+   for (WaveClipList::Node *it=GetClipIterator(); it; it=it->GetNext())
+      clips.Add(it->GetData());
+   
+   clips.Sort(SortClipArrayCmpFunc);
+}
+
 // Indentation settings for Vim and Emacs and unique identifier for Arch, a
 // version control system. Please do not modify past this point.
 //
