@@ -220,10 +220,11 @@ class WaveTrack: public Track {
    bool Lock();
    bool Unlock();
 
-   // Utility function to convert between times in seconds
+   // Utility functions to convert between times in seconds
    // and sample positions
 
    longSampleCount TimeToLongSamples(double t0);
+   double LongSamplesToTime(longSampleCount pos);
 
    // Get access to the clips in the tracks. This is used by
    // track artists and also by TrackPanel when sliding...it would
@@ -248,6 +249,10 @@ class WaveTrack: public Track {
 
    // Get number of clips in this WaveTrack
    int GetNumClips() const;
+   
+   // Add all wave clips to the given array 'clips' and sort the array by
+   // clip start time. The array is emptied prior to adding the clips.
+   void FillSortedClipArray(WaveClipArray& clips);
 
    // Before calling 'Offset' on a clip, use this function to see if the
    // offsetting is allowed with respect to the other clips in this track.
