@@ -833,9 +833,9 @@ bool CommandManager::HandleXMLTag(const wxChar *tag, const wxChar **attrs)
          if (!value)
             break;
          
-         if (!wxStrcmp(attr, wxT("name")))
+         if (!wxStrcmp(attr, wxT("name")) && XMLValueChecker::IsGoodString(value))
             name = value;
-         if (!wxStrcmp(attr, wxT("key")))
+         if (!wxStrcmp(attr, wxT("key")) && XMLValueChecker::IsGoodString(value))
             key = value;
       }
 
@@ -868,7 +868,7 @@ void CommandManager::WriteXML(XMLWriter &xmlFile)
    unsigned int j;
 
    xmlFile.StartTag(wxT("audacitykeyboard"));
-   xmlFile.WriteAttr(wxT("audacityversion"), wxT(AUDACITY_VERSION_STRING));
+   xmlFile.WriteAttr(wxT("audacityversion"), AUDACITY_VERSION_STRING);
 
    for(j=0; j<mCommandList.GetCount(); j++) {
       if (!mCommandList[j]->multi) {
