@@ -15,8 +15,6 @@
 
 #if USE_SOUNDTOUCH
 
-#include "SoundTouch.h"
-
 #include "ChangePitch.h"
 
 #include "../PitchName.h"
@@ -92,8 +90,8 @@ void EffectChangePitch::DeduceFrequencies()
 
       track->Get((samplePtr) buffer, floatSample, start, analyzeSize);
       for(i=0; i<numWindows; i++) {
-         ComputeSpectrum(buffer+i*windowSize, windowSize, windowSize/2,
-                         (int)rate, windowSize, rate, freq, true);
+         ComputeSpectrum(buffer+i*windowSize, windowSize,
+                         windowSize, rate, freq, true);
          for(j=0; j<windowSize/2; j++)
             freqa[j] += freq[j];
       }
@@ -392,13 +390,12 @@ ChangePitchDialog::ChangePitchDialog(EffectChangePitch * effect,
    pBoxSizer_OK->Add(32, 8); // horizontal spacer
 
    wxButton * pButton_Cancel =
-       new wxButton(this, wxID_CANCEL, _("Cancel"), wxDefaultPosition, wxDefaultSize, 0);
+       new wxButton(this, wxID_CANCEL, _("&Cancel"), wxDefaultPosition, wxDefaultSize, 0);
    pBoxSizer_OK->Add(pButton_Cancel, 0, wxALIGN_CENTER | wxALL, 4);
 
    wxButton * pButton_OK =
-       new wxButton(this, wxID_OK, _("OK"), wxDefaultPosition, wxDefaultSize, 0);
+       new wxButton(this, wxID_OK, _("&OK"), wxDefaultPosition, wxDefaultSize, 0);
    pButton_OK->SetDefault();
-   pButton_OK->SetFocus();
    pBoxSizer_OK->Add(pButton_OK, 0, wxALIGN_CENTER | wxALL, 4);
 
    pBoxSizer_Dialog->Add(pBoxSizer_OK, 0, wxALIGN_CENTER | wxALL, 8);
