@@ -161,7 +161,8 @@ bool Effect::DoEffect(wxWindow *parent, int flags,
    bool returnVal = true;
    bool skipFlag = CheckWhetherSkipEffect();
    if (skipFlag == false) {
-      GetActiveProject()->ProgressShow(GetEffectName(), GetEffectAction());
+      GetActiveProject()->ProgressShow(StripAmpersand(GetEffectName()),
+                                       GetEffectAction());
       returnVal = Process();
       GetActiveProject()->ProgressHide();
    }
@@ -298,7 +299,10 @@ void Effect::Preview()
    // Effect is already inited; we call Process, End, and then Init
    // again, so the state is exactly the way it was before Preview
    // was called.
+   GetActiveProject()->ProgressShow(StripAmpersand(GetEffectName()),
+                                    _("Preparing preview"));
    Process();
+   GetActiveProject()->ProgressHide();
    End();
    Init();
 
@@ -311,23 +315,31 @@ void Effect::Preview()
    if (mixRight)
       playbackTracks.Add(mixRight);
 
+
    // Start audio playing
-   
+
    int token =
       gAudioIO->StartStream(playbackTracks, recordingTracks, NULL,
                             rate, t0, t1, NULL);
-   if (token) {
-      wxBusyCursor busy;
-      ::wxMilliSleep((int)(previewLen*1000));
 
-      while(gAudioIO->IsStreamActive(token)) {
+   if (token) {
+      bool previewing = true;
+
+      GetActiveProject()->ProgressShow(StripAmpersand(GetEffectName()),
+                                       _("Previewing"));
+
+      while (gAudioIO->IsStreamActive(token) && previewing) {
          ::wxMilliSleep(100);
+         int t = int(1000 * (gAudioIO->GetStreamTime() / t1));
+         previewing = GetActiveProject()->ProgressUpdate(t);
       }
       gAudioIO->StopStream();
 
-      while(gAudioIO->IsBusy()) {
+      while (gAudioIO->IsBusy()) {
          ::wxMilliSleep(100);
       }
+
+      GetActiveProject()->ProgressHide();
    }
    else {
       wxMessageBox(_("Error while opening sound device. Please check the output device settings and the project sample rate."),
