@@ -36,6 +36,7 @@ class wxString;
 class wxBoxSizer;
 class wxChoice;
 class wxRadioButton;
+class wxCheckBox;
 
 #include "Effect.h"
 #include "../xml/XMLTagHandler.h"
@@ -86,11 +87,16 @@ public:
       return wxString(_("Equalization..."));
    }
 
+   virtual wxString GetEffectIdentifier() {
+      return wxString(wxT("Equalization"));
+   }
+
    virtual wxString GetEffectAction() {
       return wxString(_("Performing Equalization"));
    }
 
    virtual bool PromptUser();
+   virtual bool DontPromptUser();
    virtual bool TransferParameters( Shuttle & shuttle );
 
    virtual bool Process();
@@ -112,8 +118,12 @@ private:
    float *mFilterFuncR;
    float *mFilterFuncI;
    int mM;
-   float mdBMax;
-   float mdBMin;
+   wxString mCurveName;
+   bool mLin;
+   double mdBMax;
+   double mdBMin;
+   bool mDrawMode;
+   int mInterp;
 
 public:
    enum curveType {
@@ -159,6 +169,8 @@ public:
    float dBMin;
    bool RecalcRequired;
 
+   Envelope *mEnvelope;
+
 private:
 
    wxBitmap *mBitmap;
@@ -175,8 +187,6 @@ private:
    double mLoFreq;
    double mHiFreq;
 
-   Envelope *mEnvelope;
-
    DECLARE_EVENT_TABLE()
 };
 
@@ -193,7 +203,7 @@ public:
    // constructors and destructors
    EqualizationDialog(EffectEqualization * effect,
                double loFreq, double hiFreq,
-               float *filterFuncR, float *filterFuncI, long windowSize,
+               float *filterFuncR, float *filterFuncI, long windowSize, wxString CurveName,
                wxWindow *parent, wxWindowID id,
                const wxString &title,
                const wxPoint& pos = wxDefaultPosition,
@@ -207,16 +217,21 @@ public:
    virtual bool CalcFilter();
 
    void EnvelopeUpdated();
+   void EnvelopeUpdated(Envelope *env);
    static const double thirdOct[];
    wxRadioButton *mFaderOrDraw[2];
    wxChoice *mInterpChoice;
+   wxCheckBox *mLinFreq;
    int M;
+   wxString curveName;
+   bool linCheck;
    float dBMin;
    float dBMax;
    double whens[NUM_PTS];
-   double whensFreq[NUM_PTS];
    double whenSliders[NUMBER_OF_BANDS+1];
    int bandsInUse;
+   bool drawMode;
+   int interp;
 
 private:
    void MakeEqualizationDialog();
@@ -226,6 +241,7 @@ private:
    void SaveCurves();
    void Select(int sel);
    void setCurve(Envelope *env, int currentCurve);
+   void setCurve(Envelope *env, wxString curveName);
    void setCurve(Envelope *env);
    void GraphicEQ(Envelope *env);
    void spline(double x[], double y[], int n, double y2[]);
@@ -254,6 +270,7 @@ private:
       drawRadioID,
       sliderRadioID,
       ID_INTERP,
+      ID_LIN_FREQ,
       ID_SLIDER   // needs to come last
    };
 
@@ -269,6 +286,10 @@ private:
    void OnSliderDBMIN( wxCommandEvent &event );
    void OnDrawRadio(wxCommandEvent &event );
    void OnSliderRadio(wxCommandEvent &event );
+   void OnLinFreq(wxCommandEvent &event );
+   void UpdateGraphic(void);
+   void EnvLogToLin(void);
+   void EnvLinToLog(void);
    void ErrMin(void);
    void OnCurve( wxCommandEvent &event );
    void OnSaveAs( wxCommandEvent &event );
@@ -292,7 +313,8 @@ private:
    double m_EQVals[NUMBER_OF_BANDS+1];
 
    EqualizationPanel *mPanel;
-   Envelope *mEnvelope;
+   Envelope *mLogEnvelope;
+   Envelope *mLinEnvelope;
    wxBoxSizer *mCurveSizer;
    wxChoice *mCurve;
    wxButton *mDelete;
@@ -307,6 +329,9 @@ private:
    wxBoxSizer *szrC;
    wxBoxSizer *szrG;
    wxBoxSizer *szrV;
+   wxBoxSizer *szrH;
+   wxBoxSizer *szrI;
+   wxBoxSizer *szrL;
    wxBoxSizer *szr3;
    wxBoxSizer *szr4;
    wxBoxSizer *szr2;
