@@ -16,14 +16,15 @@
 
 #include "Amplify.h"
 // #include "AvcCompressor.h"
+#include "AutoDuck.h"
 #include "BassBoost.h"
 #include "ChangeSpeed.h"
 #include "ClickRemoval.h"
 #include "Compressor.h"
+#include "DtmfGen.h"
 #include "Echo.h"
 #include "Equalization.h"
 #include "Fade.h"
-#include "Filter.h"
 #include "Invert.h"
 #include "Leveller.h"
 #include "Noise.h"
@@ -75,6 +76,7 @@ void LoadEffects()
    Effect::RegisterEffect(new EffectNoise());
    Effect::RegisterEffect(new EffectSilence());
    Effect::RegisterEffect(new EffectToneGen());
+   Effect::RegisterEffect(new EffectDtmf());
    // A little magic to convert 'Tone' to chirps.
    Effect::RegisterEffect(&((new EffectToneGen())->EnableForChirps()));
 
@@ -89,6 +91,8 @@ void LoadEffects()
    // In this list, designating an effect as 'SIMPLE_EFFECT' just means
    // that it should be included in even the most basic of menus.
    // This was introduced for CleanSpeech mode.
+   
+   Effect::RegisterEffect(new EffectAutoDuck());
    Effect::RegisterEffect(new EffectBassBoost());
    Effect::RegisterEffect(new EffectChangeSpeed());
 	#ifdef USE_SOUNDTOUCH
@@ -101,7 +105,6 @@ void LoadEffects()
    Effect::RegisterEffect(new EffectEqualization());
    Effect::RegisterEffect(new EffectFadeIn(), SIMPLE_EFFECT);
    Effect::RegisterEffect(new EffectFadeOut(), SIMPLE_EFFECT);
-   Effect::RegisterEffect(new EffectFilter());
    Effect::RegisterEffect(new EffectInvert());
    Effect::RegisterEffect(new EffectLeveller(), SIMPLE_EFFECT);
    Effect::RegisterEffect(new EffectNoiseRemoval(), SIMPLE_EFFECT);
