@@ -55,7 +55,7 @@ EffectToneGen::EffectToneGen()
 wxString EffectToneGen::GetEffectDescription() { 
    // Note: This is useful only after values have been set. 
    /// \todo update to include *all* chirp parameters??
-   const wxChar* waveformNames[] = {wxT("sine"), wxT("square"), wxT("sawtooth")};
+   const wxChar* waveformNames[] = {wxT("sine"), wxT("square"), wxT("sawtooth"), wxT("square, no alias")};
    return wxString::Format(_("Applied effect: Generate %s wave %s, frequency = %.2f Hz, amplitude = %.2f, %.6lf seconds"), 
       waveformNames[waveform], mbChirp ? wxT("chirp") : wxT("tone"), frequency[0], amplitude[0], length); 
 } 
@@ -63,14 +63,18 @@ wxString EffectToneGen::GetEffectDescription() {
 bool EffectToneGen::PromptUser()
 {
    wxArrayString waveforms;
-   if (mT1 > mT0)
-      length = mT1 - mT0;
-
+   ToneGenDialog dlog(mParent, mbChirp ? _("Chirp Generator") : _("Tone Generator"));
    waveforms.Add(_("Sine"));
    waveforms.Add(_("Square"));
    waveforms.Add(_("Sawtooth"));
+   waveforms.Add(_("Square, no alias"));
+   dlog.isSelection= false;
 
-   ToneGenDialog dlog(mParent, mbChirp ? _("Chirp Generator") : _("Tone Generator"));
+   if (mT1 > mT0) {
+      length = mT1 - mT0;
+      dlog.isSelection= true;
+   }
+  
    dlog.mbChirp = mbChirp;
    dlog.waveform = waveform;
    dlog.frequency[0] = frequency[0];
@@ -81,6 +85,7 @@ bool EffectToneGen::PromptUser()
    dlog.waveforms = &waveforms;
    dlog.Init();
    dlog.TransferDataToWindow();
+   dlog.Fit();
    dlog.ShowModal();
 
    if (dlog.GetReturnCode() == wxID_CANCEL)
@@ -101,7 +106,7 @@ bool EffectToneGen::PromptUser()
 }
 
 bool EffectToneGen::TransferParameters( Shuttle & shuttle )
-{  
+{
 /// \todo this should in time be using ShuttleGui too.
 //   shuttle.TransferInt("",,0);
    return true;
@@ -111,74 +116,107 @@ bool EffectToneGen::MakeTone(float *buffer, sampleCount len)
 {
    double throwaway = 0;        //passed to modf but never used
    int i;
+   double f = 0.0;
 
-   int iSample;
-   double f;
-
-   // t is our Blend parameter, 0.0 at left end, 1.0 at right end 
-   // of the complete tone.
-   double t;
-   double BlendedAmplitude;
    double BlendedFrequency;
-   double BlendedLogFrequency;
+   double BlendedAmplitude;
 
-   // Do our divisions here, outside the loop, for greater efficiency.
+   // calculate delta, and reposition from where we left
+   double amplitudeQuantum = (amplitude[1]-amplitude[0]) / numSamples;
+   double frequencyQuantum = (frequency[1]-frequency[0]) / numSamples;
+   BlendedAmplitude = amplitude[0] + amplitudeQuantum * mSample;
 
-   // Multiplier to convert from the sample number to the position in blend.
-   double BlendMultiplier = 1.0f/numSamples;
-   // Multiplier to convert from cycles-per-second to cycles-per-sample.
-   double PositionMultiplier = 1.0f/ mCurRate;
+   // precalculations:
+   double pre2PI = 2 * M_PI;
+   double pre4divPI = 4. / M_PI;
 
+   /*
+    Duplicate the code for the two cases, log interpolation and linear interpolation
+    I hope this is more readable, a bit faster (only one branching in mbLogInterpolation)
+    Local variables are declared inside respective branch, globals are declared up.
+   */
+
+   // this for log interpolation
    if( mbLogInterpolation )
    {
       logFrequency[0] = log10( frequency[0] );
       logFrequency[1] = log10( frequency[1] );
-   }
+      // calculate delta, and reposition from where we left
+      double logfrequencyQuantum = (logFrequency[1]-logFrequency[0]) / numSamples;
+      double BlendedLogFrequency = logFrequency[0] + logfrequencyQuantum * mSample;
 
-   for (i = 0; i < len; i++)
-   {
-      iSample = i+mSample;
-      t = iSample * BlendMultiplier;
-      BlendedAmplitude = amplitude[0] + (amplitude[1]-amplitude[0]) * t;
-
-      if( i == (len/2) )
-         i=i;
-
-      // Log interpolation is not currently used.
-      // A linear shift in frequency is what most people expect, and
-      // gives straight lines on a frequency plot.
-      if( mbLogInterpolation )
+      for (i = 0; i < len; i++)
       {
-         BlendedLogFrequency = logFrequency[0] + (logFrequency[1]-logFrequency[0]) * t;
          BlendedFrequency = pow( 10.0, (double)BlendedLogFrequency );
+         switch (waveform) {
+            case 0:    //sine
+               f = (float) sin(pre2PI * mPositionInCycles/mCurRate);
+               break;
+            case 1:    //square
+               f = (modf(mPositionInCycles/mCurRate, &throwaway) < 0.5) ? 1.0f :-1.0f;
+               break;
+            case 2:    //sawtooth
+               f = (2 * modf(mPositionInCycles/mCurRate+0.5f, &throwaway)) -1.0f;
+               break;
+            default:
+               break;
+         }
+         // insert value in buffer
+         buffer[i] = BlendedAmplitude * f;
+         // update freq,amplitude
+         mPositionInCycles += BlendedFrequency;
+         BlendedAmplitude += amplitudeQuantum;
+         BlendedLogFrequency += logfrequencyQuantum;
       }
-      else
+
+   } else {
+      // this for regular case, linear interpolation
+      BlendedFrequency = frequency[0] + frequencyQuantum * mSample;
+      double a,b;
+      int k;
+      for (i = 0; i < len; i++)
       {
-         BlendedFrequency = frequency[0] + (frequency[1]-frequency[0]) * t;
-      }
+         switch (waveform) {
+            case 0:    //sine
+               f = (float) sin(pre2PI * mPositionInCycles/mCurRate);
+               break;
+            case 1:    //square
+               f = (modf(mPositionInCycles/mCurRate, &throwaway) < 0.5) ? 1.0f :-1.0f;
+               break;
+            case 2:    //sawtooth
+               f = (2 * modf(mPositionInCycles/mCurRate+0.5f, &throwaway)) -1.0f;
+               break;
+         case 3:    //square, no alias.  Good down to 110Hz @ 44100Hz sampling.
+               //do fundamental (k=1) outside loop
+               b = (1. + cos((pre2PI * BlendedFrequency)/mCurRate))/pre4divPI;  //scaling
+               f = (float) pre4divPI * sin(pre2PI * mPositionInCycles/mCurRate);
+               for(k=3; (k<200) && (k * BlendedFrequency < mCurRate/2.); k+=2)
+               {
+                  //Hanning Window in freq domain
+                  a = 1. + cos((pre2PI * k * BlendedFrequency)/mCurRate);
+                  //calc harmonic, apply window, scale to amplitude of fundamental
+                  f += (float) a * sin(pre2PI * mPositionInCycles/mCurRate * k)/(b*k);
+               }
+               break;
 
-      // Add cycles/second * second/samples * 1-sample
-      mPositionInCycles += BlendedFrequency * PositionMultiplier;
-
-      switch (waveform) {
-         case 0:    //sine
-            f = (float) sin(2 * M_PI * mPositionInCycles);
-            break;
-         case 1:    //square
-            f = (modf(mPositionInCycles, &throwaway) < 0.5) ? 1.0f :-1.0f;
-            break;
-         case 2:    //sawtooth
-            f = (2 * modf(mPositionInCycles+0.5f, &throwaway)) -1.0f;
-            break;
-         default:
-            break;
+            default:
+               break;
+         }
+         // insert value in buffer
+         buffer[i] = BlendedAmplitude * f;
+         // update freq,amplitude
+         mPositionInCycles += BlendedFrequency;
+         BlendedFrequency += frequencyQuantum;
+         BlendedAmplitude += amplitudeQuantum;
       }
-      buffer[i] = BlendedAmplitude * f;
    }
+   // update external placeholder
    mSample += len;
    return true;
 }
 
+
+
 bool EffectToneGen::Process()
 {
    if (length <= 0.0)
@@ -211,7 +249,7 @@ bool EffectToneGen::Process()
       track->Clear(mT0, mT1);
       track->Paste(mT0, tmp);
       delete tmp;
-      
+
       //Iterate to the next track
       track = (WaveTrack *)iter.Next();
    }
@@ -232,6 +270,14 @@ bool EffectToneGen::Process()
 #define AMP_MIN 0
 #define AMP_MAX 1
 
+#define ID_TONE_DURATION_TEXT 10001
+
+BEGIN_EVENT_TABLE(ToneGenDialog, EffectDialog)
+    EVT_COMMAND(wxID_ANY, EVT_TIMETEXTCTRL_UPDATED, ToneGenDialog::OnTimeCtrlUpdate)
+    EVT_TEXT(ID_TONE_DURATION_TEXT, ToneGenDialog::OnToneGenDurationText)
+END_EVENT_TABLE()
+
+
 ToneGenDialog::ToneGenDialog(wxWindow * parent, const wxString & title)
 : EffectDialog(parent, title, INSERT_EFFECT)
 {
@@ -243,11 +289,22 @@ void ToneGenDialog::PopulateOrExchangeStandard( ShuttleGui & S )
 {
    S.StartMultiColumn(2, wxCENTER);
    {
-      S.TieChoice( _("Waveform:"), waveform,  waveforms);
+      S.TieChoice( _("Waveform") + wxString(wxT(":")), waveform,  waveforms);
       S.SetSizeHints(-1,-1);
-      S.TieTextBox( _("Frequency / Hz"),frequency[0], 5);
+      S.TieTextBox( _("Frequency (Hz)"),frequency[0], 5);
       S.TieTextBox( _("Amplitude (0-1)"),amplitude[0], 5);
-      S.TieTextBox( _("Length (seconds)"),length, 5 );
+      S.AddFixedText(_("Length"), false);
+      mToneDurationT = new
+      TimeTextCtrl(this,
+                   ID_TONE_DURATION_TEXT,
+                   TimeTextCtrl::GetBuiltinFormat(isSelection==true?(wxT("hh:mm:ss + samples")):(wxT("seconds"))),
+                   length,
+                   44100,
+                   wxDefaultPosition,
+                   wxDefaultSize,
+                   true);
+      S.AddWindow(mToneDurationT);
+      mToneDurationT->EnableMenu();
    }
    S.EndMultiColumn();
 }
@@ -274,7 +331,18 @@ void ToneGenDialog::PopulateOrExchangeExtended( ShuttleGui & S )
    S.EndMultiColumn();
    S.StartMultiColumn(2, wxCENTER);
    {
-      S.TieTextBox( _("Length (seconds)"),length, 10 );
+      S.AddFixedText(_("Length"), false);
+      mToneDurationT = new
+      TimeTextCtrl(this,
+                   ID_TONE_DURATION_TEXT,
+                   TimeTextCtrl::GetBuiltinFormat(isSelection==true?(wxT("hh:mm:ss + samples")):(wxT("seconds"))),
+                   length,
+                   44100,
+                   wxDefaultPosition,
+                   wxDefaultSize,
+                   true);
+      S.AddWindow(mToneDurationT);
+      mToneDurationT->EnableMenu();
    }
    S.EndMultiColumn();
 }
@@ -289,8 +357,15 @@ void ToneGenDialog::PopulateOrExchange(ShuttleGui & S)
 
 bool ToneGenDialog::TransferDataToWindow()
 {
-   ShuttleGui S( this, eIsSettingToDialog );
-   PopulateOrExchange( S );
+//  if you don't remove these, there will be a double
+//  timetextctrl in the dialog. Don't know why...
+//   ShuttleGui S( this, eIsSettingToDialog );
+//   PopulateOrExchange( S );
+
+   mToneDurationT->SetTimeValue(length);
+   mToneDurationT->SetFocus();
+   mToneDurationT->Fit();
+
    return true;
 }
 
@@ -298,6 +373,7 @@ bool ToneGenDialog::TransferDataFromWindow()
 {
    ShuttleGui S( this, eIsGettingFromDialog );
    PopulateOrExchange( S );
+
    amplitude[0] = TrapDouble(amplitude[0], AMP_MIN, AMP_MAX);
    frequency[0] = TrapDouble(frequency[0], FREQ_MIN, FREQ_MAX);
    amplitude[1] = TrapDouble(amplitude[1], AMP_MIN, AMP_MAX);
@@ -305,6 +381,14 @@ bool ToneGenDialog::TransferDataFromWindow()
    return true;
 }
 
+void ToneGenDialog::OnToneGenDurationText(wxCommandEvent & event) {
+   length = mToneDurationT->GetTimeValue();
+}
+
+void ToneGenDialog::OnTimeCtrlUpdate(wxCommandEvent & event) {
+   this->Fit();
+}
+
 // Indentation settings for Vim and Emacs and unique identifier for Arch, a
 // version control system. Please do not modify past this point.
 //
