@@ -176,8 +176,7 @@ bool VSTEffect::Init()
             GetSamples((WaveTrack *)right, &rstart, &rlen);
             
             if (llen != rlen || ((WaveTrack *)left)->GetRate() != ((WaveTrack *)right)->GetRate()) {
-               wxMessageBox(_("Sorry, VST Effects cannot be performed on stereo tracks where "
-                            "the individual channels of the track do not match."));
+               wxMessageBox(_("Sorry, VST Effects cannot be performed on stereo tracks where the individual channels of the track do not match."));
                return false;
             }
          }
@@ -449,7 +448,7 @@ VSTEffectDialog::VSTEffectDialog(wxWindow * parent,
                    vst->GetPreviewName(),
                    wxPoint(10, y), wxSize(80, 30));
    wxButton *cancel =
-       new wxButton(this, wxID_CANCEL, _("Cancel"), wxPoint(130, y), wxSize(80, 30));
+       new wxButton(this, wxID_CANCEL, _("&Cancel"), wxPoint(130, y), wxSize(80, 30));
    wxButton *ok =
        new wxButton(this, wxID_OK, _("OK"), wxPoint(220, y), wxSize(80, 30));
 
