@@ -35,6 +35,7 @@ effects from this one class.
 #include <wx/statbox.h>
 #include <wx/stattext.h>
 #include <wx/textctrl.h>
+#include <wx/tokenzr.h>
 #include <wx/intl.h>
 #include <wx/scrolwin.h>
 
@@ -45,7 +46,7 @@ effects from this one class.
 LadspaEffect::LadspaEffect(const LADSPA_Descriptor *data)
 {
    mData = data;
-   pluginName = LAT1CTOWX(data->Name);
+   pluginName = LAT1CTOWX(mData->Name);
 
    fInBuffer = NULL;
    fOutBuffer = NULL;
@@ -148,6 +149,21 @@ wxString LadspaEffect::GetEffectName()
       return pluginName;
 }
 
+wxString LadspaEffect::GetEffectIdentifier()
+{
+   wxStringTokenizer st(pluginName, wxT(" "));
+   wxString id;
+
+   // CamelCase the name
+   while (st.HasMoreTokens()) {
+      wxString tok = st.GetNextToken();
+
+      id += tok.Left(1).MakeUpper() + tok.Mid(1);
+   }
+
+   return id;
+}
+
 wxString LadspaEffect::GetEffectAction()
 {
    return wxString::Format(_("Performing Effect: %s"), 
@@ -170,9 +186,7 @@ bool LadspaEffect::Init()
          
          if (((WaveTrack *)left)->GetRate() !=
              ((WaveTrack *)right)->GetRate()) {
-            wxMessageBox(_("Sorry, Plug-in Effects cannot be performed \
-                            on stereo tracks where the individual \
-                            channels of the track do not match."));
+            wxMessageBox(_("Sorry, Plug-in Effects cannot be performed on stereo tracks where the individual channels of the track do not match."));
             return false;
          }
       }
@@ -576,6 +590,12 @@ LadspaEffectDialog::LadspaEffectDialog(LadspaEffect *eff,
                                               wxDefaultPosition,
                                               wxDefaultSize,
                                               wxVSCROLL | wxTAB_TRAVERSAL);
+
+   // Try to give the window a sensible default/minimum size
+   w->SetMinSize(wxSize(
+      wxMax(600, parent->GetSize().GetWidth() * 2/3),
+      parent->GetSize().GetHeight() / 2));
+                                              
    w->SetScrollRate(0, 20);
    vSizer->Add(w, 1, wxEXPAND|wxALL, 5);
 
