@@ -47,6 +47,7 @@ and ImportLOF.cpp.
 #include "ImportPCM.h"
 #include "ImportMP3.h"
 #include "ImportOGG.h"
+#include "ImportQT.h"
 #include "ImportRaw.h"
 #include "ImportLOF.h"
 #include "ImportFLAC.h"
@@ -68,6 +69,10 @@ Importer::Importer()
    GetFLACImportPlugin(mImportPluginList, mUnusableImportPluginList);
    GetMP3ImportPlugin(mImportPluginList, mUnusableImportPluginList);
    GetLOFImportPlugin(mImportPluginList, mUnusableImportPluginList);
+
+   #ifdef USE_QUICKTIME
+   GetQTImportPlugin(mImportPluginList, mUnusableImportPluginList);
+   #endif
 }
 
 Importer::~Importer()
@@ -200,7 +205,7 @@ int Importer::Import(wxString fName,
    //WMA files of various forms
    if ((extension.IsSameAs(wxT("wmv"), false))||(extension.IsSameAs(wxT("wma"), false))||(extension.IsSameAs(wxT("asf"), false))) {
       errorMessage = wxT("\"") + fName + wxT("\"") + 
-         _(" is an Windows Media Audio file. \nAudacity cannot open this type of file to due to patent restrictions.\nYou need to convert it to a supported audio format.");
+         _(" is a Windows Media Audio file. \nAudacity cannot open this type of file to due to patent restrictions.\nYou need to convert it to a supported audio format.");
       return 0;
    }
    //AAC files of various forms (probably not encrypted)
