@@ -96,7 +96,7 @@ void DirectoriesPrefs::PopulateOrExchange( ShuttleGui & S )
    }
    S.EndStatic();
 
-   S.StartStatic( _("Auto save"),1);
+   S.StartStatic( _("Auto save"),0);
    {
       S.TieCheckBox( _("Auto save a copy of the project in a separate folder"),
                      wxT("/Directories/AutoSaveEnabled"), true);
@@ -106,6 +106,13 @@ void DirectoriesPrefs::PopulateOrExchange( ShuttleGui & S )
       S.EndThreeColumn();
    }
    S.EndStatic();
+   
+   S.StartStatic( _("Audio cache"),0);
+   {
+      S.TieCheckBox( _("Always hold all audio data in memory"),
+                     wxT("/Directories/CacheBlockFiles"), false);
+   }
+   S.EndStatic();
 }
 
 void DirectoriesPrefs::OnChooseTempDir(wxCommandEvent &event)
