@@ -70,9 +70,9 @@ void GUIPrefs::PopulateOrExchange( ShuttleGui & S )
    S.StartStatic( _("Behaviors"),1 );
    {
       S.TieCheckBox( _("&Autoscroll while playing"),
-         wxT("/GUI/AutoScroll"), false);
+         wxT("/GUI/AutoScroll"), true);
       S.TieCheckBox( _("Al&ways allow pausing"),
-         wxT("/GUI/AlwaysEnablePause"), true);
+         wxT("/GUI/AlwaysEnablePause"), false);
       S.TieCheckBox( _("&Update spectrogram while playing"),
          wxT("/GUI/UpdateSpectrogram"), true);
       S.TieCheckBox( _("&Quit Audacity upon closing last window"),
@@ -87,6 +87,8 @@ void GUIPrefs::PopulateOrExchange( ShuttleGui & S )
          wxT("/GUI/CircularTrackNavigation"), false );
       S.TieCheckBox( _("&Present track number to accessibility aid"), 
          wxT("/GUI/PresentTrackNumber"), true );
+      S.TieCheckBox( _("Editing a &clip can move other clips"),
+         wxT("/GUI/EditClipCanMove"), true );
    }
    S.EndStatic();
    S.StartStatic( _("Show / Hide"),1 );
