@@ -114,7 +114,6 @@ PrefsDialog::PrefsDialog(wxWindow * parent):
    SetSize(rect);
 
    wxBoxSizer *topSizer = new wxBoxSizer(wxVERTICAL);
-
 // mCategories = new wxNotebook(this, -1, wxDefaultPosition, wxDefaultSize
 // mCategories = new wxChoicebook(this, -1, wxDefaultPosition, wxDefaultSize
 // mCategories = new wxListbook(this, -1, wxDefaultPosition, wxDefaultSize
@@ -161,13 +160,13 @@ PrefsDialog::PrefsDialog(wxWindow * parent):
 
    mCategories->SetSelection(selected);
 
+#if defined(__WXGTK__)
+   mCategories->GetTreeCtrl()->EnsureVisible(mCategories->GetTreeCtrl()->GetRootItem());
+#endif
+
    mOK = new wxButton(this,
                       wxID_OK, _("&OK"));
-
-#ifndef TARGET_CARBON
    mOK->SetDefault();
-   mOK->SetFocus();
-#endif
 
    mCancel = new wxButton(this,
                           wxID_CANCEL,
