@@ -79,7 +79,7 @@ END_EVENT_TABLE()
 
 //Standard constructor
 ControlToolBar::ControlToolBar()
-: ToolBar(ControlBarID, _("Control"))
+: ToolBar(ControlBarID, _("Control"), wxT("Control"))
 {
    mPaused = false;
    mSizer = NULL;
@@ -159,7 +159,7 @@ void ControlToolBar::MakeLoopImage()
    wxImage * down2      = OverlayImage(bmpRecoloredDownLarge,   bmpLoop, xoff + 1, yoff + 1);
    wxImage * disable2   = OverlayImage(bmpRecoloredUpLarge,     bmpLoopDisabled, xoff, yoff);
 
-   mPlay->SetAlternateImages(up2, hilite2, down2, disable2);
+   mPlay->SetAlternateImages(*up2, *hilite2, *down2, *disable2);
 
    delete up2;
    delete hilite2;
@@ -416,7 +416,8 @@ void ControlToolBar::SetRecord(bool down)
 
 void ControlToolBar::PlayPlayRegion(double t0, double t1,
                                     bool looped /* = false */,
-                                    bool cutpreview /* = false */)
+                                    bool cutpreview /* = false */,
+                                    TimeTrack *timetrack /* = NULL */)
 {
    if (gAudioIO->IsBusy()) {
       mPlay->PopUp();
@@ -425,7 +426,7 @@ void ControlToolBar::PlayPlayRegion(double t0, double t1,
    
    if (cutpreview && t0==t1)
       return; /* msmeyer: makes no sense */
-   
+      
    mStop->Enable();
    mRewind->Disable();
    mBatch->Disable();
@@ -493,13 +494,26 @@ void ControlToolBar::PlayPlayRegion(double t0, double t1,
             double tcp0 = t0-beforeLen;
             double tcp1 = (t1+afterLen) - (t1-t0);
             SetupCutPreviewTracks(tcp0, t0, t1, tcp1);
-            token = gAudioIO->StartStream(
-                mCutPreviewTracks->GetWaveTrackArray(false),
-                WaveTrackArray(), NULL, p->GetRate(), tcp0, tcp1, p, false,
-                t0, t1-t0);
+            if (mCutPreviewTracks)
+            {
+               token = gAudioIO->StartStream(
+                  mCutPreviewTracks->GetWaveTrackArray(false),
+                  WaveTrackArray(), NULL, p->GetRate(), tcp0, tcp1, p, false,
+                  t0, t1-t0);
+            } else
+            {
+               // Cannot create cut preview tracks, clean up and exit
+               SetPlay(false);
+               SetStop(false);
+               SetRecord(false);
+               return;
+            }
          } else {
+            if (!timetrack) {
+               timetrack = t->GetTimeTrack();
+            }
             token = gAudioIO->StartStream(t->GetWaveTrackArray(false),
-                                  WaveTrackArray(), t->GetTimeTrack(),
+                                  WaveTrackArray(), timetrack,
                                   p->GetRate(), t0, t1, p, looped);
          }
          if (token != 0) {
@@ -670,8 +684,7 @@ void ControlToolBar::OnRecord(wxCommandEvent &evt)
       size_t numProjects = gAudacityProjects.Count();
       bool tracks = (p && !p->GetTracks()->IsEmpty());
       if (tracks || (numProjects > 1)) {
-         wxMessageBox(_("CleanSpeech only allows recording mono track.\n"
-            wxT("Recording not possible when more than one window open.")),
+         wxMessageBox(_("CleanSpeech only allows recording mono tracks.\nRecording is not possible when more than one window is open."),
             _("Recording not permitted"),
             wxOK | wxICON_INFORMATION,
             this);
@@ -703,45 +716,96 @@ void ControlToolBar::OnRecord(wxCommandEvent &evt)
 
       bool duplex;
       gPrefs->Read(wxT("/AudioIO/Duplex"), &duplex, true);
-      int recordingChannels = gPrefs->Read(wxT("/AudioIO/RecordChannels"), 1);
-
       if( duplex )
          playbackTracks = t->GetWaveTrackArray(false);
       else
          playbackTracks = WaveTrackArray();
 
-      for( int c = 0; c < recordingChannels; c++ )
-      {
-         WaveTrack *newTrack = p->GetTrackFactory()->NewWaveTrack();
-         int initialheight = newTrack->GetHeight();
-         newTrack->SetOffset(t0);
-         newTrack->SetHeight(initialheight / recordingChannels);
-         if( recordingChannels == 2 )
+      // If SHIFT key was down, the user wants append to tracks
+      int recordingChannels = 0;
+      bool shifted = mRecord->WasShiftDown();
+      if (shifted) {
+         TrackListIterator it(t);
+         WaveTrack *wt;
+         bool sel = false;
+         double allt0 = t0;
+
+         // Find the maximum end time of selected and all wave tracks
+         for (Track *tt = it.First(); tt; tt = it.Next()) {
+            if (tt->GetKind() == Track::Wave) {
+               wt = (WaveTrack *)tt;
+               if (wt->GetEndTime() > allt0) {
+                  allt0 = wt->GetEndTime();
+               }
+            
+               if (tt->GetSelected()) {
+                  sel = true;
+                  if (wt->GetEndTime() > t0) {
+                     t0 = wt->GetEndTime();
+                  }
+               }
+            }
+         }
+
+         // Use end time of all wave tracks if none selected
+         if (!sel) {
+            t0 = allt0;
+         }
+
+         // Pad selected/all wave tracks to make them all the same length
+         for (Track *tt = it.First(); tt; tt = it.Next()) {
+            if (tt->GetKind() == Track::Wave && (tt->GetSelected() || !sel)) {
+               wt = (WaveTrack *)tt;
+               t1 = wt->GetEndTime();
+               if (t1 < t0) {
+                  WaveTrack *newTrack = p->GetTrackFactory()->NewWaveTrack();
+                  newTrack->InsertSilence(0.0, t0 - t1);
+                  newTrack->Flush();
+                  wt->Clear(t1, t0);
+                  wt->Paste(t1, newTrack);
+                  delete newTrack;
+               }
+               newRecordingTracks.Add(wt);
+            }
+         }
+
+         t1 = 1000000000.0;     // record for a long, long time (tens of years)
+      }
+      else {
+         recordingChannels = gPrefs->Read(wxT("/AudioIO/RecordChannels"), 1);
+         for( int c = 0; c < recordingChannels; c++ )
          {
-            if( c == 0 )
+            WaveTrack *newTrack = p->GetTrackFactory()->NewWaveTrack();
+            int initialheight = newTrack->GetHeight();
+            newTrack->SetOffset(t0);
+            newTrack->SetHeight(initialheight / recordingChannels);
+            if( recordingChannels == 2 )
             {
-               newTrack->SetChannel(Track::LeftChannel);
-               newTrack->SetLinked(true);
+               if( c == 0 )
+               {
+                  newTrack->SetChannel(Track::LeftChannel);
+                  newTrack->SetLinked(true);
+               }
+               else
+               {
+                  newTrack->SetChannel(Track::RightChannel);
+                  newTrack->SetTeamed(true);
+               }
             }
             else
             {
-               newTrack->SetChannel(Track::RightChannel);
-               newTrack->SetTeamed(true);
+               newTrack->SetChannel( Track::MonoChannel );
             }
-         }
-         else
-         {
-            newTrack->SetChannel( Track::MonoChannel );
-         }
 
-         newRecordingTracks.Add(newTrack);
+            newRecordingTracks.Add(newTrack);
+         }
+         
+         // msmeyer: StartStream calls a callback which triggers auto-save, so
+         // we add the tracks where recording is done into now. We remove them
+         // later if starting the stream fails
+         for (unsigned int i = 0; i < newRecordingTracks.GetCount(); i++)
+            t->Add(newRecordingTracks[i]);
       }
-      
-      // msmeyer: StartStream calls a callback which triggers auto-save, so
-      // we add the tracks where recording is done into now. We remove them
-      // later if starting the stream fails
-      for (unsigned int i = 0; i < newRecordingTracks.GetCount(); i++)
-         t->Add(newRecordingTracks[i]);
 
       int token = gAudioIO->StartStream(playbackTracks,
                                         newRecordingTracks, t->GetTimeTrack(),
@@ -756,10 +820,11 @@ void ControlToolBar::OnRecord(wxCommandEvent &evt)
       }
       else {
          // msmeyer: Delete recently added tracks if opening stream fails
-         for (unsigned int i = 0; i < newRecordingTracks.GetCount(); i++)
-         {
-            t->Remove(newRecordingTracks[i]);
-            delete newRecordingTracks[i];
+         if (!shifted) {
+            for (unsigned int i = 0; i < newRecordingTracks.GetCount(); i++) {
+               t->Remove(newRecordingTracks[i]);
+               delete newRecordingTracks[i];
+            }
          }
 
          // msmeyer: Show error message if stream could not be opened
