@@ -1,4 +1,4 @@
- /**********************************************************************
+/**********************************************************************
 
   Audacity: A Digital Audio Editor
 
@@ -133,7 +133,7 @@ class ToolFrame:public wxFrame
       Layout();
 
       // Inform toolbar of change
-      bar->SetDocked( false, true );
+      bar->SetDocked( NULL, true );
 
       // Make sure resizable floaters don't get any smaller than initial size
       if( bar->IsResizable() )
@@ -152,6 +152,18 @@ class ToolFrame:public wxFrame
       mManager->ProcessEvent( event );
    }
 
+   // 
+   // Handle toolbar updates
+   //
+   void OnToolBarUpdate( wxCommandEvent & event )
+   {
+      // Resize floater window to exactly contain toolbar
+      mBar->GetParent()->SetClientSize( mBar->GetMinSize() );
+
+      // Allow it to propagate to our parent
+      event.Skip();
+   }
+
    //
    // Handle frame paint events
    //
@@ -276,6 +288,7 @@ BEGIN_EVENT_TABLE( ToolFrame, wxFrame )
    EVT_PAINT( ToolFrame::OnPaint )
    EVT_MOUSE_EVENTS( ToolFrame::OnMotion )
    EVT_CLOSE( ToolFrame::OnClose )
+   EVT_COMMAND( wxID_ANY, EVT_TOOLBAR_UPDATED, ToolFrame::OnToolBarUpdate )
 END_EVENT_TABLE()
 
 IMPLEMENT_CLASS( ToolManager, wxEvtHandler );
@@ -371,8 +384,8 @@ ToolManager::ToolManager( AudacityProject *parent )
                      this );
 
    // Create the top and bottom docks
-   mTopDock = new ToolDock( this, parent, TopDockID );
-   mBotDock = new ToolDock( this, parent, BotDockID );
+   mTopDock = new ToolDock( this, mParent, TopDockID );
+   mBotDock = new ToolDock( this, mParent, BotDockID );
 
    // Create all of the toolbars
    mBars[ ToolsBarID ]         = new ToolsToolBar();
@@ -427,6 +440,62 @@ ToolManager::~ToolManager()
    delete mDown;
 }
 
+void ToolManager::Reset()
+{
+   int ndx;
+
+   // Disconnect all docked bars
+   for( ndx = 0; ndx < ToolBarCount; ndx++ )
+   {
+      wxWindow *parent;
+      ToolDock *dock;
+      ToolBar *bar = mBars[ ndx ];
+
+      // Disconnect the bar
+      if( bar->IsDocked() )
+      {
+         bar->GetDock()->Undock( bar );
+         parent = NULL;
+      }
+      else
+      {
+         parent = bar->GetParent();
+      }
+
+      if( ndx == SelectionBarID )
+      {
+         dock = mBotDock;
+
+         wxCommandEvent e;
+         bar->GetEventHandler()->ProcessEvent(e);
+      }
+      else
+      {
+         dock = mTopDock;
+         bar->ReCreateButtons();
+      }
+
+      bar->EnableDisableButtons();
+#if 0
+      if( bar->IsResizable() )
+      {
+         bar->SetSize(bar->GetBestFittingSize());
+      }
+#endif
+      dock->Dock( bar );
+
+      Expose( ndx, ndx ==  DeviceBarID ? false : true );
+
+      if( parent )
+      {
+         parent->Destroy();
+      }
+   }
+
+   LayoutToolBars();
+   Updated();
+} 
+
 //
 // Read the toolbar states
 //
@@ -494,7 +563,7 @@ void ToolManager::ReadConfig()
          }
 
          // Set the width
-         if( width[ ndx ] )
+         if( width[ ndx ] >= bar->GetSize().x )
          {
             wxSize sz( width[ ndx ], bar->GetSize().y );
             bar->SetSize( sz );
@@ -527,7 +596,7 @@ void ToolManager::ReadConfig()
          if( width[ ndx ] != -1 && height[ ndx ] != -1 )
          {
             wxSize sz( width[ ndx ], height[ ndx ] );
-            f->SetMinSize( sz );
+            f->SetSizeHints( sz );
             f->SetSize( sz );
             f->Layout();
          }
@@ -536,7 +605,7 @@ void ToolManager::ReadConfig()
          bar->Expose( show[ ndx ] );
 
          // Inform toolbar of change
-         bar->SetDocked( false, false );
+         bar->SetDocked( NULL, false );
       }
 
       // Change back to the bar root
@@ -731,6 +800,24 @@ void ToolManager::ShowHide( int type )
    }
 }
 
+//
+// Set the visible/hidden state of a toolbar
+//
+void ToolManager::Expose( int type, bool show )
+{
+   ToolBar *t = mBars[ type ];
+
+   // Handle docked and floaters differently
+   if( t->IsDocked() )
+   {
+      t->GetDock()->Expose( type, show );
+   }
+   else
+   {
+      t->Expose( show );
+   }
+}
+
 //
 // Ask both docks to (re)layout their bars
 //
@@ -791,7 +878,7 @@ void ToolManager::OnMouse( wxMouseEvent & event )
       else
       {
          // Calling SetDocked() to force the grabber button to popup
-         mDragBar->SetDocked( false, false );
+         mDragBar->SetDocked( NULL, false );
       }
 
       // Done dragging
@@ -999,7 +1086,7 @@ void ToolManager::OnGrabber( GrabberEvent & event )
       mp -= mDragOffset;
 
       // Inform toolbar of change
-      mDragBar->SetDocked( false, true );
+      mDragBar->SetDocked( NULL, true );
 
       // Construct a new floater
       mDragWindow = new ToolFrame( mParent, this, mDragBar, mp );
