@@ -166,6 +166,7 @@ different formats.
 #include <wx/sizer.h>
 #include <wx/stattext.h>
 #include <wx/tooltip.h>
+#include <wx/toplevel.h>
 
 #define ID_MENU 9800
 
@@ -320,6 +321,10 @@ TimeTextCtrl::TimeTextCtrl(wxWindow *parent,
 
 TimeTextCtrl::~TimeTextCtrl()
 {
+   wxCommandEvent e(EVT_RELEASE_KEYBOARD);
+   e.SetEventObject(this);
+   GetParent()->GetEventHandler()->ProcessEvent(e);
+
    if (mBackgroundBitmap)
       delete mBackgroundBitmap;
    if (mDigitFont)
@@ -805,6 +810,8 @@ void TimeTextCtrl::OnContext(wxContextMenuEvent &event)
       return;
    }
 
+   SetFocus();
+
    for(i=0; i<GetNumBuiltins(); i++) {
       menu.AppendCheckItem(ID_MENU+i, GetBuiltinName(i));
       if (mFormatString == GetBuiltinFormat(i))
@@ -839,8 +846,6 @@ void TimeTextCtrl::OnMouse(wxMouseEvent &event)
       Refresh(false);
    }
    else if (event.RightDown() && mMenuEnabled) {
-      SetFocus();
-
       wxContextMenuEvent e;
       OnContext(e);
    }
@@ -895,7 +900,7 @@ void TimeTextCtrl::OnCaptureKey(wxCommandEvent &event)
          return;
 
       default:
-         if (keyCode >= '0' || keyCode <= '9')
+         if (keyCode >= '0' && keyCode <= '9')
             return;
    }
 
@@ -980,8 +985,8 @@ void TimeTextCtrl::OnKeyDown(wxKeyEvent &event)
    } 
 
    else if (keyCode == WXK_RETURN || keyCode == WXK_NUMPAD_ENTER) {
-      wxWindow *parent = GetParent();
-      wxWindow *def = parent->GetDefaultItem();
+      wxTopLevelWindow *tlw = wxDynamicCast(wxGetTopLevelParent(this), wxTopLevelWindow);
+      wxWindow *def = tlw->GetDefaultItem();
       if (def && def->IsEnabled()) {
          wxCommandEvent cevent(wxEVT_COMMAND_BUTTON_CLICKED,
                                def->GetId());
